//---------------------------------------------------------------------
//---------------------------------------------------------------------
// This is a LED Control logic with PWM at 100khz.
// Vincent Huang, Galaxy TW
//
// History :
// 2015-12-14 Created
// 2016-02-15 Modify
//---------------------------------------------------------------------
`include "../top/es5654bq_syspld_defines.vh"
//---------------------------------------------------------------------
module led_display (
        // Clock signals
        input            MGMT_CLK,
        input            MGMT_RST_N,
        // PWM
        input   [ 7: 0]  LED_PWM_G,
        input   [ 7: 0]  LED_PWM_A,
        // 
        input            LED_LOC_A_CTRL,
        input            LED_DIAG_G_CTRL,
        input            LED_DIAG_A_CTRL,
        input            LED_FAN_G_CTRL,
        input            LED_FAN_A_CTRL,
        input            LED_DEBUG,
        input   [ 1: 0]  LED_PSU_G_CTRL,
        input   [ 1: 0]  LED_PSU_A_CTRL,
        input   [ 1: 0]  PS_AC_ALERT,
        input   [ 1: 0]  PS_PRSNT,
        input   [ 1: 0]  PS_12V_PG,
        input            FAN_INT_L,
        // 
        output  [ 1: 0]  LED_PSU_G,
        output  [ 1: 0]  LED_PSU_A,
        output           LED_DIAG_G,
        output           LED_DIAG_A,
        output           LED_FAN_G,
        output           LED_FAN_A,
        output           LED_LOC_A
      );

	// Parameter for this design
	localparam   LED_ON  = 1'b0;
	localparam   LED_OFF = 1'b1;

	// signals for this design
	wire	     clock;
	wire	     reset_n;
	wire [1 : 0] led_psu_pwm_g;
	wire [1 : 0] led_psu_pwm_a;
	wire         led_diag_pwm_g;
	wire         led_diag_pwm_a;
	wire    	 led_fan_pwm_g;
	wire    	 led_fan_pwm_a;
	wire         led_loc_pwm_g;
	wire         led_loc_pwm_a;

    // Buffers
    assign clock   = MGMT_CLK;
    assign reset_n = MGMT_RST_N;


    // PSU LED statement
    led_pwm LED_PSU_G_1 ( pwm_cntr, LED_PWM_G, led_psu_pwm_g[0] );
    led_pwm LED_PSU_A_1 ( pwm_cntr, LED_PWM_A, led_psu_pwm_a[0] );
    led_pwm LED_PSU_G_2 ( pwm_cntr, LED_PWM_G, led_psu_pwm_g[1] );
    led_pwm LED_PSU_A_2 ( pwm_cntr, LED_PWM_A, led_psu_pwm_a[1] );

	// PSU LED-G
    assign LED_PSU_G[0] = LED_DEBUG ? led_psu_pwm_g[0] : ( (PS_PRSNT[0] || (!(PS_12V_PG[0] && PS_AC_ALERT[0]))) ? LED_OFF : LED_ON );
    assign LED_PSU_G[1] = LED_DEBUG ? led_psu_pwm_g[1] : ( (PS_PRSNT[1] || (!(PS_12V_PG[1] && PS_AC_ALERT[1]))) ? LED_OFF : LED_ON );
	// PSU LED-A
    assign LED_PSU_A[0] = LED_DEBUG ? led_psu_pwm_a[0] : ( (PS_PRSNT[0] || ( (PS_12V_PG[0] && PS_AC_ALERT[0]))) ? LED_OFF : LED_ON );
    assign LED_PSU_A[1] = LED_DEBUG ? led_psu_pwm_a[1] : ( (PS_PRSNT[1] || ( (PS_12V_PG[1] && PS_AC_ALERT[1]))) ? LED_OFF : LED_ON );


    // Diag. LED statement
    led_pwm LED_DIAG_G_I ( pwm_cntr, LED_PWM_G, led_diag_pwm_g );
    led_pwm LED_DIAG_A_I ( pwm_cntr, LED_PWM_A, led_diag_pwm_a );

    assign LED_DIAG_G = LED_DEBUG ? led_diag_pwm_g : ( (!LED_DIAG_G_CTRL) ? LED_ON : LED_OFF );
	assign LED_DIAG_A = LED_DEBUG ? led_diag_pwm_a : ( (!LED_DIAG_A_CTRL) ? LED_ON : LED_OFF );


    // Fan LED statement
    led_pwm LED_FAN_G_I ( pwm_cntr, LED_PWM_G, led_fan_pwm_g );
    led_pwm LED_FAN_A_I ( pwm_cntr, LED_PWM_A, led_fan_pwm_a );

    assign LED_FAN_G = LED_DEBUG ? led_fan_pwm_g : ( ( FAN_INT_L) ? LED_ON : LED_OFF );
	assign LED_FAN_A = LED_DEBUG ? led_fan_pwm_a : ( (!FAN_INT_L) ? LED_ON : LED_OFF );


    // Loc LED statement
    led_pwm LED_LOC_A_I ( pwm_cntr, LED_PWM_A, led_loc_pwm_a );

	assign LED_LOC_A = LED_DEBUG ? led_loc_pwm_a : ( (!LED_LOC_A_CTRL) ? LED_ON : LED_OFF );


    /************** LED Duty Control Logic ****************/

    /* The idea here is to PWM the LED display
     signals at a frequency of 100khz
     with a 8-bit width value. */
    reg [7:0] pwm_cntr;

    always @(posedge clock, negedge reset_n)
    begin
      if (!reset_n)
        pwm_cntr <= 8'h00;
      else
        pwm_cntr <= pwm_cntr + 8'd1;
    end

endmodule
