//---------------------------------------------------------------------
//---------------------------------------------------------------------
// MAC LED Encode for Project ES5654BQ CPLD1
// Vincent Huang, Galaxy TW
//
// History :
// 2016-01-26 Created
// 2016-02-15 Modify
//---------------------------------------------------------------------
module mac_led_encode (
        // Clock signals
        input            MGMT_CLK,
        input            MGMT_RST_N,
        // LED  Stream
        input            LED_CLK,
        input            LED_DATA,
        // SFP+ Port 1 ~ Port 24
        output  [23: 0]  MAC_LED_SFP_A,
        output  [23: 0]  MAC_LED_SFP_G,
		  output  [83: 0]  LED_DAT1_STREAM
      );

    // Signals for LED stream
    reg  [83: 0]  mac_led_parallel_data_sfp;																		// 20160503
	 assign LED_DAT1_STREAM = mac_led_parallel_data_sfp;                                        // Read Stream from MAC LED_DATA1

    // Clock Signals
    wire reset_n = MGMT_RST_N;

    // LED Data Mapping for SFP+ 1 ~ 4, 20160503	 
    generate 
      genvar  sfp_port_num_1;
      for ( sfp_port_num_1=0; sfp_port_num_1<4; sfp_port_num_1=sfp_port_num_1+1 )
		begin : MAC_LED_BIT_MAPPING_SFP_1
			 assign MAC_LED_SFP_G[sfp_port_num_1] = ( mac_led_parallel_data_sfp[((sfp_port_num_1*3)+3):((sfp_port_num_1*3)+2)] == 2'b00 )? (! mac_led_parallel_data_sfp[(sfp_port_num_1*3)+4]) : 1'b1;	   // SFP+ 10G
			 assign MAC_LED_SFP_A[sfp_port_num_1] = ( mac_led_parallel_data_sfp[((sfp_port_num_1*3)+3):((sfp_port_num_1*3)+2)] == 2'b10 )? (! mac_led_parallel_data_sfp[(sfp_port_num_1*3)+4]) : 1'b1;		// SFP+ 1G OFF
		end
    endgenerate
	 // LED Data Mapping for SFP+ 5 ~ 8, 20160503
    generate 
      genvar  sfp_port_num_2;
      for ( sfp_port_num_2=4; sfp_port_num_2<8; sfp_port_num_2=sfp_port_num_2+1 )
      begin : MAC_LED_BIT_MAPPING_SFP_2
			 assign MAC_LED_SFP_G[sfp_port_num_2] = ( mac_led_parallel_data_sfp[((sfp_port_num_2*3)+5):((sfp_port_num_2*3)+4)] == 2'b00 )? (! mac_led_parallel_data_sfp[(sfp_port_num_2*3)+6]) : 1'b1;	   // SFP+ 10G
			 assign MAC_LED_SFP_A[sfp_port_num_2] = ( mac_led_parallel_data_sfp[((sfp_port_num_2*3)+5):((sfp_port_num_2*3)+4)] == 2'b10 )? (! mac_led_parallel_data_sfp[(sfp_port_num_2*3)+6]) : 1'b1;		// SFP+ 1G OFF
		end
    endgenerate
	 // LED Data Mapping for SFP+ 9 ~ 12, 20160503
    generate 
      genvar  sfp_port_num_3;
      for ( sfp_port_num_3=8; sfp_port_num_3<12; sfp_port_num_3=sfp_port_num_3+1 )
      begin : MAC_LED_BIT_MAPPING_SFP_3
          assign MAC_LED_SFP_G[sfp_port_num_3] = ( mac_led_parallel_data_sfp[((sfp_port_num_3*3)+7):((sfp_port_num_3*3)+6)] == 2'b00 )? (! mac_led_parallel_data_sfp[(sfp_port_num_3*3)+8]) : 1'b1;	   // SFP+ 10G
			 assign MAC_LED_SFP_A[sfp_port_num_3] = ( mac_led_parallel_data_sfp[((sfp_port_num_3*3)+7):((sfp_port_num_3*3)+6)] == 2'b10 )? (! mac_led_parallel_data_sfp[(sfp_port_num_3*3)+8]) : 1'b1;		// SFP+ 1G OFF
		end    
    endgenerate
	 // LED Data Mapping for SFP+ 13 ~ 16, 20160503
    generate 
      genvar  sfp_port_num_4;
      for ( sfp_port_num_4=12; sfp_port_num_4<16; sfp_port_num_4=sfp_port_num_4+1 )
      begin : MAC_LED_BIT_MAPPING_SFP_4
			 assign MAC_LED_SFP_G[sfp_port_num_4] = ( mac_led_parallel_data_sfp[((sfp_port_num_4*3)+9):((sfp_port_num_4*3)+8)] == 2'b00 )? (! mac_led_parallel_data_sfp[(sfp_port_num_4*3)+10]) : 1'b1;	   // SFP+ 10G
			 assign MAC_LED_SFP_A[sfp_port_num_4] = ( mac_led_parallel_data_sfp[((sfp_port_num_4*3)+9):((sfp_port_num_4*3)+8)] == 2'b10 )? (! mac_led_parallel_data_sfp[(sfp_port_num_4*3)+10]) : 1'b1;		// SFP+ 1G OFF
		end  
    endgenerate
	 // LED Data Mapping for SFP+ 17 ~ 20, 20160503
    generate 
      genvar  sfp_port_num_5;
      for ( sfp_port_num_5=16; sfp_port_num_5<20; sfp_port_num_5=sfp_port_num_5+1 )
      begin : MAC_LED_BIT_MAPPING_SFP_5
			 assign MAC_LED_SFP_G[sfp_port_num_5] = ( mac_led_parallel_data_sfp[((sfp_port_num_5*3)+11):((sfp_port_num_5*3)+10)] == 2'b00 )? (! mac_led_parallel_data_sfp[(sfp_port_num_5*3)+12]) : 1'b1;	   // SFP+ 10G
			 assign MAC_LED_SFP_A[sfp_port_num_5] = ( mac_led_parallel_data_sfp[((sfp_port_num_5*3)+11):((sfp_port_num_5*3)+10)] == 2'b10 )? (! mac_led_parallel_data_sfp[(sfp_port_num_5*3)+12]) : 1'b1;		// SFP+ 1G OFF
		end   
    endgenerate
	 // LED Data Mapping for SFP+ 21 ~ 24, 20160503
    generate 
      genvar  sfp_port_num_6;
      for ( sfp_port_num_6=20; sfp_port_num_6<24; sfp_port_num_6=sfp_port_num_6+1 )
      begin : MAC_LED_BIT_MAPPING_SFP_6
			 assign MAC_LED_SFP_G[sfp_port_num_6] = ( mac_led_parallel_data_sfp[((sfp_port_num_6*3)+13):((sfp_port_num_6*3)+12)] == 2'b00 )? (! mac_led_parallel_data_sfp[(sfp_port_num_6*3)+14]) : 1'b1;	   // SFP+ 10G
			 assign MAC_LED_SFP_A[sfp_port_num_6] = ( mac_led_parallel_data_sfp[((sfp_port_num_6*3)+13):((sfp_port_num_6*3)+12)] == 2'b10 )? (! mac_led_parallel_data_sfp[(sfp_port_num_6*3)+14]) : 1'b1;		// SFP+ 1G OFF
		end   
    endgenerate

    // MAC LED Stream to Parallel output
    always @( posedge LED_CLK or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        mac_led_parallel_data_sfp <= 84'hF_FFFF_FFFF_FFFF_FFFF_FFFF;										// 20160503
      else
        mac_led_parallel_data_sfp <= { LED_DATA, mac_led_parallel_data_sfp[83: 1] };				// 20160503
		  //84'b01001001001000_01001001001000_01001001001000_01001001001000_01001001001000_01001001001000;    Test stream
    end

endmodule    
