//---------------------------------------------------------------------
//---------------------------------------------------------------------
// RTL Top File for Model ES5654BQ CPLD1
// Vincent Huang, Galaxy TW
//
// History :
// 2015-12-14 Created
// 2016-02-15 Modify
//---------------------------------------------------------------------
`include "./es5654bq_syspld_defines.vh"
`define I2C_S_DEVADDR 7'h60
//---------------------------------------------------------------------
module es5654bq_syspld_top (
        // Clock signals
        input            CPLD1_25MHz,
        output           CPLD12_SYNC,
        input            CPLD2_MOD_ABS,
        input            CPLD2_QSFP_INT,
        output           CPLD2_QSFP_RST,
        input            CPLD21_SYNC,
        // Reset
        input            RESET_BUTTON_RST,
        input            POWER_RST_N,
        output           MANU_RST,
        input            RESET_SYS_CPLD,
        output           RESET_MAC,
        output           CPLD_RESET,
        output           MAC_RESET_N,
        output           MGMT_PHY_RST_N,
        output           IDT8V89307_RESET,
        output  [ 6: 0]  PCA9548_7x_RESET,
        output           PCA9548_1_RESET,
        output           PCA9548_5_RESET,
        output           PCA9548_RESET,
        // Interrupt
        input            CPU_JTAG_RST,       // Reserved
        input            INT_MGMT_PHY_N,
        input            IDT8V89307_INT_REQ,
        input            FAN_INT_L,
        input            MAC_CPLD_INT_L,
        input            CPLD2_INT,
        input   [ 2: 0]  LM75BD_INT,
        output           CPLD23_INT_CPU,
        output           SYS_CPLD_INT_CPU,
        output           MAC_INT_N,          // Reserved
        // LED Stream
        input            LED_CLK,
        input            LED_DATA,
        // SFP+ Port 1 ~ Port 24
        output  [23: 0]  LED_SFP_A,
        output  [23: 0]  LED_SFP_G,
        // LED display
        output  [ 1: 0]  LED_PSU_A,
        output  [ 1: 0]  LED_PSU_G,
        output           LED_DIAG_A,
        output           LED_DIAG_G,
        output           LED_FAN_A,
        output           LED_FAN_G,
        output           LED_LOC_A,
        // I2C Bus
        input            I2C_0_SCL,
        inout            I2C_0_SDA,
        inout            FAN_SCL_1,
        inout            FAN_SDA_1,
        inout            FAN_SCL_2,
        inout            FAN_SDA_2,
        inout            LM75BD_SCL,
        inout            LM75BD_SDA,
        // PSU siganls
        input   [ 1: 0]  PS_AC_ALERT,
        input   [ 1: 0]  PS_PRSNT,
        input   [ 1: 0]  PS_12V_PG,
        output  [ 1: 0]  PS_PDB_ALERT,
        output  [ 1: 0]  PS_ON,
        // USB signals,
        input            USB1_VBUS,
        input            USB_PWRFLT_N,
        output           USB_RESET_N,
        output           USB_PWRON_N,
        output           USB1_PWRFAULT,
        // XCVR signals
        input   [23: 0]  XSFP_MOD_ABS,
        input   [23: 0]  XSFP_RXLOS,
        output  [23: 0]  XSFP_TXDIS,
        input   [23: 0]  XSFP_TXFLT,
        // Misc
        input   [ 2: 0]  PCB_VER,
        input   [ 2: 0]  BOARD_ID,
        input            ID_BUTTON,
        input            CPU_PROCHOT_R,
        input            CPU_THERMALTRIP_R,
        input            L1_RCVRD_CLK_VALID_BKUP,
        input            L1_RCVRD_CLK_VALID,
        input            ONE_PPS_CLK,
        output           ONE_PPS_GPIO0,
        output           ONE_PPS_CPU,
        input            UCD9090_ALERT_L,    // TBD
        output           EE_WP,
        output           FAN_IDLE,           // TBD
        output           SW1_SELECT,
        output           SHUTDOWN_WAKE,
        output           CPLD1_TCAM_CRST_3V3_L,
        output           CPLD1_TCAM_PERST_3V3_L,
        output           CPLD1_TCAM_SRST_3V3_L,
        input   [ 1: 0]  TCAM_CPLD1_GIO_3V3_L
      );
    
    // signals for I2C Slave Instance
    wire           host_i2c_scl_in_sig;
    wire           host_i2c_sda_in_sig;
    wire           i2c_s_sda_sig_n;
    wire           i2c_read_sig;
    wire           i2c_read_done_sig;
    wire           i2c_write_sig;
    wire  [ 7: 0]  i2c_offset_addr_sig;
    wire  [ 7: 0]  i2c_offset_rdata_sig;
    wire  [ 7: 0]  i2c_offset_wdata_sig;
    // signals for Regfile Instance
    wire  [23: 0]  reg_led_sfp_a_sig;
    wire  [23: 0]  reg_led_sfp_g_sig;
    // signals for mac_led_encode Instance
    wire  [23: 0]  mac_led_sfp_a_sig;
    wire  [23: 0]  mac_led_sfp_g_sig;
    // Signals for LED display
    wire           led_loc_a_sig;
    wire           led_diag_g_sig;
    wire           led_diag_a_sig;
    wire           led_fan_g_sig;
    wire           led_fan_a_sig;
    wire           led_debug_sig;
    wire  [ 1: 0]  led_psu_g_sig_2b;
    wire  [ 1: 0]  led_psu_a_sig_2b;
    wire  [ 7: 0]  r_led_pwm_g_8b;
    wire  [ 7: 0]  r_led_pwm_a_8b;
	 wire  [83: 0]  led_dat1_sig;
    // Signals for Reset
    wire           manu_rst_sig;

    // Top
    wire   clk25               = CPLD1_25MHz;
    wire   global_rst_n        = RESET_SYS_CPLD | RESET_MAC;
    // Buffer for SFP+ Port 1 ~ Port 23 signals
    assign LED_SFP_A           = reg_led_sfp_a_sig & mac_led_sfp_a_sig;
    assign LED_SFP_G           = reg_led_sfp_g_sig & mac_led_sfp_g_sig;
    // Buffer for Reset signals
    assign MANU_RST            = RESET_BUTTON_RST & POWER_RST_N & manu_rst_sig;
    // Buffer for I2C signals
    assign host_i2c_scl_in_sig = I2C_0_SCL;
    assign host_i2c_sda_in_sig = I2C_0_SDA;
    assign I2C_0_SDA           = ( !i2c_s_sda_sig_n ) ? 1'b0 : 1'bZ;
    assign MAC_INT_N           = 1'bz;
    assign FAN_IDLE            = 1'bz;
    assign FAN_SCL_1           = 1'bz;
    assign FAN_SDA_1           = 1'bz;
    assign FAN_SCL_2           = 1'bz;
    assign FAN_SDA_2           = 1'bz;
    assign LM75BD_SCL          = 1'bz;
    assign LM75BD_SDA          = 1'bz;


    //-----------------------------------------------------------------
    // Call a Instance of I2C Slave
    //-----------------------------------------------------------------
    i2c_s #(
        .I2C_DevADDR      ( `I2C_S_DEVADDR         ),
        .Clock_Frequency  ( 50                     )
        ) I2C_SLAVE_I     (
        .clk              ( clk25                  ),
        .reset_n          ( global_rst_n           ),
        .soft_reset_n     ( 1'b1                   ),
        .scl_i            ( host_i2c_scl_in_sig    ),
        .sda_i            ( host_i2c_sda_in_sig    ),
        .sda_oe_n         ( i2c_s_sda_sig_n        ),
        .clk_stretch_n    ( 1'b1                   ),
        .readdata         ( i2c_offset_rdata_sig   ),
        .writedata        ( i2c_offset_wdata_sig   ),
        .address          ( i2c_offset_addr_sig    ),
        .read             ( i2c_read_sig           ),
        .read_done        ( i2c_read_done_sig      ),
        .write            ( i2c_write_sig          )
      );


    //-----------------------------------------------------------------
    // Call a Instance of MAC LED Encode
    //-----------------------------------------------------------------
    mac_led_encode MAC_LED_ENCODE_I (
        // Clock signals
        .MGMT_CLK       ( clk25             ),
        .MGMT_RST_N     ( global_rst_n      ),
        // LED  Stream
        .LED_CLK        ( LED_CLK           ),
        .LED_DATA       ( LED_DATA          ),
        // SFP+ Port 0 ~ Port 23
        .MAC_LED_SFP_A  ( mac_led_sfp_a_sig ),
        .MAC_LED_SFP_G  ( mac_led_sfp_g_sig ),
		  .LED_DAT1_STREAM( led_dat1_sig      )
      );


    //-----------------------------------------------------------------
    // Call a Instance of LED Display
    //-----------------------------------------------------------------
    led_display LED_I (
        // Clock signals
        .MGMT_CLK           ( clk25                       ),
        .MGMT_RST_N         ( global_rst_n                ),
        // PWM
        .LED_PWM_G          ( r_led_pwm_g_8b              ),
        .LED_PWM_A          ( r_led_pwm_a_8b              ),
        // 
        .LED_LOC_A_CTRL     ( led_loc_a_sig               ),
        .LED_DIAG_G_CTRL    ( led_diag_g_sig              ),
        .LED_DIAG_A_CTRL    ( led_diag_a_sig              ),
        .LED_FAN_G_CTRL     ( led_fan_g_sig               ),
        .LED_FAN_A_CTRL     ( led_fan_a_sig               ),
        .LED_DEBUG          ( led_debug_sig               ),
        .LED_PSU_G_CTRL     ( led_psu_g_sig_2b            ),
        .LED_PSU_A_CTRL     ( led_psu_a_sig_2b            ),
        .PS_AC_ALERT        ( PS_AC_ALERT                 ),
        .PS_PRSNT           ( PS_PRSNT                    ),
        .PS_12V_PG          ( PS_12V_PG                   ),
        .FAN_INT_L          ( FAN_INT_L                   ),
        // 
        .LED_PSU_G          ( LED_PSU_G                   ),
        .LED_PSU_A          ( LED_PSU_A                   ),
        .LED_DIAG_G         ( LED_DIAG_G                  ),
        .LED_DIAG_A         ( LED_DIAG_A                  ),
        .LED_FAN_G          ( LED_FAN_G                   ),
        .LED_FAN_A          ( LED_FAN_A                   ),
        .LED_LOC_A          ( LED_LOC_A                   )
      );

    //-----------------------------------------------------------------
    // Call a Instance of Regfiles
    //-----------------------------------------------------------------
    regfile REGFILES_I (
        // Clock signals
        .MGMT_CLK                   ( clk25                       ),
        .MGMT_RST_N                 ( global_rst_n                ),
        .INT_MGMT_PHY_N             ( INT_MGMT_PHY_N              ),
        .SYS_CPLD_INT_CPU           ( SYS_CPLD_INT_CPU            ),
        .CPLD23_INT_CPU             ( CPLD23_INT_CPU              ),
        // I2C signals
        .I2C_READ                   ( i2c_read_sig                ),
        .I2C_READ_DONE              ( i2c_read_done_sig           ),
        .I2C_WRITE                  ( i2c_write_sig               ),
        .I2C_OFFSET_ADDR            ( i2c_offset_addr_sig         ),
        .I2C_OFFSET_WDATA           ( i2c_offset_wdata_sig        ),
        .I2C_OFFSET_RDATA           ( i2c_offset_rdata_sig        ),
        // Register 0x00  --  Read
        .BOARD_ID                   ( BOARD_ID                    ),
        .PCB_VER                    ( PCB_VER                     ),
        // Register 0x02  --  Read
        .PS_AC_ALERT                ( PS_AC_ALERT                 ),
        .PS_12V_PG                  ( PS_12V_PG                   ),
        .PS_PRSNT                   ( PS_PRSNT                    ),
        // XCVR signals
        // Register 0x04 ~ 0x12  --  Read
        .XSFP_MOD_ABS               ( XSFP_MOD_ABS                ),
        // Register 0x14 ~ 0x19  --  Read
        .XSFP_TXFLT                 ( XSFP_TXFLT                  ),
        // Register 0x20 ~ 0x25  --  Read & Write
        .XSFP_TXDIS                 ( XSFP_TXDIS                  ),
        // Register 0x30 ~ 0x35  --  Read
        .XSFP_RXLOS                 ( XSFP_RXLOS                  ),
        // Register 0x40 ~ 0x45  --  Read & Write
        .REG_LED_SFP_A              ( reg_led_sfp_a_sig           ),
        .REG_LED_SFP_G              ( reg_led_sfp_g_sig           ),
        // Register 0x50  --  Read & Write
        .MAC_RESET_N                ( MAC_RESET_N                 ),
        .MANU_RST                   ( manu_rst_sig                ),
        .USB_RESET_N                ( USB_RESET_N                 ),
        .CPLD2_QSFP_RST             ( CPLD2_QSFP_RST              ),
        .CPLD_RESET                 ( CPLD_RESET                  ),
        .IDT8V89307_RESET           ( IDT8V89307_RESET            ),
        .MGMT_PHY_RST_N             ( MGMT_PHY_RST_N              ),
        // Register 0x51  --  Read & Write
        .CPLD1_TCAM_CRST_L          ( CPLD1_TCAM_CRST_3V3_L       ),
        .CPLD1_TCAM_PERST_L         ( CPLD1_TCAM_PERST_3V3_L      ),
        .CPLD1_TCAM_SRST_L          ( CPLD1_TCAM_SRST_3V3_L       ),
        .RESET_MAC                  ( RESET_MAC                   ),
        .RESET_SYS_CPLD             ( RESET_SYS_CPLD              ),
        .CPU_JTAG_RST               ( CPU_JTAG_RST                ),
        // Register 0x52 -- Read & Write
        .PCA9548_5_RESET            ( PCA9548_5_RESET             ),
        .PCA9548_1_RESET            ( PCA9548_1_RESET             ),
        .PCA9548_RESET              ( PCA9548_RESET               ),
        // Register 0x53 -- Read & Write
        .PCA9548_7x_RESET           ( PCA9548_7x_RESET            ),
        // Register 0x60 -- Read
        .IDT8V89307_INT_REQ         ( IDT8V89307_INT_REQ          ),
        .FAN_INT_L                  ( FAN_INT_L                   ),
        .MAC_CPLD_INT_L             ( MAC_CPLD_INT_L              ),
        .CPLD2_INT                  ( CPLD2_INT                   ),
        .LM75BD_INT                 ( LM75BD_INT                  ),
        // Register 0x62 -- Read
        .CPLD2_MOD_ABS              ( CPLD2_MOD_ABS               ),
        .CPLD2_QSFP_INT             ( CPLD2_QSFP_INT              ),
        .TCAM_CPLD1_GIO_L           ( TCAM_CPLD1_GIO_3V3_L        ),
        // Register 0x64 -- Read & Write
        .CPU_THERMALTRIP_R          ( CPU_THERMALTRIP_R           ),
        .CPU_PROCHOT_R              ( CPU_PROCHOT_R               ),
        .L1_RCVRD_CLK_VALID_BKUP    ( L1_RCVRD_CLK_VALID_BKUP     ),
        .L1_RCVRD_CLK_VALID         ( L1_RCVRD_CLK_VALID          ),
        .ONE_PPS_CLK                ( ONE_PPS_CLK                 ),
        .ONE_PPS_GPIO0              ( ONE_PPS_GPIO0               ),
        .ONE_PPS_CPU                ( ONE_PPS_CPU                 ),
        // Register 0x65 -- Read & Write
        .LED_LOC_A                  ( led_loc_a_sig               ),
        .LED_DIAG_G                 ( led_diag_g_sig              ),
        .LED_DIAG_A                 ( led_diag_a_sig              ),
        .LED_FAN_G                  ( led_fan_g_sig               ),
        .LED_FAN_A                  ( led_fan_a_sig               ),
        // Register 0x66 -- Read & Write
        .LED_DEBUG_MODE             ( led_debug_sig               ),
        .LED_PSU_G                  ( led_psu_g_sig_2b            ),
        .LED_PSU_A                  ( led_psu_a_sig_2b            ),
        // Register 0x67 -- Read & Write
        .USB_PWRON_N                ( USB_PWRON_N                 ),
        .USB1_VBUS                  ( USB1_VBUS                   ),
        .USB1_PWRFAULT              ( USB1_PWRFAULT               ),
        .USB_PWRFLT_N               ( USB_PWRFLT_N                ),
        // Register 0x68 -- Read & Write
        .SHUTDOWN_WAKE              ( SHUTDOWN_WAKE               ),
        .ID_BUTTON                  ( ID_BUTTON                   ),
        .PS_PDB_ALERT               ( PS_PDB_ALERT                ),
        .SW1_SELECT                 ( SW1_SELECT                  ),
        .EE_WP                      ( EE_WP                       ),
        .PS_ON                      ( PS_ON                       ),
        // Register 0x69 -- Read & Write
        .CPLD12_SYNC                (                             ),  // Reserved
        .CPLD21_SYNC                (                             ),  // Reserved
        // Register 0x70 -- Read & Write
        .LED_PWM_G                  ( r_led_pwm_g_8b              ),
        // Register 0x71 -- Read & Write
        .LED_PWM_A                  ( r_led_pwm_a_8b              ),
		  // Register 0x81~ -- Read
		  .LED1_STREAM                ( led_dat1_sig                )
      );


endmodule
