//---------------------------------------------------------------------
//---------------------------------------------------------------------
// Offser Register Files for ES5654BQ CPLD1
// Vincent Huang, Galaxy TW
//
// History :
// 2016-01-26 Created
// 2016-02-15 Modify
//---------------------------------------------------------------------
`include "./es5654bq_syspld_defines.vh"
//---------------------------------------------------------------------
module regfile (
        // Clock signals
        input            MGMT_CLK,
        input            MGMT_RST_N,
        input            INT_MGMT_PHY_N,
        output           SYS_CPLD_INT_CPU,
        output           CPLD23_INT_CPU,
        // I2C Signal
        input            I2C_READ,
        input            I2C_READ_DONE,
        input            I2C_WRITE,
        input   [ 7: 0]  I2C_OFFSET_ADDR,
        input   [ 7: 0]  I2C_OFFSET_WDATA,
        output  [ 7: 0]  I2C_OFFSET_RDATA,
        // Register 0x 00  --  Read
        input   [ 2: 0]  BOARD_ID,
        input   [ 2: 0]  PCB_VER,
        // Register 0x 02  --  Read
        input   [ 1: 0]  PS_AC_ALERT,
        input   [ 1: 0]  PS_12V_PG,
        input   [ 1: 0]  PS_PRSNT,
        // XCVR signals
        // Register 0x04 ~ 0x12  --  Read
        input   [23: 0]  XSFP_MOD_ABS,
        // Register 0x14 ~ 0x19  --  Read
        input   [23: 0]  XSFP_TXFLT,
        // Register 0x20 ~ 0x25  --  Read & Write
        output  [23: 0]  XSFP_TXDIS,
        // Register 0x30 ~ 0x35  --  Read
        input   [23: 0]  XSFP_RXLOS,
        // Register 0x40 ~ 0x45  --  Read & Write
        output  [23: 0]  REG_LED_SFP_A,
        output  [23: 0]  REG_LED_SFP_G,
        // Register 0x50  --  Read & Write
        output           MAC_RESET_N,
        output           MANU_RST,
        output           USB_RESET_N,
        output           CPLD2_QSFP_RST,
        output           CPLD_RESET,
        output           IDT8V89307_RESET,
        output           MGMT_PHY_RST_N,
        // Register 0x51  --  Read & Write
        output           CPLD1_TCAM_CRST_L,
        output           CPLD1_TCAM_PERST_L,
        output           CPLD1_TCAM_SRST_L,
        output           RESET_MAC,
        input            RESET_SYS_CPLD,
        input            CPU_JTAG_RST,
        // Register 0x52 -- Read & Write
        output           PCA9548_5_RESET,
        output           PCA9548_1_RESET,
        output           PCA9548_RESET,
        // Register 0x53 -- Read & Write
        output  [ 6: 0]  PCA9548_7x_RESET,
        // Register 0x60 -- Read
        input            IDT8V89307_INT_REQ,
        input            FAN_INT_L,
        input            MAC_CPLD_INT_L,
        input            CPLD2_INT,
        input   [ 2: 0]  LM75BD_INT,
        // Register 0x62 -- Read
        input            CPLD2_MOD_ABS,
        input            CPLD2_QSFP_INT,
        input   [ 1: 0]  TCAM_CPLD1_GIO_L,
        // Register 0x64 -- Read & Write
        input            CPU_THERMALTRIP_R,
        input            CPU_PROCHOT_R,
        input            L1_RCVRD_CLK_VALID_BKUP,
        input            L1_RCVRD_CLK_VALID,
        input            ONE_PPS_CLK,
        output           ONE_PPS_GPIO0,
        output           ONE_PPS_CPU,
        // Register 0x65 -- Read & Write
        output           LED_LOC_A,
        output           LED_DIAG_G,
        output           LED_DIAG_A,
        output           LED_FAN_G,
        output           LED_FAN_A,
        // Register 0x66 -- Read & Write
        output           LED_DEBUG_MODE,
        output  [ 1: 0]  LED_PSU_G,
        output  [ 1: 0]  LED_PSU_A,
        // Register 0x67 -- Read & Write
        output           USB_PWRON_N,
        input            USB1_VBUS,
        output           USB1_PWRFAULT,
        input            USB_PWRFLT_N,
        // Register 0x68 -- Read & Write
        output           SHUTDOWN_WAKE,
        input            ID_BUTTON,
        output  [ 1: 0]  PS_PDB_ALERT,
        output           SW1_SELECT,
        output           EE_WP,
        output  [ 1: 0]  PS_ON,
        // Register 0x69 -- Read & Write
        input            CPLD12_SYNC,
        input            CPLD21_SYNC,
        // Register 0x70 -- Read & Write
        output  [ 7: 0]  LED_PWM_G,
        // Register 0x71 -- Read & Write
        output  [ 7: 0]  LED_PWM_A,
		  // Register 0x72~ -- Read
		  input   [83: 0]  LED1_STREAM
      );

    // Signals for I2C Bus
    reg  [ 7: 0]  r_i2c_readdata;
    // Signals for SPF+ Transciver
    reg  [23: 0]  r_xsfp_int_24b;
    reg  [23: 0]  r_xsfp_int_mask_24b;
    reg  [23: 0]  r_xsfp_prsnt_24b;
    reg  [23: 0]  r_xsfp_tx_fault_24b;
    reg  [23: 0]  r_xsfp_tx_fault_mask_24b;
    reg  [23: 0]  r_xsfp_tx_dis_24b;
    reg  [23: 0]  r_xsfp_tx_dis_mask_24b;
    reg  [23: 0]  r_xsfp_rx_loss_24b;
    reg  [23: 0]  r_xsfp_rx_loss_mask_24b;
    // Signals for Reset
    reg           r_mac_reset_n;
    reg           r_manu_reset;
    reg           r_usb_reset_n;
    reg           r_cpld2_qsfp_reset_n;
    reg           r_cpld_reset;
    reg           r_idt8v89307_reset;
    reg           r_mgmt_phy_reset_n;
    reg           r_cpld1_tcam_crst_l;
    reg           r_cpld1_tcam_perst_l;
    reg           r_cpld1_tcam_srst_l;
    reg           r_reset_mac;
    reg           r_pca9548_5_reset_n;
    reg           r_pca9548_1_reset_n;
    reg           r_pca9548_reset_n;
    reg  [ 6: 0]  r_pca9548_7x_reset_n_7b;
    // Signals for Interrupt
    wire          r_idt8v89307_int_req;
    wire          r_fan_int_l;
    wire          r_mac_cpld_int_l;
    wire          r_cpld2_int;
    wire [ 2: 0]  r_lm75bd_int_3b;
    reg           r_idt8v89307_int_req_mask;
    reg           r_fan_int_l_mask;
    reg           r_mac_cpld_int_l_mask;
    reg           r_cpld2_int_mask;
    reg  [ 2: 0]  r_lm75bd_int_mask_3b;
    // Signals for LED PWM Control
    reg  [23: 0]  r_sfp_led_amber_24b;
    reg  [23: 0]  r_sfp_led_green_24b;
    // Signals for LED control
    reg           r_led_loc_a;
    reg           r_led_diag_g;
    reg           r_led_diag_a;
    reg           r_led_fan_g;
    reg           r_led_fan_a;
    reg           r_led_debug;
    reg  [ 1: 0]  r_led_psu_g_2b;
    reg  [ 1: 0]  r_led_psu_a_2b;
    reg  [ 7: 0]  r_led_pwm_g_8b;
    reg  [ 7: 0]  r_led_pwm_a_8b;
	 // Signals for LED_1 stream
	 wire  [83: 0]  r_led1_stream;
    // Signals for USB control
    reg           r_usb_pwr_ctrl;
    reg           r_usb1_pwr_fault;
    // Signals for MISC.
    reg           r_shutdown_wake;
    reg  [ 1: 0]  r_ps_pdb_alert_2b;
    reg           r_sw1_select;
    reg           r_ee_wp;
    reg  [ 1: 0]  r_ps_on_2b;
    reg           r_1pps_gpio0_mask;
    reg           r_1pps_cpu_mask;
    reg           r_cpld2_mod_abs_mask;
    reg           r_cpld2_mod_qsfp_int_mask;
    reg  [ 1: 0]  r_tcam_cpld1_gio_l_mask_2b;
    wire          r_cpld2_mod_abs;
    wire          r_cpld2_mod_qsfp_int;
    wire [ 1: 0]  r_tcam_cpld1_gio_l_2b;
    // Signals for delay signals
    reg  [31: 0]  r_delay_cnt;
    wire          r_mac_reset_n_delay_1ms;

    // Clock Signals
    wire clock   = MGMT_CLK;
    wire reset_n = MGMT_RST_N;
    // The Interrupt process for System
    assign SYS_CPLD_INT_CPU = ( &r_tcam_cpld1_gio_l_2b ) &    r_cpld2_mod_abs   & ( &r_xsfp_int_24b ) & r_cpld2_mod_qsfp_int &
                                 r_idt8v89307_int_req    & ( &r_lm75bd_int_3b ) &    r_fan_int_l      & INT_MGMT_PHY_N;
    assign CPLD23_INT_CPU   = r_cpld2_int;
	 
	 // Read stream from MAC LED_DATA1
	 assign r_led1_stream = LED1_STREAM;

    //-----------------------------------------------------------------
    // Misc. Statement
    //-----------------------------------------------------------------
    // Buffers for Reset
    assign MANU_RST           = r_manu_reset;
    assign MAC_RESET_N        = r_mac_reset_n_delay_1ms & reset_n;
    assign USB_RESET_N        = r_usb_reset_n           & reset_n;
    assign CPLD2_QSFP_RST     = r_cpld2_qsfp_reset_n    & reset_n;
    assign CPLD_RESET         = r_cpld_reset            & reset_n;
    assign IDT8V89307_RESET   = r_idt8v89307_reset      & reset_n;
    assign MGMT_PHY_RST_N     = r_mgmt_phy_reset_n      & reset_n;
    assign CPLD1_TCAM_CRST_L  = r_cpld1_tcam_crst_l     & reset_n;
    assign CPLD1_TCAM_PERST_L = r_cpld1_tcam_perst_l    & reset_n;
    assign CPLD1_TCAM_SRST_L  = r_cpld1_tcam_srst_l     & reset_n;
    assign RESET_MAC          = r_reset_mac;
    assign PCA9548_5_RESET    = r_pca9548_5_reset_n     & reset_n;
    assign PCA9548_1_RESET    = r_pca9548_1_reset_n     & reset_n;
    assign PCA9548_RESET      = r_pca9548_reset_n       & reset_n;
    generate                                        
        genvar  i;
        for ( i=0; i<7; i=i+1) begin : PCA9548_7X_RST_PROC
            assign PCA9548_7x_RESET[i] = r_pca9548_7x_reset_n_7b[i] & reset_n;
        end
    endgenerate

    // Buffers for SPF+ Transciver
    assign XSFP_TXDIS  = r_xsfp_tx_dis_24b;
    // Buffers for Interrupt
    assign r_idt8v89307_int_req  = IDT8V89307_INT_REQ | r_idt8v89307_int_req_mask;
    assign r_fan_int_l           = FAN_INT_L          | r_fan_int_l_mask;
    assign r_mac_cpld_int_l      = MAC_CPLD_INT_L     | r_mac_cpld_int_l_mask;
    assign r_cpld2_int           = CPLD2_INT          | r_cpld2_int_mask;
    assign r_lm75bd_int_3b       = LM75BD_INT         | r_lm75bd_int_mask_3b;
    assign r_tcam_cpld1_gio_l_2b = TCAM_CPLD1_GIO_L   | r_tcam_cpld1_gio_l_mask_2b;
    assign r_cpld2_mod_abs       = CPLD2_MOD_ABS      | r_cpld2_mod_abs_mask;
    assign r_cpld2_mod_qsfp_int  = CPLD2_QSFP_INT     | r_cpld2_mod_qsfp_int_mask;
    // Buffers for LED control
    assign REG_LED_SFP_A  = r_sfp_led_amber_24b;
    assign REG_LED_SFP_G  = r_sfp_led_green_24b;
    // Buffers for LED control
    assign LED_LOC_A      = r_led_loc_a;
    assign LED_DIAG_G     = r_led_diag_g;
    assign LED_DIAG_A     = r_led_diag_a;
    assign LED_FAN_G      = r_led_fan_g;
    assign LED_FAN_A      = r_led_fan_a;
    assign LED_DEBUG_MODE = r_led_debug;
    assign LED_PSU_G      = r_led_psu_g_2b;
    assign LED_PSU_A      = r_led_psu_a_2b;
    assign LED_PWM_G      = r_led_pwm_g_8b;
    assign LED_PWM_A      = r_led_pwm_a_8b;
    // Buffers for USB control
    assign USB_PWRON_N    = r_usb_pwr_ctrl    & USB1_VBUS;
    assign USB1_PWRFAULT  = r_usb1_pwr_fault  & USB_PWRFLT_N;
    // Buffers for MISC.
    assign SHUTDOWN_WAKE  = r_shutdown_wake;
    assign PS_PDB_ALERT   = r_ps_pdb_alert_2b & PS_AC_ALERT;
    assign SW1_SELECT     = r_sw1_select;
    assign EE_WP          = r_ee_wp;
    assign PS_ON          = r_ps_on_2b  | PS_PRSNT;
    assign ONE_PPS_GPIO0  = ONE_PPS_CLK & ~r_1pps_gpio0_mask;
    assign ONE_PPS_CPU    = ONE_PPS_CLK & ~r_1pps_cpu_mask;

    // Delay 1ms
    assign r_mac_reset_n_delay_1ms = ( r_delay_cnt >= 32'd25_000 ) ? r_mac_reset_n : 1'b0;

    always@ ( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_delay_cnt <= 32'h0000_0000;
      else if ( r_delay_cnt == 32'd25_000 ) // delay 1ms   1ms / 40ns = 25000
        r_delay_cnt <= r_delay_cnt;
      else
        r_delay_cnt <= r_delay_cnt + 32'h1;
    end


    //----------------------------------------------------------------------
    // SFP+ PLUGGABLE Signals
    //----------------------------------------------------------------------
    generate
      genvar xsfp_int_1_loop;
      for ( xsfp_int_1_loop=0; xsfp_int_1_loop<8; xsfp_int_1_loop=xsfp_int_1_loop+1 )
      begin : XSFP_INT_1_PROC
        always@ ( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_xsfp_int_24b[xsfp_int_1_loop] <= 1'b1;
          else if ( r_xsfp_int_mask_24b[xsfp_int_1_loop] == 1'b0 )
          begin
            if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_INT_1 )    // 0x04 -- SYSPLD_REG_XSFP_INT_1
              r_xsfp_int_24b[xsfp_int_1_loop] <= I2C_OFFSET_WDATA[xsfp_int_1_loop];
            else
              r_xsfp_int_24b[xsfp_int_1_loop] <= XSFP_MOD_ABS[xsfp_int_1_loop];
          end
          else
            r_xsfp_int_24b[xsfp_int_1_loop] <= 1'b1;
        end
      end
    endgenerate
    generate
      genvar xsfp_int_2_loop;
      for ( xsfp_int_2_loop=8; xsfp_int_2_loop<16; xsfp_int_2_loop=xsfp_int_2_loop+1 )
      begin : XSFP_INT_2_PROC
        always@ ( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_xsfp_int_24b[xsfp_int_2_loop] <= 1'b1;
          else if ( r_xsfp_int_mask_24b[xsfp_int_2_loop] == 1'b0 )
          begin
            if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_INT_2 )    // 0x06 -- SYSPLD_REG_XSFP_INT_2
              r_xsfp_int_24b[xsfp_int_2_loop] <= I2C_OFFSET_WDATA[xsfp_int_2_loop-8];
            else
              r_xsfp_int_24b[xsfp_int_2_loop] <= XSFP_MOD_ABS[xsfp_int_2_loop];
          end
          else
            r_xsfp_int_24b[xsfp_int_2_loop] <= 1'b1;
        end
      end
    endgenerate
    generate
      genvar xsfp_int_3_loop;
      for ( xsfp_int_3_loop=16; xsfp_int_3_loop<`XSFP_NUM; xsfp_int_3_loop=xsfp_int_3_loop+1 )
      begin : XSFP_INT_3_PROC
        always@ ( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_xsfp_int_24b[xsfp_int_3_loop] <= 1'b1;
          else if ( r_xsfp_int_mask_24b[xsfp_int_3_loop] == 1'b0 )
          begin
            if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_INT_3 )    // 0x08 -- SYSPLD_REG_XSFP_INT_3
              r_xsfp_int_24b[xsfp_int_3_loop] <= I2C_OFFSET_WDATA[xsfp_int_3_loop-16];
            else
              r_xsfp_int_24b[xsfp_int_3_loop] <= XSFP_MOD_ABS[xsfp_int_3_loop];
          end
          else
            r_xsfp_int_24b[xsfp_int_3_loop] <= 1'b1;
        end
      end
    endgenerate

    // Detected SFP+ Present
    generate
      genvar xsfp_prsnt_loop;
      for ( xsfp_prsnt_loop=0; xsfp_prsnt_loop<`XSFP_NUM; xsfp_prsnt_loop=xsfp_prsnt_loop+1 )
      begin : XSFP_PRSNT_1_PROC
        always@ ( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_xsfp_prsnt_24b[xsfp_prsnt_loop] <= 1'b1;
          else
            r_xsfp_prsnt_24b[xsfp_prsnt_loop] <= XSFP_MOD_ABS[xsfp_prsnt_loop];
        end
      end
    endgenerate

    // Detected SFP+ TX Fault
    generate
      genvar xsfp_txflt_loop;
      for ( xsfp_txflt_loop=0; xsfp_txflt_loop<`XSFP_NUM; xsfp_txflt_loop=xsfp_txflt_loop+1 )
      begin : XSFP_TXFLT_PROC
        always@ ( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_xsfp_tx_fault_24b[xsfp_txflt_loop] <= 1'b1;
          else if ( r_xsfp_tx_fault_mask_24b[xsfp_txflt_loop] == 1'b0 )
            r_xsfp_tx_fault_24b[xsfp_txflt_loop] <= XSFP_TXFLT[xsfp_txflt_loop];
          else
            r_xsfp_tx_fault_24b[xsfp_txflt_loop] <= 1'b1;
        end
      end
    endgenerate

    // Detected SFP+ RX LOSS
    generate
      genvar xsfp_rxlos_loop;
      for ( xsfp_rxlos_loop=0; xsfp_rxlos_loop<`XSFP_NUM; xsfp_rxlos_loop=xsfp_rxlos_loop+1 )
      begin : XSFP_RXLOS_PROC
        always@ ( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_xsfp_rx_loss_24b[xsfp_rxlos_loop] <= 1'b1;
          else if ( r_xsfp_rx_loss_mask_24b[xsfp_rxlos_loop] == 1'b0 )
            r_xsfp_rx_loss_24b[xsfp_rxlos_loop] <= XSFP_RXLOS[xsfp_rxlos_loop];
          else
            r_xsfp_rx_loss_24b[xsfp_rxlos_loop] <= 1'b1;
        end
      end
    endgenerate


    //-----------------------------------------------------------------
    // Write Operation Statement by I2C Host
    //-----------------------------------------------------------------
    // 0x05 -- SYSPLD_REG_XSFP_INT_MASK_1
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_int_mask_24b[7:0] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_INT_MASK_1 )
        r_xsfp_int_mask_24b[7:0] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x07 -- SYSPLD_REG_XSFP_INT_MASK_2
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_int_mask_24b[15:8] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_INT_MASK_2 )
        r_xsfp_int_mask_24b[15:8] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x09 -- SYSPLD_REG_XSFP_INT_MASK_3
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_int_mask_24b[23:16] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_INT_MASK_3 )
        r_xsfp_int_mask_24b[23:16] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x15 -- SYSPLD_REG_XSFP_TX_FAULT_MASK_1
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_fault_mask_24b[7:0] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_TX_FAULT_MASK_1 )
        r_xsfp_tx_fault_mask_24b[7:0] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x17 -- SYSPLD_REG_XSFP_TX_FAULT_MASK_2
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_fault_mask_24b[15:8] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_TX_FAULT_MASK_2 )
        r_xsfp_tx_fault_mask_24b[15:8] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x19 -- SYSPLD_REG_XSFP_TX_FAULT_MASK_3
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_fault_mask_24b[23:16] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_TX_FAULT_MASK_3 )
        r_xsfp_tx_fault_mask_24b[23:16] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x20 -- SYSPLD_REG_XSFP_TX_DIS_1
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_dis_24b[7:0] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_TX_DIS_1 )
        r_xsfp_tx_dis_24b[7:0] <= I2C_OFFSET_WDATA[7:0] & ( ~r_xsfp_tx_dis_mask_24b[7:0] );
    end

    // 0x21 -- SYSPLD_REG_XSFP_TX_DIS_MASK_1
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_dis_mask_24b[7:0] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_TX_DIS_MASK_1 )
        r_xsfp_tx_dis_mask_24b[7:0] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x22 -- SYSPLD_REG_XSFP_TX_DIS_2
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_dis_24b[15:8] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_TX_DIS_2 )
        r_xsfp_tx_dis_24b[15:8] <= I2C_OFFSET_WDATA[7:0] & ( ~r_xsfp_tx_dis_mask_24b[15:8] );
    end

    // 0x23 -- SYSPLD_REG_XSFP_TX_DIS_MASK_2
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_dis_mask_24b[15:8] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_TX_DIS_MASK_2 )
        r_xsfp_tx_dis_mask_24b[15:8] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x24 -- SYSPLD_REG_XSFP_TX_DIS_3
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_dis_24b[23:16] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_TX_DIS_3 )
        r_xsfp_tx_dis_24b[23:16] <= I2C_OFFSET_WDATA[7:0] & ( ~r_xsfp_tx_dis_mask_24b[23:16] );
    end

    // 0x25 -- SYSPLD_REG_XSFP_TX_DIS_MASK_3
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_dis_mask_24b[23:16] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_TX_DIS_MASK_3 )
        r_xsfp_tx_dis_mask_24b[23:16] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x31 -- SYSPLD_REG_XSFP_RX_LOS_MASK_1
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_rx_loss_mask_24b[7:0] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_RX_LOS_MASK_1 )
        r_xsfp_rx_loss_mask_24b[7:0] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x33 -- SYSPLD_REG_XSFP_RX_LOS_MASK_2
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_rx_loss_mask_24b[15:8] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_RX_LOS_MASK_2 )
        r_xsfp_rx_loss_mask_24b[15:8] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x35 -- SYSPLD_REG_XSFP_RX_LOS_MASK_3
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_rx_loss_mask_24b[23:16] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_XSFP_RX_LOS_MASK_3 )
        r_xsfp_rx_loss_mask_24b[23:16] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x40 -- SYSPLD_REG_1G_10G_LED_1
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_sfp_led_amber_24b[3:0] <= 4'hF;
        r_sfp_led_green_24b[3:0] <= 4'hF;			// 20160504
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_1G_10G_LED_1 )
      begin
        r_sfp_led_amber_24b[3:0] <= { I2C_OFFSET_WDATA[1], I2C_OFFSET_WDATA[3], I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_sfp_led_green_24b[3:0] <= { I2C_OFFSET_WDATA[0], I2C_OFFSET_WDATA[2], I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end

    // 0x41 -- SYSPLD_REG_1G_10G_LED_2
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_sfp_led_amber_24b[7:4] <= 4'hF;
        r_sfp_led_green_24b[7:4] <= 4'hF;			// 20160504
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_1G_10G_LED_2 )
      begin
        r_sfp_led_amber_24b[7:4] <= { I2C_OFFSET_WDATA[1], I2C_OFFSET_WDATA[3], I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_sfp_led_green_24b[7:4] <= { I2C_OFFSET_WDATA[0], I2C_OFFSET_WDATA[2], I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end

    // 0x42 -- SYSPLD_REG_1G_10G_LED_3
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_sfp_led_amber_24b[11:8] <= 4'hF;
        r_sfp_led_green_24b[11:8] <= 4'hF;			// 20160504
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_1G_10G_LED_3 )
      begin
        r_sfp_led_amber_24b[11:8] <= { I2C_OFFSET_WDATA[1], I2C_OFFSET_WDATA[3], I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_sfp_led_green_24b[11:8] <= { I2C_OFFSET_WDATA[0], I2C_OFFSET_WDATA[2], I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end

    // 0x43 -- SYSPLD_REG_1G_10G_LED_4
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_sfp_led_amber_24b[15:12] <= 4'hF;
        r_sfp_led_green_24b[15:12] <= 4'hF;			// 20160504
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_1G_10G_LED_4 )
      begin
        r_sfp_led_amber_24b[15:12] <= { I2C_OFFSET_WDATA[1], I2C_OFFSET_WDATA[3], I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_sfp_led_green_24b[15:12] <= { I2C_OFFSET_WDATA[0], I2C_OFFSET_WDATA[2], I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end

    // 0x44 -- SYSPLD_REG_1G_10G_LED_5
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_sfp_led_amber_24b[19:16] <= 4'hF;
        r_sfp_led_green_24b[19:16] <= 4'hF;			// 20160504
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_1G_10G_LED_5 )
      begin
        r_sfp_led_amber_24b[19:16] <= { I2C_OFFSET_WDATA[1], I2C_OFFSET_WDATA[3], I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_sfp_led_green_24b[19:16] <= { I2C_OFFSET_WDATA[0], I2C_OFFSET_WDATA[2], I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end

    // 0x45 -- SYSPLD_REG_1G_10G_LED_6
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_sfp_led_amber_24b[23:20] <= 4'hF;
        r_sfp_led_green_24b[23:20] <= 4'hF;			// 20160504
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_1G_10G_LED_6 )
      begin
        r_sfp_led_amber_24b[23:20] <= { I2C_OFFSET_WDATA[1], I2C_OFFSET_WDATA[3], I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_sfp_led_green_24b[23:20] <= { I2C_OFFSET_WDATA[0], I2C_OFFSET_WDATA[2], I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end

    // 0x50 -- SYSPLD_REG_SYSTEM_RST_5
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_mac_reset_n        <= 1'b1;
        r_manu_reset         <= 1'b1;
        r_usb_reset_n        <= 1'b1;
        r_cpld2_qsfp_reset_n <= 1'b1;
        r_cpld_reset         <= 1'b1;
        r_idt8v89307_reset   <= 1'b1;
        r_mgmt_phy_reset_n   <= 1'b1;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_SYSTEM_RST_5 )
      begin
        r_mac_reset_n        <= I2C_OFFSET_WDATA[7];
        r_manu_reset         <= I2C_OFFSET_WDATA[6];
        r_usb_reset_n        <= I2C_OFFSET_WDATA[5];
        r_cpld2_qsfp_reset_n <= I2C_OFFSET_WDATA[3];
        r_cpld_reset         <= I2C_OFFSET_WDATA[2];
        r_idt8v89307_reset   <= I2C_OFFSET_WDATA[1];
        r_mgmt_phy_reset_n   <= I2C_OFFSET_WDATA[0];
      end
    end

    // 0x11 -- SYSPLD_REG_SYSTEM_RST_6
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_cpld1_tcam_crst_l  <= 1'b1;
        r_cpld1_tcam_perst_l <= 1'b1;
        r_cpld1_tcam_srst_l  <= 1'b1;
        r_reset_mac          <= 1'b0;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_SYSTEM_RST_6 )
      begin
        r_cpld1_tcam_crst_l  <= I2C_OFFSET_WDATA[7];
        r_cpld1_tcam_perst_l <= I2C_OFFSET_WDATA[6];
        r_cpld1_tcam_srst_l  <= I2C_OFFSET_WDATA[5];
        r_reset_mac          <= I2C_OFFSET_WDATA[4];
      end
    end

    // 0x52 -- SYSPLD_REG_SYSTEM_RST_7
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_pca9548_5_reset_n <= 1'b1;
        r_pca9548_1_reset_n <= 1'b1;
        r_pca9548_reset_n   <= 1'b1;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_SYSTEM_RST_7 )
      begin
        r_pca9548_5_reset_n <= I2C_OFFSET_WDATA[5];
        r_pca9548_1_reset_n <= I2C_OFFSET_WDATA[1];
        r_pca9548_reset_n   <= I2C_OFFSET_WDATA[0];
      end
    end

    // 0x13 -- SYSPLD_REG_SYSTEM_RST_8
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_pca9548_7x_reset_n_7b <= 7'h7F;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_SYSTEM_RST_8 )
        r_pca9548_7x_reset_n_7b <= I2C_OFFSET_WDATA[6:0];
    end

    // 0x61 -- SYSPLD_REG_SYSTEM_INT_MASK_5
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_idt8v89307_int_req_mask <= 1'b0;
        r_fan_int_l_mask          <= 1'b0;
        r_mac_cpld_int_l_mask     <= 1'b0;
        r_cpld2_int_mask          <= 1'b0;
        r_lm75bd_int_mask_3b      <= 3'h0;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_SYSTEM_INT_MASK_5 )
      begin
        r_idt8v89307_int_req_mask <= I2C_OFFSET_WDATA[7];
        r_fan_int_l_mask          <= I2C_OFFSET_WDATA[6];
        r_mac_cpld_int_l_mask     <= I2C_OFFSET_WDATA[5];
        r_cpld2_int_mask          <= I2C_OFFSET_WDATA[3];
        r_lm75bd_int_mask_3b      <= I2C_OFFSET_WDATA[2:0];
      end
    end

    // 0x63 -- SYSPLD_REG_SYSTEM_INT_MASK_6
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_cpld2_mod_abs_mask            <= 1'b0;
        r_cpld2_mod_qsfp_int_mask       <= 1'b0;
        r_tcam_cpld1_gio_l_mask_2b[1:0] <= 2'h0;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_SYSTEM_INT_MASK_6 )
      begin
        r_cpld2_mod_abs_mask            <= I2C_OFFSET_WDATA[3];
        r_cpld2_mod_qsfp_int_mask       <= I2C_OFFSET_WDATA[2];
        r_tcam_cpld1_gio_l_mask_2b[1:0] <= I2C_OFFSET_WDATA[1:0];
      end
    end

    // 0x64 -- SYSPLD_REG_CLK_1PPS_MASK
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_1pps_gpio0_mask <= 1'b0;
        r_1pps_cpu_mask   <= 1'b0;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_CLK_1PPS_MASK )
      begin
        r_1pps_gpio0_mask <= I2C_OFFSET_WDATA[1];
        r_1pps_cpu_mask   <= I2C_OFFSET_WDATA[0];
      end
    end

    // 0x65 -- SYSPLD_REG_SYSTEM_LED_1
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_led_loc_a  <= 1'b0;
        r_led_diag_g <= 1'b0;
        r_led_diag_a <= 1'b0;
        r_led_fan_g  <= 1'b0;
        r_led_fan_a  <= 1'b0;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_SYSTEM_LED_1 )
      begin
        r_led_loc_a  <= I2C_OFFSET_WDATA[4];
        r_led_diag_g <= I2C_OFFSET_WDATA[3];
        r_led_diag_a <= I2C_OFFSET_WDATA[2];
        r_led_fan_g  <= I2C_OFFSET_WDATA[1];
        r_led_fan_a  <= I2C_OFFSET_WDATA[0];
      end
    end

    // 0x66 -- SYSPLD_REG_SYSTEM_LED_2
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_led_debug       <= 1'b0;
        r_led_psu_g_2b    <= 2'h0;
        r_led_psu_a_2b    <= 2'h0;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_SYSTEM_LED_2 )
      begin
        r_led_debug       <= I2C_OFFSET_WDATA[7];
        r_led_psu_g_2b[0] <= I2C_OFFSET_WDATA[6];
        r_led_psu_a_2b[0] <= I2C_OFFSET_WDATA[5];
        r_led_psu_g_2b[1] <= I2C_OFFSET_WDATA[4];
        r_led_psu_a_2b[1] <= I2C_OFFSET_WDATA[3];
      end
    end

    // 0x67 -- SYSPLD_REG_USB_STATUS
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_usb_pwr_ctrl   <= 1'b1;
        r_usb1_pwr_fault <= 1'b1;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_USB_STATUS )
      begin
        r_usb_pwr_ctrl   <= I2C_OFFSET_WDATA[3];
        r_usb1_pwr_fault <= I2C_OFFSET_WDATA[1];
      end
    end

    // 0x68 -- SYSPLD_REG_MISC_STATUS_1
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_shutdown_wake   <= 1'b1;
        r_ps_pdb_alert_2b <= 2'h3;
        r_sw1_select      <= 1'b0;
        r_ee_wp           <= 1'b0;
        r_ps_on_2b        <= 2'h0;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_MISC_STATUS_1 )
      begin
        r_shutdown_wake   <= I2C_OFFSET_WDATA[7];
        r_ps_pdb_alert_2b <= I2C_OFFSET_WDATA[5:4];
        r_sw1_select      <= I2C_OFFSET_WDATA[3];
        r_ee_wp           <= I2C_OFFSET_WDATA[2];
        r_ps_on_2b        <= I2C_OFFSET_WDATA[1:0];
      end
    end

    // 0x70 -- SYSPLD_REG_SYSTEM_LED_3
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_led_pwm_g_8b  <= 8'hAA;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_SYSTEM_LED_3 )
        r_led_pwm_g_8b  <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x71 -- SYSPLD_REG_SYSTEM_LED_4
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_led_pwm_a_8b  <= 8'h55;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `SYSPLD_REG_SYSTEM_LED_4 )
        r_led_pwm_a_8b  <= I2C_OFFSET_WDATA[7:0];
    end


    //----------------------------------------------------------------------
    // Read Operation Statement by I2C Host
    //----------------------------------------------------------------------
    assign I2C_OFFSET_RDATA = r_i2c_readdata;

    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_i2c_readdata <= 8'h00;
      end
      else if ( I2C_READ == 1'b1 )
      begin
        case ( I2C_OFFSET_ADDR )
          `SYSPLD_REG_BOARD_INFO                 : r_i2c_readdata <= { 2'h0,  BOARD_ID, PCB_VER                                   };  // 0x00 : SYSPLD_REG_BOARD_INFO
          `SYSPLD_REG_CPLD_VERSION               : r_i2c_readdata <= {        `CPLD_VER                                           };  // 0x01 : SYSPLD_REG_CPLD_VERSION
          `SYSPLD_REG_POWER_STATUS               : r_i2c_readdata <= { 2'h0,  PS_AC_ALERT[0], PS_AC_ALERT[1],                         // 0x02 : SYSPLD_REG_POWER_STATUS
                                                                              PS_12V_PG[0],   PS_12V_PG[1],
                                                                              PS_PRSNT[0],    PS_PRSNT[1]                         };
          `SYSPLD_REG_XSFP_INT_1                 : r_i2c_readdata <= {        r_xsfp_int_24b[7:0]                                 };  // 0x04 : SYSPLD_REG_XSFP_INT_1
          `SYSPLD_REG_XSFP_INT_MASK_1            : r_i2c_readdata <= {        r_xsfp_int_mask_24b[7:0]                            };  // 0x05 : SYSPLD_REG_XSFP_INT_MASK_1
          `SYSPLD_REG_XSFP_INT_2                 : r_i2c_readdata <= {        r_xsfp_int_24b[15:8]                                };  // 0x06 : SYSPLD_REG_XSFP_INT_2
          `SYSPLD_REG_XSFP_INT_MASK_2            : r_i2c_readdata <= {        r_xsfp_int_mask_24b[15:8]                           };  // 0x07 : SYSPLD_REG_XSFP_INT_MASK_2
          `SYSPLD_REG_XSFP_INT_3                 : r_i2c_readdata <= {        r_xsfp_int_24b[23:16]                               };  // 0x08 : SYSPLD_REG_XSFP_INT_3
          `SYSPLD_REG_XSFP_INT_MASK_3            : r_i2c_readdata <= {        r_xsfp_int_mask_24b[23:16]                          };  // 0x09 : SYSPLD_REG_XSFP_INT_MASK_3
          `SYSPLD_REG_XSFP_PRSNT_1               : r_i2c_readdata <= {        r_xsfp_prsnt_24b[7:0]                               };  // 0x10 : SYSPLD_REG_XSFP_PRSNT_1
          `SYSPLD_REG_XSFP_PRSNT_2               : r_i2c_readdata <= {        r_xsfp_prsnt_24b[15:8]                              };  // 0x11 : SYSPLD_REG_XSFP_PRSNT_2
          `SYSPLD_REG_XSFP_PRSNT_3               : r_i2c_readdata <= {        r_xsfp_prsnt_24b[23:16]                             };  // 0x12 : SYSPLD_REG_XSFP_PRSNT_3
          `SYSPLD_REG_XSFP_TX_FAULT_1            : r_i2c_readdata <= {        r_xsfp_tx_fault_24b[7:0]                            };  // 0x14 : SYSPLD_REG_XSFP_TX_FAULT_1
          `SYSPLD_REG_XSFP_TX_FAULT_MASK_1       : r_i2c_readdata <= {        r_xsfp_tx_fault_mask_24b[7:0]                       };  // 0x15 : SYSPLD_REG_XSFP_TX_FAULT_MASK_1
          `SYSPLD_REG_XSFP_TX_FAULT_2            : r_i2c_readdata <= {        r_xsfp_tx_fault_24b[15:8]                           };  // 0x16 : SYSPLD_REG_XSFP_TX_FAULT_2
          `SYSPLD_REG_XSFP_TX_FAULT_MASK_2       : r_i2c_readdata <= {        r_xsfp_tx_fault_mask_24b[15:8]                      };  // 0x17 : SYSPLD_REG_XSFP_TX_FAULT_MASK_2
          `SYSPLD_REG_XSFP_TX_FAULT_3            : r_i2c_readdata <= {        r_xsfp_tx_fault_24b[23:16]                          };  // 0x18 : SYSPLD_REG_XSFP_TX_FAULT_3
          `SYSPLD_REG_XSFP_TX_FAULT_MASK_3       : r_i2c_readdata <= {        r_xsfp_tx_fault_mask_24b[23:16]                     };  // 0x19 : SYSPLD_REG_XSFP_TX_FAULT_MASK_3
          `SYSPLD_REG_XSFP_TX_DIS_1              : r_i2c_readdata <= {        r_xsfp_tx_dis_24b[7:0]                              };  // 0x20 : SYSPLD_REG_XSFP_TX_DIS_1
          `SYSPLD_REG_XSFP_TX_DIS_MASK_1         : r_i2c_readdata <= {        r_xsfp_tx_dis_mask_24b[7:0]                         };  // 0x21 : SYSPLD_REG_XSFP_TX_DIS_MASK_1
          `SYSPLD_REG_XSFP_TX_DIS_2              : r_i2c_readdata <= {        r_xsfp_tx_dis_24b[15:8]                             };  // 0x22 : SYSPLD_REG_XSFP_TX_DIS_2
          `SYSPLD_REG_XSFP_TX_DIS_MASK_2         : r_i2c_readdata <= {        r_xsfp_tx_dis_mask_24b[15:8]                        };  // 0x23 : SYSPLD_REG_XSFP_TX_DIS_MASK_2
          `SYSPLD_REG_XSFP_TX_DIS_3              : r_i2c_readdata <= {        r_xsfp_tx_dis_24b[23:16]                            };  // 0x24 : SYSPLD_REG_XSFP_TX_DIS_3
          `SYSPLD_REG_XSFP_TX_DIS_MASK_3         : r_i2c_readdata <= {        r_xsfp_tx_dis_mask_24b[23:16]                       };  // 0x25 : SYSPLD_REG_XSFP_TX_DIS_MASK_3
          `SYSPLD_REG_XSFP_RX_LOS_1              : r_i2c_readdata <= {        r_xsfp_rx_loss_24b[7:0]                             };  // 0x30 : SYSPLD_REG_XSFP_RX_LOS_1
          `SYSPLD_REG_XSFP_RX_LOS_MASK_1         : r_i2c_readdata <= {        r_xsfp_rx_loss_mask_24b[7:0]                        };  // 0x31 : SYSPLD_REG_XSFP_RX_LOS_MASK_1
          `SYSPLD_REG_XSFP_RX_LOS_2              : r_i2c_readdata <= {        r_xsfp_rx_loss_24b[15:8]                            };  // 0x32 : SYSPLD_REG_XSFP_RX_LOS_2
          `SYSPLD_REG_XSFP_RX_LOS_MASK_2         : r_i2c_readdata <= {        r_xsfp_rx_loss_mask_24b[15:8]                       };  // 0x33 : SYSPLD_REG_XSFP_RX_LOS_MASK_2
          `SYSPLD_REG_XSFP_RX_LOS_3              : r_i2c_readdata <= {        r_xsfp_rx_loss_24b[23:16]                           };  // 0x34 : SYSPLD_REG_XSFP_RX_LOS_3
          `SYSPLD_REG_XSFP_RX_LOS_MASK_3         : r_i2c_readdata <= {        r_xsfp_rx_loss_mask_24b[23:16]                      };  // 0x35 : SYSPLD_REG_XSFP_RX_LOS_MASK_3
          `SYSPLD_REG_1G_10G_LED_1               : r_i2c_readdata <= {        r_sfp_led_amber_24b[0],    r_sfp_led_green_24b[0],      // 0x40 : SYSPLD_REG_1G_10G_LED_1
                                                                              r_sfp_led_amber_24b[1],    r_sfp_led_green_24b[1], 
                                                                              r_sfp_led_amber_24b[2],    r_sfp_led_green_24b[2], 
                                                                              r_sfp_led_amber_24b[3],    r_sfp_led_green_24b[3]   };
          `SYSPLD_REG_1G_10G_LED_2               : r_i2c_readdata <= {        r_sfp_led_amber_24b[4],    r_sfp_led_green_24b[4],      // 0x41 : SYSPLD_REG_1G_10G_LED_2
                                                                              r_sfp_led_amber_24b[5],    r_sfp_led_green_24b[5], 
                                                                              r_sfp_led_amber_24b[6],    r_sfp_led_green_24b[6], 
                                                                              r_sfp_led_amber_24b[7],    r_sfp_led_green_24b[7]   };
          `SYSPLD_REG_1G_10G_LED_3               : r_i2c_readdata <= {        r_sfp_led_amber_24b[8],    r_sfp_led_green_24b[8],      // 0x42 : SYSPLD_REG_1G_10G_LED_3
                                                                              r_sfp_led_amber_24b[9],    r_sfp_led_green_24b[9], 
                                                                              r_sfp_led_amber_24b[10],   r_sfp_led_green_24b[10],
                                                                              r_sfp_led_amber_24b[11],   r_sfp_led_green_24b[11]  };
          `SYSPLD_REG_1G_10G_LED_4               : r_i2c_readdata <= {        r_sfp_led_amber_24b[12],   r_sfp_led_green_24b[12],     // 0x43 : SYSPLD_REG_1G_10G_LED_4
                                                                              r_sfp_led_amber_24b[13],   r_sfp_led_green_24b[13],
                                                                              r_sfp_led_amber_24b[14],   r_sfp_led_green_24b[14],
                                                                              r_sfp_led_amber_24b[15],   r_sfp_led_green_24b[15]  };
          `SYSPLD_REG_1G_10G_LED_5               : r_i2c_readdata <= {        r_sfp_led_amber_24b[16],   r_sfp_led_green_24b[16],     // 0x44 : SYSPLD_REG_1G_10G_LED_5
                                                                              r_sfp_led_amber_24b[17],   r_sfp_led_green_24b[17],
                                                                              r_sfp_led_amber_24b[18],   r_sfp_led_green_24b[18],
                                                                              r_sfp_led_amber_24b[19],   r_sfp_led_green_24b[19]  };
          `SYSPLD_REG_1G_10G_LED_6               : r_i2c_readdata <= {        r_sfp_led_amber_24b[20],   r_sfp_led_green_24b[20],     // 0x45 : SYSPLD_REG_1G_10G_LED_6
                                                                              r_sfp_led_amber_24b[21],   r_sfp_led_green_24b[21],
                                                                              r_sfp_led_amber_24b[22],   r_sfp_led_green_24b[22],
                                                                              r_sfp_led_amber_24b[23],   r_sfp_led_green_24b[23]  };
          `SYSPLD_REG_SYSTEM_RST_5               : r_i2c_readdata <= {        r_mac_reset_n,                                          // 0x50 : SYSPLD_REG_SYSTEM_RST_5
                                                                              r_manu_reset,
                                                                              r_usb_reset_n,
                                                                       1'h0,  r_cpld2_qsfp_reset_n,
                                                                              r_cpld_reset,
                                                                              r_idt8v89307_reset,
                                                                              r_mgmt_phy_reset_n                                  };
          `SYSPLD_REG_SYSTEM_RST_6               : r_i2c_readdata <= {        r_cpld1_tcam_crst_l,                                    // 0x51 : SYSPLD_REG_SYSTEM_RST_6
                                                                              r_cpld1_tcam_perst_l,
                                                                              r_cpld1_tcam_srst_l,
                                                                              r_reset_mac,
                                                                              RESET_SYS_CPLD,
                                                                              CPU_JTAG_RST,
                                                                       2'h0                                                       };
          `SYSPLD_REG_SYSTEM_RST_7               : r_i2c_readdata <= { 2'h0,  r_pca9548_5_reset_n,                                    // 0x52 : SYSPLD_REG_SYSTEM_RST_7
                                                                       3'h0,  r_pca9548_1_reset_n,
                                                                              r_pca9548_reset_n                                   };
          `SYSPLD_REG_SYSTEM_RST_8               : r_i2c_readdata <= { 1'h0,  r_pca9548_7x_reset_n_7b[6:0]                        };  // 0x53 : SYSPLD_REG_SYSTEM_RST_8

          `SYSPLD_REG_SYSTEM_INT_5               : r_i2c_readdata <= {        r_idt8v89307_int_req,                                   // 0x60 : SYSPLD_REG_SYSTEM_INT_5
                                                                              r_fan_int_l,
                                                                              r_mac_cpld_int_l,
                                                                       1'h0,  r_cpld2_int,
                                                                              r_lm75bd_int_3b[2:0]                                };
          `SYSPLD_REG_SYSTEM_INT_MASK_5          : r_i2c_readdata <= {        r_idt8v89307_int_req_mask,                              // 0x61 : SYSPLD_REG_SYSTEM_INT_MASK_5
                                                                              r_fan_int_l_mask,
                                                                              r_mac_cpld_int_l_mask,
                                                                       1'h0,  r_cpld2_int_mask,
                                                                              r_lm75bd_int_mask_3b[2:0]                           };
          `SYSPLD_REG_SYSTEM_INT_6               : r_i2c_readdata <= { 6'h00, r_cpld2_mod_abs,                                        // 0x62 : SYSPLD_REG_SYSTEM_INT_6
                                                                              r_cpld2_mod_qsfp_int,
                                                                              r_tcam_cpld1_gio_l_2b[1:0]                          };
          `SYSPLD_REG_SYSTEM_INT_MASK_6          : r_i2c_readdata <= { 6'h00, r_cpld2_mod_abs_mask,                                   // 0x63 : SYSPLD_REG_SYSTEM_INT_MASK_6
                                                                              r_cpld2_mod_qsfp_int_mask,
                                                                              r_tcam_cpld1_gio_l_mask_2b[1:0]                     };
          `SYSPLD_REG_CLK_1PPS_MASK              : r_i2c_readdata <= { 1'b0,  CPU_THERMALTRIP_R,                                      // 0x64 : SYSPLD_REG_CLK_1PPS_MASK
                                                                              CPU_PROCHOT_R,
                                                                              L1_RCVRD_CLK_VALID_BKUP,
                                                                              L1_RCVRD_CLK_VALID,
                                                                              r_1pps_gpio0_mask,
                                                                              r_1pps_cpu_mask                                     };
          `SYSPLD_REG_SYSTEM_LED_1               : r_i2c_readdata <= { 3'h0,  r_led_loc_a,                                            // 0x65 : SYSPLD_REG_SYSTEM_LED_1
                                                                              r_led_diag_g,
                                                                              r_led_diag_a,
                                                                              r_led_fan_g,
                                                                              r_led_fan_a                                         };
          `SYSPLD_REG_SYSTEM_LED_2               : r_i2c_readdata <= {        r_led_debug,                                            // 0x66 : SYSPLD_REG_SYSTEM_LED_2
                                                                              r_led_psu_g_2b[0],
                                                                              r_led_psu_a_2b[0],
                                                                              r_led_psu_g_2b[1],
                                                                              r_led_psu_a_2b[1],
                                                                       3'h0,                                                      };
          `SYSPLD_REG_USB_STATUS                 : r_i2c_readdata <= { 4'h0,  r_usb_pwr_ctrl,                                         // 0x67 : SYSPLD_REG_USB_STATUS
                                                                              USB1_VBUS,
                                                                              r_usb1_pwr_fault,
                                                                              USB_PWRFLT_N                                        };
          `SYSPLD_REG_MISC_STATUS_1              : r_i2c_readdata <= {        r_shutdown_wake,                                        // 0x68 : SYSPLD_REG_MISC_STATUS_1
                                                                              ID_BUTTON,
                                                                              r_ps_pdb_alert_2b[1:0],
                                                                              r_sw1_select,
                                                                              r_ee_wp,
                                                                              r_ps_on_2b[1:0]                                     };
          `SYSPLD_REG_MISC_STATUS_2              : r_i2c_readdata <= { 6'h0,  CPLD12_SYNC,                                            // 0x69 : SYSPLD_REG_MISC_STATUS_2
                                                                              CPLD21_SYNC                                         };
          `SYSPLD_REG_SYSTEM_LED_3               : r_i2c_readdata <= {        r_led_pwm_g_8b[7:0]                                 };  // 0x70 : SYSPLD_REG_SYSTEM_LED_3
          `SYSPLD_REG_SYSTEM_LED_4               : r_i2c_readdata <= {        r_led_pwm_a_8b[7:0]                                 };  // 0x71 : SYSPLD_REG_SYSTEM_LED_4
			 
			 `SYSPLD_REG_STREAM_LED1_1               : r_i2c_readdata <= {        r_led1_stream[7:0]                                  };  // 0x81 : SYSPLD_REG_STREAM_LED_1
			 `SYSPLD_REG_STREAM_LED1_2               : r_i2c_readdata <= {        r_led1_stream[15:8]                                 };  // 0x82 : SYSPLD_REG_STREAM_LED_2
			 `SYSPLD_REG_STREAM_LED1_3               : r_i2c_readdata <= {        r_led1_stream[23:16]                                };  // 0x83 : SYSPLD_REG_STREAM_LED_3
			 `SYSPLD_REG_STREAM_LED1_4               : r_i2c_readdata <= {        r_led1_stream[31:24]                                };  // 0x84 : SYSPLD_REG_STREAM_LED_4
			 `SYSPLD_REG_STREAM_LED1_5               : r_i2c_readdata <= {        r_led1_stream[39:32]                                };  // 0x85 : SYSPLD_REG_STREAM_LED_5
			 `SYSPLD_REG_STREAM_LED1_6               : r_i2c_readdata <= {        r_led1_stream[47:40]                                };  // 0x86 : SYSPLD_REG_STREAM_LED_6
			 `SYSPLD_REG_STREAM_LED1_7               : r_i2c_readdata <= {        r_led1_stream[55:48]                                };  // 0x87 : SYSPLD_REG_STREAM_LED_7
			 `SYSPLD_REG_STREAM_LED1_8               : r_i2c_readdata <= {        r_led1_stream[63:56]                                };  // 0x88 : SYSPLD_REG_STREAM_LED_8
			 `SYSPLD_REG_STREAM_LED1_9               : r_i2c_readdata <= {        r_led1_stream[71:64]                                };  // 0x89 : SYSPLD_REG_STREAM_LED_9
			 `SYSPLD_REG_STREAM_LED1_10              : r_i2c_readdata <= {        r_led1_stream[79:72]                                };  // 0x90 : SYSPLD_REG_STREAM_LED_10
			 `SYSPLD_REG_STREAM_LED1_11              : r_i2c_readdata <= { 4'h0,  r_led1_stream[83:80]                                };  // 0x91 : SYSPLD_REG_STREAM_LED_11
          default                                : r_i2c_readdata <= 8'hFF;
        endcase
      end
    end

endmodule
