//---------------------------------------------------------------------
//---------------------------------------------------------------------
// This is a LED logic with PWM.
// Vincent Huang, Galaxy TW
//
// History :
// 2016-02-01 Created
//---------------------------------------------------------------------
module led_display (
        // Clock signals
        input           MGMT_CLK, 
        input           MGMT_RST_N,
        // QSFP PWM
        input  [ 7: 0]  REG_PORT_10G_BY4_LED_PWM_R,
        input  [ 7: 0]  REG_PORT_10G_BY4_LED_PWM_G,
        input  [ 7: 0]  REG_PORT_10G_BY4_LED_PWM_B,
        input  [ 7: 0]  REG_PORT_25G_BY4_LED_PWM_R,
        input  [ 7: 0]  REG_PORT_25G_BY4_LED_PWM_G,
        input  [ 7: 0]  REG_PORT_25G_BY4_LED_PWM_B,
        //
        output          REG_10G_BY4_LED_R,
        output          REG_10G_BY4_LED_G,
        output          REG_10G_BY4_LED_B,
        output          REG_25G_BY4_LED_R,
        output          REG_25G_BY4_LED_G,
        output          REG_25G_BY4_LED_B
      );


    // Sigansl for this design
    wire  [23: 0]  led_pwm_10g_by4;
    wire  [23: 0]  led_pwm_25g_by4;
    wire  [ 2: 0]  led_dispay_10g_by4;
    wire  [ 2: 0]  led_dispay_25g_by4;

    // Buffers
    wire clock   = MGMT_CLK;
    wire reset_n = MGMT_RST_N;
    
    assign led_pwm_10g_by4 = { REG_PORT_10G_BY4_LED_PWM_B, REG_PORT_10G_BY4_LED_PWM_G, REG_PORT_10G_BY4_LED_PWM_R };
    assign led_pwm_25g_by4 = { REG_PORT_25G_BY4_LED_PWM_B, REG_PORT_25G_BY4_LED_PWM_G, REG_PORT_25G_BY4_LED_PWM_R };
    
    assign { REG_10G_BY4_LED_B, REG_10G_BY4_LED_G, REG_10G_BY4_LED_R } = led_dispay_10g_by4;
    assign { REG_25G_BY4_LED_B, REG_25G_BY4_LED_G, REG_25G_BY4_LED_R } = led_dispay_25g_by4;

    /************** LED Duty Control Logic ****************/
    /* The idea here is to PWM the LED display
     signals at a frequency of 100khz
     with a 8-bit width value. 
    */
    reg  [ 7: 0]  pwm_cntr;

    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        pwm_cntr <= 8'h00;
      else
        pwm_cntr <= pwm_cntr + 8'd1;
    end

    // 4x10G LED PWM 
    generate 
      genvar  pwm_10g_by4_num;
      for ( pwm_10g_by4_num=0; pwm_10g_by4_num<3; pwm_10g_by4_num=pwm_10g_by4_num+1 ) 
      begin : PWM_10G_BY4_LOOP
        led_pwm LED_PWM_10G_BY4_I (  clock, reset_n, pwm_cntr, led_pwm_10g_by4[(pwm_10g_by4_num*8)+7 : (pwm_10g_by4_num*8)], led_dispay_10g_by4[pwm_10g_by4_num] );
      end    
    endgenerate

    // 4x25G LED PWM 
    generate 
      genvar  pwm_25g_by4_num;
      for ( pwm_25g_by4_num=0; pwm_25g_by4_num<3; pwm_25g_by4_num=pwm_25g_by4_num+1 ) 
      begin : PWM_25G_BY4_LOOP
        led_pwm LED_PWM_25G_BY4_I ( clock, reset_n, pwm_cntr, led_pwm_25g_by4[(pwm_25g_by4_num*8)+7 : (pwm_25g_by4_num*8)], led_dispay_25g_by4[pwm_25g_by4_num] );
      end    
    endgenerate

endmodule
