//---------------------------------------------------------------------
//---------------------------------------------------------------------
// MAC LED Encode for Project ES5654BQ CPLD2
// Vincent Huang, Galaxy TW
//
// History :
// 2016-02-01 Created
//---------------------------------------------------------------------
module mac_led_encode (
        // Clock signals
        input            MGMT_CLK,
        input            MGMT_RST_N,
		  output  [11: 0]	 MAC_PORT_CONFIG,				// 20160512
		  output  [23: 0]	 MAC_LANE_CTRL,				// 20160601
        // LED  Stream
        input   [ 1: 0]  LED_CLK,
        input   [ 1: 0]  LED_DATA,
        // SFP+ Port 25 ~ Port 48
        output  [23: 0]  MAC_LED_SFP_A,
        output  [23: 0]  MAC_LED_SFP_G,
        // QSFP Port 49 ~ Port 54
        output  [ 5: 0]  MAC_PORT_100G_LED_G,
        output  [ 5: 0]  MAC_PORT_40G_LED_B,
		  output  [83: 0]  LED_DAT0_STREAM,
		  output  [83: 0]  LED_DAT2_STREAM
      );

    // Signals for LED stream
    reg   [83: 0]  mac_led_parallel_data_qsfp;		// 20160512
    reg   [83: 0]  mac_led_parallel_data_sfp;
	 assign LED_DAT0_STREAM = mac_led_parallel_data_qsfp;                                       // Read Stream from MAC LED_DATA0
	 assign LED_DAT2_STREAM = mac_led_parallel_data_sfp;                                        // Read Stream from MAC LED_DATA2

    // Clock Signals
    wire reset_n = MGMT_RST_N;

     // LED Data Mapping for SFP+ 25 ~ 28, 20160504
    generate 
      genvar  sfp_port_num_1;
      for ( sfp_port_num_1=0; sfp_port_num_1<4; sfp_port_num_1=sfp_port_num_1+1 )
		begin : MAC_LED_BIT_MAPPING_SFP_1
			 assign MAC_LED_SFP_G[sfp_port_num_1] = ( mac_led_parallel_data_sfp[((sfp_port_num_1*3)+3):((sfp_port_num_1*3)+2)] == 2'b00 )? (! mac_led_parallel_data_sfp[(sfp_port_num_1*3)+4]): 1'b1;	   // SFP+ 10G
			 assign MAC_LED_SFP_A[sfp_port_num_1] = ( mac_led_parallel_data_sfp[((sfp_port_num_1*3)+3):((sfp_port_num_1*3)+2)] == 2'b10 )? (! mac_led_parallel_data_sfp[(sfp_port_num_1*3)+4]): 1'b1;		// SFP+ 1G OFF
		end
    endgenerate
	 // LED Data Mapping for SFP+ 29 ~ 32, 20160504
    generate 
      genvar  sfp_port_num_2;
      for ( sfp_port_num_2=4; sfp_port_num_2<8; sfp_port_num_2=sfp_port_num_2+1 )
      begin : MAC_LED_BIT_MAPPING_SFP_2
			 assign MAC_LED_SFP_G[sfp_port_num_2] = ( mac_led_parallel_data_sfp[((sfp_port_num_2*3)+5):((sfp_port_num_2*3)+4)] == 2'b00 )? (! mac_led_parallel_data_sfp[(sfp_port_num_2*3)+6]): 1'b1;	   // SFP+ 10G
			 assign MAC_LED_SFP_A[sfp_port_num_2] = ( mac_led_parallel_data_sfp[((sfp_port_num_2*3)+5):((sfp_port_num_2*3)+4)] == 2'b10 )? (! mac_led_parallel_data_sfp[(sfp_port_num_2*3)+6]): 1'b1;		// SFP+ 1G OFF
		end
    endgenerate
	 // LED Data Mapping for SFP+ 33 ~ 36, 20160504
    generate 
      genvar  sfp_port_num_3;
      for ( sfp_port_num_3=8; sfp_port_num_3<12; sfp_port_num_3=sfp_port_num_3+1 )
      begin : MAC_LED_BIT_MAPPING_SFP_3
          assign MAC_LED_SFP_G[sfp_port_num_3] = ( mac_led_parallel_data_sfp[((sfp_port_num_3*3)+7):((sfp_port_num_3*3)+6)] == 2'b00 )? (! mac_led_parallel_data_sfp[(sfp_port_num_3*3)+8]): 1'b1;	   // SFP+ 10G
			 assign MAC_LED_SFP_A[sfp_port_num_3] = ( mac_led_parallel_data_sfp[((sfp_port_num_3*3)+7):((sfp_port_num_3*3)+6)] == 2'b10 )? (! mac_led_parallel_data_sfp[(sfp_port_num_3*3)+8]): 1'b1;		// SFP+ 1G OFF
		end    
    endgenerate
	 // LED Data Mapping for SFP+ 37 ~ 40, 20160504
    generate 
      genvar  sfp_port_num_4;
      for ( sfp_port_num_4=12; sfp_port_num_4<16; sfp_port_num_4=sfp_port_num_4+1 )
      begin : MAC_LED_BIT_MAPPING_SFP_4
			 assign MAC_LED_SFP_G[sfp_port_num_4] = ( mac_led_parallel_data_sfp[((sfp_port_num_4*3)+9):((sfp_port_num_4*3)+8)] == 2'b00 )? (! mac_led_parallel_data_sfp[(sfp_port_num_4*3)+10]): 1'b1;	   // SFP+ 10G
			 assign MAC_LED_SFP_A[sfp_port_num_4] = ( mac_led_parallel_data_sfp[((sfp_port_num_4*3)+9):((sfp_port_num_4*3)+8)] == 2'b10 )? (! mac_led_parallel_data_sfp[(sfp_port_num_4*3)+10]): 1'b1;		// SFP+ 1G OFF
		end  
    endgenerate
	 // LED Data Mapping for SFP+ 41 ~ 44, 20160504
    generate 
      genvar  sfp_port_num_5;
      for ( sfp_port_num_5=16; sfp_port_num_5<20; sfp_port_num_5=sfp_port_num_5+1 )
      begin : MAC_LED_BIT_MAPPING_SFP_5
			 assign MAC_LED_SFP_G[sfp_port_num_5] = ( mac_led_parallel_data_sfp[((sfp_port_num_5*3)+11):((sfp_port_num_5*3)+10)] == 2'b00 )? (! mac_led_parallel_data_sfp[(sfp_port_num_5*3)+12]): 1'b1;	   // SFP+ 10G
			 assign MAC_LED_SFP_A[sfp_port_num_5] = ( mac_led_parallel_data_sfp[((sfp_port_num_5*3)+11):((sfp_port_num_5*3)+10)] == 2'b10 )? (! mac_led_parallel_data_sfp[(sfp_port_num_5*3)+12]): 1'b1;		// SFP+ 1G OFF
		end   
    endgenerate
	 // LED Data Mapping for SFP+ 45 ~ 48, 20160504
    generate 
      genvar  sfp_port_num_6;
      for ( sfp_port_num_6=20; sfp_port_num_6<24; sfp_port_num_6=sfp_port_num_6+1 )
      begin : MAC_LED_BIT_MAPPING_SFP_6
			 assign MAC_LED_SFP_G[sfp_port_num_6] = ( mac_led_parallel_data_sfp[((sfp_port_num_6*3)+13):((sfp_port_num_6*3)+12)] == 2'b00 )? (! mac_led_parallel_data_sfp[(sfp_port_num_6*3)+14]): 1'b1;	   // SFP+ 10G
			 assign MAC_LED_SFP_A[sfp_port_num_6] = ( mac_led_parallel_data_sfp[((sfp_port_num_6*3)+13):((sfp_port_num_6*3)+12)] == 2'b10 )? (! mac_led_parallel_data_sfp[(sfp_port_num_6*3)+14]): 1'b1;		// SFP+ 1G OFF
		end   
    endgenerate
	 
    generate 
      genvar  qsfp_port_num;
      for ( qsfp_port_num=0; qsfp_port_num<6; qsfp_port_num=qsfp_port_num+1 )
      begin : MAC_LED_BIT_MAPPING_QSFP
		  // Lane 0
		  assign MAC_LANE_CTRL[(4*qsfp_port_num)] = ( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+1):(qsfp_port_num*14)] == 2'b00 )?
		                                            (( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+3):((qsfp_port_num*14)+2)] == 2'b01 )? mac_led_parallel_data_qsfp[(qsfp_port_num*14)+4]:
		                                             (( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+3):((qsfp_port_num*14)+2)] == 2'b10 )? mac_led_parallel_data_qsfp[(qsfp_port_num*14)+4]:
		                                            1'b1)): 1'b1;
		  // Lane 1
        assign MAC_LANE_CTRL[(4*qsfp_port_num)+1] = ( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+1):(qsfp_port_num*14)] == 2'b00 )?
		                                            (( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+6):((qsfp_port_num*14)+5)] == 2'b01 )? mac_led_parallel_data_qsfp[(qsfp_port_num*14)+7]:
		                                             (( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+6):((qsfp_port_num*14)+5)] == 2'b10 )? mac_led_parallel_data_qsfp[(qsfp_port_num*14)+7]:
		                                            1'b1)): 1'b1;
		  // Lane 2
        assign MAC_LANE_CTRL[(4*qsfp_port_num)+2] = ( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+1):(qsfp_port_num*14)] == 2'b00 )?
		                                            (( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+9):((qsfp_port_num*14)+8)] == 2'b01 )? mac_led_parallel_data_qsfp[(qsfp_port_num*14)+10]:
		                                             (( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+9):((qsfp_port_num*14)+8)] == 2'b10 )? mac_led_parallel_data_qsfp[(qsfp_port_num*14)+10]:
		                                            1'b1)): 1'b1;
		  // Lane 3
        assign MAC_LANE_CTRL[(4*qsfp_port_num)+3] = ( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+1):(qsfp_port_num*14)] == 2'b00 )?
		                                            (( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+12):((qsfp_port_num*14)+11)] == 2'b01 )? mac_led_parallel_data_qsfp[(qsfp_port_num*14)+13]:
		                                             (( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+12):((qsfp_port_num*14)+11)] == 2'b10 )? mac_led_parallel_data_qsfp[(qsfp_port_num*14)+13]:
		                                            1'b1)): 1'b1;

		  // Determine the break out mode
		  assign MAC_PORT_CONFIG[(2*qsfp_port_num+1):(2*qsfp_port_num)] = ( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+1):(qsfp_port_num*14)] == 2'b00 )?
																			mac_led_parallel_data_qsfp[((qsfp_port_num*14)+3):((qsfp_port_num*14)+2)]: 2'b00;

        assign MAC_PORT_100G_LED_G[qsfp_port_num]  = ( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+1):(qsfp_port_num*14)] == 2'b01 )?
																			(( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+3):((qsfp_port_num*14)+2)] == 2'b01 )?
																			(! mac_led_parallel_data_qsfp[(qsfp_port_num*14)+4]): 1'b1): 1'b1;	   // QSFP28 100G;
        assign MAC_PORT_40G_LED_B[qsfp_port_num]   = ( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+1):(qsfp_port_num*14)] == 2'b01 )?
																			(( mac_led_parallel_data_qsfp[((qsfp_port_num*14)+3):((qsfp_port_num*14)+2)] == 2'b10 )?
																			(! mac_led_parallel_data_qsfp[(qsfp_port_num*14)+4]): 1'b1): 1'b1;	   // QSFP28 40G;
      end
    endgenerate
    // MAC LED Stream to Parallel output
    always @( posedge LED_CLK[0] or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        mac_led_parallel_data_qsfp <= 84'hF_FFFF_FFFF_FFFF_FFFF_FFFF;
      else
        mac_led_parallel_data_qsfp <= { LED_DATA[0], mac_led_parallel_data_qsfp[83: 1] };
    end
    always @( posedge LED_CLK[1] or negedge reset_n )
	 begin
      if ( reset_n == 1'b0 )
        mac_led_parallel_data_sfp <= 84'hF_FFFF_FFFF_FFFF_FFFF_FFFF;											// 20160503
      else
        mac_led_parallel_data_sfp <= { LED_DATA[1], mac_led_parallel_data_sfp[83: 1] };				// 20160503
    end

endmodule    
