//---------------------------------------------------------------------
//---------------------------------------------------------------------
// This is a LED Decode for LAN Port
// Vincent Huang, Galaxy TW
//
// History :
// 2016-02-01 Created
//---------------------------------------------------------------------
module port_led (
        // Clock signals
        input           MGMT_CLK,
        input           MGMT_RST_N,
        input  [ 1: 0]  REG_PORT_CONFIG,
		  input  [ 3: 0]  REG_LANE_CTRL,
        //
        input           REG_10G_BY4_LED_R,
        input           REG_10G_BY4_LED_G,
        input           REG_10G_BY4_LED_B,
        input           REG_25G_BY4_LED_R,
        input           REG_25G_BY4_LED_G,
        input           REG_25G_BY4_LED_B,
        // QSFP LED Port by REGFILE
        output          REG_PORT_LANE0_LED_R,
        output          REG_PORT_LANE0_LED_G,
        output          REG_PORT_LANE0_LED_B,
        output          REG_PORT_LANE1_LED_R,
        output          REG_PORT_LANE1_LED_G,
        output          REG_PORT_LANE1_LED_B,
        output          REG_PORT_LANE2_LED_R,
        output          REG_PORT_LANE2_LED_G,
        output          REG_PORT_LANE2_LED_B,
        output          REG_PORT_LANE3_LED_R,
        output          REG_PORT_LANE3_LED_G,
        output          REG_PORT_LANE3_LED_B
      );


    // Signals for this Design
    reg  [ 2: 0]  r_led_10g_by4_rgb;
    reg  [ 2: 0]  r_led_25g_by4_rgb;

    reg  [ 2: 0]  lane0_led_rgb_sig;
    reg  [ 2: 0]  lane1_led_rgb_sig;
    reg  [ 2: 0]  lane2_led_rgb_sig;
    reg  [ 2: 0]  lane3_led_rgb_sig;

    // Clock signals
    wire clock   = MGMT_CLK;
    wire reset_n = MGMT_RST_N;

    // Buffers
    assign { REG_PORT_LANE0_LED_R, REG_PORT_LANE0_LED_G, REG_PORT_LANE0_LED_B } = lane0_led_rgb_sig;
    assign { REG_PORT_LANE1_LED_R, REG_PORT_LANE1_LED_G, REG_PORT_LANE1_LED_B } = lane1_led_rgb_sig;
    assign { REG_PORT_LANE2_LED_R, REG_PORT_LANE2_LED_G, REG_PORT_LANE2_LED_B } = lane2_led_rgb_sig;
    assign { REG_PORT_LANE3_LED_R, REG_PORT_LANE3_LED_G, REG_PORT_LANE3_LED_B } = lane3_led_rgb_sig;

    // Inferred the Filop Flip
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_led_10g_by4_rgb <= 3'h7;
        r_led_25g_by4_rgb <= 3'h7;
      end
      else
      begin
        r_led_10g_by4_rgb <= { REG_10G_BY4_LED_R, REG_10G_BY4_LED_G, REG_10G_BY4_LED_B };
        r_led_25g_by4_rgb <= { REG_25G_BY4_LED_R, REG_25G_BY4_LED_G, REG_25G_BY4_LED_B };
      end
    end

    // LED Display Decode
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        lane0_led_rgb_sig = 3'h7;
        lane1_led_rgb_sig = 3'h7;
        lane2_led_rgb_sig = 3'h7;
        lane3_led_rgb_sig = 3'h7;
      end
      else
      begin
        case ( REG_PORT_CONFIG )
          2'b10   : begin
                      lane0_led_rgb_sig = ( REG_LANE_CTRL[0] == 1'b1 )? r_led_10g_by4_rgb: 3'h7;
                      lane1_led_rgb_sig = ( REG_LANE_CTRL[1] == 1'b1 )? r_led_10g_by4_rgb: 3'h7;
                      lane2_led_rgb_sig = ( REG_LANE_CTRL[2] == 1'b1 )? r_led_10g_by4_rgb: 3'h7;
                      lane3_led_rgb_sig = ( REG_LANE_CTRL[3] == 1'b1 )? r_led_10g_by4_rgb: 3'h7;
                    end
          2'b01   : begin
                      lane0_led_rgb_sig = ( REG_LANE_CTRL[0] == 1'b1 )? r_led_25g_by4_rgb: 3'h7;
                      lane1_led_rgb_sig = ( REG_LANE_CTRL[1] == 1'b1 )? r_led_25g_by4_rgb: 3'h7;
                      lane2_led_rgb_sig = ( REG_LANE_CTRL[2] == 1'b1 )? r_led_25g_by4_rgb: 3'h7;
                      lane3_led_rgb_sig = ( REG_LANE_CTRL[3] == 1'b1 )? r_led_25g_by4_rgb: 3'h7;
                    end
          default : begin
                      lane0_led_rgb_sig = 3'h7;
                      lane1_led_rgb_sig = 3'h7;
                      lane2_led_rgb_sig = 3'h7;
                      lane3_led_rgb_sig = 3'h7;
                    end
        endcase
      end
    end

endmodule
