//---------------------------------------------------------------------
//---------------------------------------------------------------------
// RTL Top File for Model ES5654BQ CPLD2
// Vincent Huang, Galaxy TW
//
// History :
// 2016-02-01 Created
//---------------------------------------------------------------------
`include "./es5654bq_cpld2_defined.vh"
`define I2C_S_DEVADDR 7'h62
//---------------------------------------------------------------------
module es5654bq_cpld2_top (
        input            CPLD2_25MHz,
        input            CPLD_RESET_N,
        output           CPLD2_INT,    // Reserved
        output           CPLD2_MOD_ABS,
        output           CPLD2_QSFP_INT,
        input            CPLD2_QSFP_RST,
        input            CPLD12_SYNC,
        output           CPLD21_SYNC,
        // I2C Bus signals
        input            I2C_0_SCL,
        inout            I2C_0_SDA,
        // QSFP signals
        input   [ 5: 0]  QSFP_INT_P,
        input   [ 5: 0]  QSFP_PRSNT_P,
        output  [ 5: 0]  QSFP_MODE_P,
        output  [ 5: 0]  QSFP_RSTN_P,
        // LED Stream
        input   [ 1: 0]  LED_CLK,
        input   [ 1: 0]  LED_DATA,
        // SFP+ Port 25 ~ Port 48
        input   [23: 0]  XSFP_MOD_ABS,
        input   [23: 0]  XSFP_RXLOS,
        output  [23: 0]  XSFP_TXDIS,
        input   [23: 0]  XSFP_TXFLT,
        output  [23: 0]  LED_SFP_A,
        output  [23: 0]  LED_SFP_G,
        // Port 49 ~ Port 54
        output  [ 5: 0]  PORT_100G_LED_G,
        output  [ 5: 0]  PORT_40G_LED_B,
        output  [ 5: 0]  PORT_LANE0_LED_R,
        output  [ 5: 0]  PORT_LANE0_LED_G,
        output  [ 5: 0]  PORT_LANE0_LED_B,
        output  [ 5: 0]  PORT_LANE1_LED_R,
        output  [ 5: 0]  PORT_LANE1_LED_G,
        output  [ 5: 0]  PORT_LANE1_LED_B,
        output  [ 5: 0]  PORT_LANE2_LED_R,
        output  [ 5: 0]  PORT_LANE2_LED_G,
        output  [ 5: 0]  PORT_LANE2_LED_B,
        output  [ 5: 0]  PORT_LANE3_LED_R,
        output  [ 5: 0]  PORT_LANE3_LED_G,
        output  [ 5: 0]  PORT_LANE3_LED_B
    );


    // signals for top
    wire           clk25;
    wire           global_rst_n;
    // signals for I2C Slave Instance
    wire           host_i2c_scl_in_sig;
    wire           host_i2c_sda_in_sig;
    wire           i2c_s_sda_sig_n;
    wire           i2c_read_sig;
    wire           i2c_read_done_sig;
    wire           i2c_write_sig;
    wire  [ 7: 0]  i2c_offset_addr_sig;
    wire  [ 7: 0]  i2c_offset_rdata_sig;
    wire  [ 7: 0]  i2c_offset_wdata_sig;
    // signals for Regfile Instance
    wire  [23: 0]  reg_led_sfp_a_sig;
    wire  [23: 0]  reg_led_sfp_g_sig;
    wire  [11: 0]  reg_port_config_sig;
    wire  [ 7: 0]  reg_port_10g_by4_led_pwm_r_sig;
    wire  [ 7: 0]  reg_port_10g_by4_led_pwm_g_sig;
    wire  [ 7: 0]  reg_port_10g_by4_led_pwm_b_sig;
    wire  [ 7: 0]  reg_port_25g_by4_led_pwm_r_sig;
    wire  [ 7: 0]  reg_port_25g_by4_led_pwm_g_sig;
    wire  [ 7: 0]  reg_port_25g_by4_led_pwm_b_sig;
    wire  [ 5: 0]  reg_port_100g_led_g_sig;
    wire  [ 5: 0]  reg_port_40g_led_b_sig;
    // signals for mac_led_encode Instance
    wire  [23: 0]  mac_led_sfp_a_sig;
    wire  [23: 0]  mac_led_sfp_g_sig;
    wire  [11: 0]  mac_port_config_sig;
	 wire  [23: 0]  mac_lane_ctrl_sig;
    wire  [ 5: 0]  mac_port_100g_led_g_sig;
    wire  [ 5: 0]  mac_port_40g_led_b_sig;
    // Signals for LED Display
    wire           reg_10g_by4_led_r_sig;
    wire           reg_10g_by4_led_g_sig;
    wire           reg_10g_by4_led_b_sig;
    wire           reg_25g_by4_led_r_sig;
    wire           reg_25g_by4_led_g_sig;
    wire           reg_25g_by4_led_b_sig;
	 wire  [83: 0]  led_dat0_sig;
	 
    wire  [ 5: 0]  reg_port_lane0_led_r_sig;
    wire  [ 5: 0]  reg_port_lane0_led_g_sig;
    wire  [ 5: 0]  reg_port_lane0_led_b_sig;
    wire  [ 5: 0]  reg_port_lane1_led_r_sig;
    wire  [ 5: 0]  reg_port_lane1_led_g_sig;
    wire  [ 5: 0]  reg_port_lane1_led_b_sig;
    wire  [ 5: 0]  reg_port_lane2_led_r_sig;
    wire  [ 5: 0]  reg_port_lane2_led_g_sig;
    wire  [ 5: 0]  reg_port_lane2_led_b_sig;
    wire  [ 5: 0]  reg_port_lane3_led_r_sig;
    wire  [ 5: 0]  reg_port_lane3_led_g_sig;
    wire  [ 5: 0]  reg_port_lane3_led_b_sig;
	 wire  [83: 0]  led_dat2_sig;
	 
	 wire   [11: 0]  PORT_CONFIG;
	 wire   [23: 0]  LANE_CONFIG;


    //-----------------------------------------------------------------
    // Buffer
    //-----------------------------------------------------------------
    assign clk25               = CPLD2_25MHz;
    assign global_rst_n        = CPLD_RESET_N;
    assign host_i2c_scl_in_sig = I2C_0_SCL;
    assign host_i2c_sda_in_sig = I2C_0_SDA;
    assign I2C_0_SDA           = ( !i2c_s_sda_sig_n ) ? 1'b0 : 1'bZ;
    assign CPLD2_INT           = 1'bz;
    // SFP+ Port 24 ~ Port 48
    assign LED_SFP_A           = reg_led_sfp_a_sig         & mac_led_sfp_a_sig;
    assign LED_SFP_G           = reg_led_sfp_g_sig         & mac_led_sfp_g_sig;
    // Port 49 ~ Port 54
    assign PORT_100G_LED_G     = reg_port_100g_led_g_sig   & mac_port_100g_led_g_sig;
    assign PORT_40G_LED_B      = reg_port_40g_led_b_sig    & mac_port_40g_led_b_sig;
    assign PORT_LANE0_LED_R    = reg_port_lane0_led_r_sig;
    assign PORT_LANE0_LED_G    = reg_port_lane0_led_g_sig;
    assign PORT_LANE0_LED_B    = reg_port_lane0_led_b_sig;
    assign PORT_LANE1_LED_R    = reg_port_lane1_led_r_sig;
    assign PORT_LANE1_LED_G    = reg_port_lane1_led_g_sig;
    assign PORT_LANE1_LED_B    = reg_port_lane1_led_b_sig;
    assign PORT_LANE2_LED_R    = reg_port_lane2_led_r_sig;
    assign PORT_LANE2_LED_G    = reg_port_lane2_led_g_sig;
    assign PORT_LANE2_LED_B    = reg_port_lane2_led_b_sig;
    assign PORT_LANE3_LED_R    = reg_port_lane3_led_r_sig;
    assign PORT_LANE3_LED_G    = reg_port_lane3_led_g_sig;
    assign PORT_LANE3_LED_B    = reg_port_lane3_led_b_sig;
	 assign PORT_CONFIG         = reg_port_config_sig       | mac_port_config_sig;
	 assign LANE_CONFIG         = mac_lane_ctrl_sig;

    //-----------------------------------------------------------------
    // Call a Instance of I2C Slave
    //-----------------------------------------------------------------
    i2c_s #(
        .I2C_DevADDR      ( `I2C_S_DEVADDR         ),
        .Clock_Frequency  ( 50                     )
        ) I2C_SLAVE_I     (
        .clk              ( clk25                  ),
        .reset_n          ( global_rst_n           ),
        .soft_reset_n     ( 1'b1                   ),
        .scl_i            ( host_i2c_scl_in_sig    ),
        .sda_i            ( host_i2c_sda_in_sig    ),
        .sda_oe_n         ( i2c_s_sda_sig_n        ),
        .clk_stretch_n    ( 1'b1                   ),
        .readdata         ( i2c_offset_rdata_sig   ),
        .writedata        ( i2c_offset_wdata_sig   ),
        .address          ( i2c_offset_addr_sig    ),
        .read             ( i2c_read_sig           ),
        .read_done        ( i2c_read_done_sig      ),
        .write            ( i2c_write_sig          )
        );


    //-----------------------------------------------------------------
    // Call a Instance of LED Display
    //-----------------------------------------------------------------
    led_display LED_DISPLAY_I (
        // Clock signals
        .MGMT_CLK                     ( clk25                          ),
        .MGMT_RST_N                   ( global_rst_n                   ),
        // QSFP PWM
        .REG_PORT_10G_BY4_LED_PWM_R   ( reg_port_10g_by4_led_pwm_r_sig ),
        .REG_PORT_10G_BY4_LED_PWM_G   ( reg_port_10g_by4_led_pwm_g_sig ),
        .REG_PORT_10G_BY4_LED_PWM_B   ( reg_port_10g_by4_led_pwm_b_sig ),
        .REG_PORT_25G_BY4_LED_PWM_R   ( reg_port_25g_by4_led_pwm_r_sig ),
        .REG_PORT_25G_BY4_LED_PWM_G   ( reg_port_25g_by4_led_pwm_g_sig ),
        .REG_PORT_25G_BY4_LED_PWM_B   ( reg_port_25g_by4_led_pwm_b_sig ),
        //
        .REG_10G_BY4_LED_R            ( reg_10g_by4_led_r_sig          ),
        .REG_10G_BY4_LED_G            ( reg_10g_by4_led_g_sig          ),
        .REG_10G_BY4_LED_B            ( reg_10g_by4_led_b_sig          ),
        .REG_25G_BY4_LED_R            ( reg_25g_by4_led_r_sig          ),
        .REG_25G_BY4_LED_G            ( reg_25g_by4_led_g_sig          ),
        .REG_25G_BY4_LED_B            ( reg_25g_by4_led_b_sig          )
      );


    //-----------------------------------------------------------------
    // Call a Instance of MAC LED Encode
    //-----------------------------------------------------------------
    mac_led_encode MAC_LED_ENCODE_I (
        // Clock signals
        .MGMT_CLK               ( clk25                    ),
        .MGMT_RST_N             ( global_rst_n             ),
		  // Determine 4x10G or 4x25G
        .MAC_PORT_CONFIG		  ( mac_port_config_sig		  ),
		  .MAC_LANE_CTRL		     ( mac_lane_ctrl_sig		  ),
        // LED  Stream
        .LED_CLK                ( LED_CLK                  ),
        .LED_DATA               ( LED_DATA                 ),
        // SFP+ Port 24 ~ Port 48
        .MAC_LED_SFP_A          ( mac_led_sfp_a_sig        ),
        .MAC_LED_SFP_G          ( mac_led_sfp_g_sig        ),
        // QSFP Port 49 ~ Port 54
        .MAC_PORT_100G_LED_G    ( mac_port_100g_led_g_sig  ),
        .MAC_PORT_40G_LED_B     ( mac_port_40g_led_b_sig   ),
		  .LED_DAT0_STREAM        ( led_dat0_sig      ),
		  .LED_DAT2_STREAM        ( led_dat2_sig      )
      );


    //-----------------------------------------------------------------
    // Call a Instance of Port LED
    //-----------------------------------------------------------------
    generate
      genvar  port_num;
      for ( port_num=0; port_num<6; port_num=port_num+1 )
      begin : PORT_LED_LOOP
        port_led LED_DISPLAY_PORT (
                // Clock signals
                .MGMT_CLK             ( clk25                              ),
                .MGMT_RST_N           ( global_rst_n                       ),
					 // Determine 4x10G or 4x25G
                .REG_PORT_CONFIG      ( PORT_CONFIG[(2*port_num+1):(2*port_num)] ),
					 .REG_LANE_CTRL        ( LANE_CONFIG[(4*port_num+3):(4*port_num)] ),
                //
                .REG_10G_BY4_LED_R    ( reg_10g_by4_led_r_sig              ),
                .REG_10G_BY4_LED_G    ( reg_10g_by4_led_g_sig              ),
                .REG_10G_BY4_LED_B    ( reg_10g_by4_led_b_sig              ),
                .REG_25G_BY4_LED_R    ( reg_25g_by4_led_r_sig              ),
                .REG_25G_BY4_LED_G    ( reg_25g_by4_led_g_sig              ),
                .REG_25G_BY4_LED_B    ( reg_25g_by4_led_b_sig              ),
                //
                .REG_PORT_LANE0_LED_R ( reg_port_lane0_led_r_sig[port_num] ),
                .REG_PORT_LANE0_LED_G ( reg_port_lane0_led_g_sig[port_num] ),
                .REG_PORT_LANE0_LED_B ( reg_port_lane0_led_b_sig[port_num] ),
                .REG_PORT_LANE1_LED_R ( reg_port_lane1_led_r_sig[port_num] ),
                .REG_PORT_LANE1_LED_G ( reg_port_lane1_led_g_sig[port_num] ),
                .REG_PORT_LANE1_LED_B ( reg_port_lane1_led_b_sig[port_num] ),
                .REG_PORT_LANE2_LED_R ( reg_port_lane2_led_r_sig[port_num] ),
                .REG_PORT_LANE2_LED_G ( reg_port_lane2_led_g_sig[port_num] ),
                .REG_PORT_LANE2_LED_B ( reg_port_lane2_led_b_sig[port_num] ),
                .REG_PORT_LANE3_LED_R ( reg_port_lane3_led_r_sig[port_num] ),
                .REG_PORT_LANE3_LED_G ( reg_port_lane3_led_g_sig[port_num] ),
                .REG_PORT_LANE3_LED_B ( reg_port_lane3_led_b_sig[port_num] )
              );
      end
    endgenerate

    //-----------------------------------------------------------------
    // Call a Instance of Register Files
    //-----------------------------------------------------------------
    regfile REGFILES_I (
        // Clock signals
        .MGMT_CLK                     ( clk25                          ),
        .MGMT_RST_N                   ( global_rst_n                   ),
        // I2C signals
        .I2C_READ                     ( i2c_read_sig                   ),
        .I2C_READ_DONE                ( i2c_read_done_sig              ),
        .I2C_WRITE                    ( i2c_write_sig                  ),
        .I2C_OFFSET_ADDR              ( i2c_offset_addr_sig            ),
        .I2C_OFFSET_WDATA             ( i2c_offset_wdata_sig           ),
        .I2C_OFFSET_RDATA             ( i2c_offset_rdata_sig           ),
        // Register 0x02 --  Read & Write
        .CPLD2_MOD_ABS                ( CPLD2_MOD_ABS                  ),
        .CPLD2_QSFP_INT               ( CPLD2_QSFP_INT                 ),
        // Register 0x03 --  Read
        .CPLD2_QSFP_RST               ( CPLD2_QSFP_RST                 ),
        // XCVR signals
        // Register 0x04 ~ 0x12  --  Read
        .XSFP_MOD_ABS                 ( XSFP_MOD_ABS                   ),
        // Register 0x14 ~ 0x19  --  Read
        .XSFP_TXFLT                   ( XSFP_TXFLT                     ),
        // Register 0x20 ~ 0x25  --  Read & Write
        .XSFP_TXDIS                   ( XSFP_TXDIS                     ),
        // Register 0x30 ~ 0x35  --  Read
        .XSFP_RXLOS                   ( XSFP_RXLOS                     ),
        // Register 0x40 ~ 0x45  --  Read & Write
        .REG_LED_SFP_A                ( reg_led_sfp_a_sig              ),
        .REG_LED_SFP_G                ( reg_led_sfp_g_sig              ),
        // QSFP signals
        // Register 0x50 -- Read & Write
        .QSFP_INT_P                   ( QSFP_INT_P                     ),
        // Register 0x52 -- Read
        .QSFP_PRSNT_P                 ( QSFP_PRSNT_P                   ),
        // Register 0x53 -- Read & Write
        .QSFP_RSTN_P                  ( QSFP_RSTN_P                    ),
        // Register 0x54 -- Read & Write
        .QSFP_MODE_P                  ( QSFP_MODE_P                    ),
        // Register 0x60 ~ 0x67 -- Read & Write
        .REG_PORT_10G_BY4_LED_PWM_R   ( reg_port_10g_by4_led_pwm_r_sig ),
        .REG_PORT_10G_BY4_LED_PWM_G   ( reg_port_10g_by4_led_pwm_g_sig ),
        .REG_PORT_10G_BY4_LED_PWM_B   ( reg_port_10g_by4_led_pwm_b_sig ),
        .REG_PORT_25G_BY4_LED_PWM_R   ( reg_port_25g_by4_led_pwm_r_sig ),
        .REG_PORT_25G_BY4_LED_PWM_G   ( reg_port_25g_by4_led_pwm_g_sig ),
        .REG_PORT_25G_BY4_LED_PWM_B   ( reg_port_25g_by4_led_pwm_b_sig ),
        .REG_PORT_100G_LED_G          ( reg_port_100g_led_g_sig        ),
        .REG_PORT_40G_LED_B           ( reg_port_40g_led_b_sig         ),
        // Register 0x68 -- Read & Write
		  .REG_port_config   			  ( reg_port_config_sig        	  ),
        .CPLD12_SYNC                  ( CPLD12_SYNC                    ),
        .CPLD21_SYNC                  ( CPLD21_SYNC                    ),
		  // Register 0x71~ -- Read
		  .LED0_STREAM                  ( led_dat0_sig                   ),
		  // Register 0x91~ -- Read
		  .LED2_STREAM                  ( led_dat2_sig                   )
      );


endmodule
