//---------------------------------------------------------------------
//---------------------------------------------------------------------
// Offser Register Files for ES5654BQ CPLD2
// Vincent Huang, Galaxy TW
//
// History :
// 2016-02-01 Created
//---------------------------------------------------------------------
`include "./es5654bq_cpld2_defined.vh"
//---------------------------------------------------------------------
module regfile (
        // Clock signals
        input            MGMT_CLK,
        input            MGMT_RST_N,
        // I2C signals
        input            I2C_READ,
        input            I2C_READ_DONE,
        input            I2C_WRITE,
        input   [ 7: 0]  I2C_OFFSET_ADDR,
        input   [ 7: 0]  I2C_OFFSET_WDATA,
        output  [ 7: 0]  I2C_OFFSET_RDATA,
        // Register 0x02  --  Read & Write
        output           CPLD2_MOD_ABS,
        output           CPLD2_QSFP_INT,
        // Register 0x03 --  Read
        input            CPLD2_QSFP_RST,
        // XCVR signals
        // Register 0x04 ~ 0x12  --  Read
        input   [23: 0]  XSFP_MOD_ABS,
        // Register 0x14 ~ 0x19  --  Read
        input   [23: 0]  XSFP_TXFLT,
        // Register 0x20 ~ 0x25  --  Read & Write
        output  [23: 0]  XSFP_TXDIS,
        // Register 0x30 ~ 0x35  --  Read
        input   [23: 0]  XSFP_RXLOS,
        // Register 0x40 ~ 0x45  --  Read & Write
        output  [23: 0]  REG_LED_SFP_A,
        output  [23: 0]  REG_LED_SFP_G,
        // QSFP signals
        // Register 0x50 -- Read & Write
        input   [ 5: 0]  QSFP_INT_P,
        // Register 0x52 -- Read
        input   [ 5: 0]  QSFP_PRSNT_P,
        // Register 0x53 -- Read & Write
        output  [ 5: 0]  QSFP_RSTN_P,
        // Register 0x54 -- Read & Write
        output  [ 5: 0]  QSFP_MODE_P,
        // Register 0x60 ~ 0x67 -- Read & Write
        output  [ 7: 0]  REG_PORT_10G_BY4_LED_PWM_R,
        output  [ 7: 0]  REG_PORT_10G_BY4_LED_PWM_G,
        output  [ 7: 0]  REG_PORT_10G_BY4_LED_PWM_B,
        output  [ 7: 0]  REG_PORT_25G_BY4_LED_PWM_R,
        output  [ 7: 0]  REG_PORT_25G_BY4_LED_PWM_G,
        output  [ 7: 0]  REG_PORT_25G_BY4_LED_PWM_B,
        output  [ 5: 0]  REG_PORT_100G_LED_G,
        output  [ 5: 0]  REG_PORT_40G_LED_B,
        // Register 0x68 -- Read & Write
        input            CPLD12_SYNC,
        output           CPLD21_SYNC,
		  // Register 0x69 -- Read & Write
		  output  [ 11: 0]	 REG_port_config,
		  // Register 0x71~ -- Read
		  input   [83: 0]  LED0_STREAM,
		  // Register 0x91~ -- Read
		  input   [83: 0]  LED2_STREAM
      );

    // Signals for I2C Bus
    reg  [ 7: 0]  r_i2c_readdata;
    reg           r_cpld2_mod_abs_mask;
    reg           r_cpld2_qsfp_int_mask;
    // Signals for SPF+ Transciver
    reg  [23: 0]  r_xsfp_int_24b;
    reg  [23: 0]  r_xsfp_int_mask_24b;
    reg  [23: 0]  r_xsfp_prsnt_24b;
    reg  [23: 0]  r_xsfp_tx_fault_24b;
    reg  [23: 0]  r_xsfp_tx_fault_mask_24b;
    reg  [23: 0]  r_xsfp_tx_dis_24b;
    reg  [23: 0]  r_xsfp_tx_dis_mask_24b;
    reg  [23: 0]  r_xsfp_rx_loss_24b;
    reg  [23: 0]  r_xsfp_rx_loss_mask_24b;
    // Signals for QSFP Transciver
    wire [31: 0]  xcvr_prsnt_rise_sig;
    wire [31: 0]  xcvr_prsnt_fall_sig;
    reg  [ 5: 0]  r_qsfp_mod_int_6b;
    reg  [ 5: 0]  r_qsfp_mod_int_mask_6b;
    reg  [ 5: 0]  r_qsfp_prsnt_6b;
    reg  [ 5: 0]  r_qsfp_mod_rst_6b;
    reg  [ 5: 0]  r_qsfp_lpmode_6b;
    // Signals for LED PWM
    reg  [23: 0]  r_sfp_led_amber_24b;
    reg  [23: 0]  r_sfp_led_green_24b;
    reg  [ 7: 0]  r_10g_by4_led_b_pwm_8b;
    reg  [ 7: 0]  r_10g_by4_led_g_pwm_8b;
    reg  [ 7: 0]  r_10g_by4_led_r_pwm_8b;
    reg  [ 7: 0]  r_25g_by4_led_b_pwm_8b;
    reg  [ 7: 0]  r_25g_by4_led_g_pwm_8b;
    reg  [ 7: 0]  r_25g_by4_led_r_pwm_8b;
    reg  [ 5: 0]  r_100g_led_green_6b;
    reg  [ 5: 0]  r_40g_led_blue_6b;
	 reg  [ 11: 0]  r_port_config;					// Test for PWM LED
	 // Signals for LED_0 stream
	 wire  [83: 0]  r_led0_stream;
	 // Signals for LED_2 stream
	 wire  [83: 0]  r_led2_stream;

    // Clock Signals
    wire clock   = MGMT_CLK;
    wire reset_n = MGMT_RST_N;
    //
    assign CPLD2_MOD_ABS                = ( &r_xsfp_int_24b    ) | r_cpld2_mod_abs_mask;
    assign CPLD2_QSFP_INT               = ( &r_qsfp_mod_int_6b ) | r_cpld2_qsfp_int_mask;
    // Buffers for SPF+ Transciver
    assign XSFP_INT                     = r_xsfp_int_24b;
    assign XSFP_TXDIS                   = r_xsfp_tx_dis_24b;
    // Buffers for QSFP Transciver
    assign QSFP_RSTN_P                  = { 6{CPLD2_QSFP_RST} } & r_qsfp_mod_rst_6b & { 6{reset_n} };
    assign QSFP_MODE_P                  = r_qsfp_lpmode_6b;
    // Buffers for LED Control
    assign REG_LED_SFP_A                = r_sfp_led_amber_24b;
    assign REG_LED_SFP_G                = r_sfp_led_green_24b;
    assign REG_PORT_10G_BY4_LED_PWM_R   = r_10g_by4_led_b_pwm_8b;
    assign REG_PORT_10G_BY4_LED_PWM_G   = r_10g_by4_led_g_pwm_8b;
    assign REG_PORT_10G_BY4_LED_PWM_B   = r_10g_by4_led_r_pwm_8b;
    assign REG_PORT_25G_BY4_LED_PWM_R   = r_25g_by4_led_b_pwm_8b;
    assign REG_PORT_25G_BY4_LED_PWM_G   = r_25g_by4_led_g_pwm_8b;
    assign REG_PORT_25G_BY4_LED_PWM_B   = r_25g_by4_led_r_pwm_8b;
    assign REG_PORT_100G_LED_G          = r_100g_led_green_6b;
    assign REG_PORT_40G_LED_B           = r_40g_led_blue_6b;
	 assign REG_port_config = r_port_config;
	 
	 // Read stream from MAC LED_DATA0
	 assign r_led0_stream = LED0_STREAM;
	 
	 // Read stream from MAC LED_DATA2
	 assign r_led2_stream = LED2_STREAM;


    //----------------------------------------------------------------------
    // SFP+ PLUGGABLE Signals
    //----------------------------------------------------------------------
    generate
      genvar xsfp_int_1_loop;
      for ( xsfp_int_1_loop=0; xsfp_int_1_loop<8; xsfp_int_1_loop=xsfp_int_1_loop+1 )
      begin : XSFP_INT_1_PROC
        always@ ( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_xsfp_int_24b[xsfp_int_1_loop] <= 1'b1;
          else if ( r_xsfp_int_mask_24b[xsfp_int_1_loop] == 1'b0 )
          begin
            if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_INT_4 )    // 0x04 -- IOPLD_REG_XSFP_INT_4
              r_xsfp_int_24b[xsfp_int_1_loop] <= I2C_OFFSET_WDATA[xsfp_int_1_loop];
            else
              r_xsfp_int_24b[xsfp_int_1_loop] <= XSFP_MOD_ABS[xsfp_int_1_loop];
          end
          else
            r_xsfp_int_24b[xsfp_int_1_loop] <= 1'b1;
        end
      end
    endgenerate
    generate
      genvar xsfp_int_2_loop;
      for ( xsfp_int_2_loop=8; xsfp_int_2_loop<16; xsfp_int_2_loop=xsfp_int_2_loop+1 )
      begin : XSFP_INT_2_PROC
        always@ ( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_xsfp_int_24b[xsfp_int_2_loop] <= 1'b1;
          else if ( r_xsfp_int_mask_24b[xsfp_int_2_loop] == 1'b0 )
          begin
            if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_INT_5 )    // 0x06 -- IOPLD_REG_XSFP_INT_5
              r_xsfp_int_24b[xsfp_int_2_loop] <= I2C_OFFSET_WDATA[xsfp_int_2_loop-8];
            else
              r_xsfp_int_24b[xsfp_int_2_loop] <= XSFP_MOD_ABS[xsfp_int_2_loop];
          end
          else
            r_xsfp_int_24b[xsfp_int_2_loop] <= 1'b1;
        end
      end
    endgenerate
    generate
      genvar xsfp_int_3_loop;
      for ( xsfp_int_3_loop=16; xsfp_int_3_loop<`XSFP_NUM; xsfp_int_3_loop=xsfp_int_3_loop+1 )
      begin : XSFP_INT_3_PROC
        always@ ( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_xsfp_int_24b[xsfp_int_3_loop] <= 1'b1;
          else if ( r_xsfp_int_mask_24b[xsfp_int_3_loop] == 1'b0 )
          begin
            if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_INT_6 )    // 0x08 -- IOPLD_REG_XSFP_INT_6
              r_xsfp_int_24b[xsfp_int_3_loop] <= I2C_OFFSET_WDATA[xsfp_int_3_loop-16];
            else
              r_xsfp_int_24b[xsfp_int_3_loop] <= XSFP_MOD_ABS[xsfp_int_3_loop];
          end
          else
            r_xsfp_int_24b[xsfp_int_3_loop] <= 1'b1;
        end
      end
    endgenerate

    // Detected SFP+ Present
    generate
      genvar xsfp_prsnt_loop;
      for ( xsfp_prsnt_loop=0; xsfp_prsnt_loop<`XSFP_NUM; xsfp_prsnt_loop=xsfp_prsnt_loop+1 )
      begin : XSFP_PRSNT_1_PROC
        always@ ( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_xsfp_prsnt_24b[xsfp_prsnt_loop] <= 1'b1;
          else
            r_xsfp_prsnt_24b[xsfp_prsnt_loop] <= XSFP_MOD_ABS[xsfp_prsnt_loop];
        end
      end
    endgenerate

    // Detected SFP+ TX Fault
    generate
      genvar xsfp_txflt_loop;
      for ( xsfp_txflt_loop=0; xsfp_txflt_loop<`XSFP_NUM; xsfp_txflt_loop=xsfp_txflt_loop+1 )
      begin : XSFP_TXFLT_PROC
        always@ ( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_xsfp_tx_fault_24b[xsfp_txflt_loop] <= 1'b1;
          else if ( r_xsfp_tx_fault_mask_24b[xsfp_txflt_loop] == 1'b0 )
            r_xsfp_tx_fault_24b[xsfp_txflt_loop] <= XSFP_TXFLT[xsfp_txflt_loop];
          else
            r_xsfp_tx_fault_24b[xsfp_txflt_loop] <= 1'b1;
        end
      end
    endgenerate

    // Detected SFP+ RX LOSS
    generate
      genvar xsfp_rxlos_loop;
      for ( xsfp_rxlos_loop=0; xsfp_rxlos_loop<`XSFP_NUM; xsfp_rxlos_loop=xsfp_rxlos_loop+1 )
      begin : XSFP_RXLOS_PROC
        always@ ( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_xsfp_rx_loss_24b[xsfp_rxlos_loop] <= 1'b1;
          else if ( r_xsfp_rx_loss_mask_24b[xsfp_rxlos_loop] == 1'b0 )
            r_xsfp_rx_loss_24b[xsfp_rxlos_loop] <= XSFP_RXLOS[xsfp_rxlos_loop];
          else
            r_xsfp_rx_loss_24b[xsfp_rxlos_loop] <= 1'b1;
        end
      end
    endgenerate


    //-----------------------------------------------------------------
    // Control Statement for QSFP
    //-----------------------------------------------------------------
    generate
      genvar zqsfp_int_loop;
      for ( zqsfp_int_loop=0; zqsfp_int_loop<`ZQSFP_NUM; zqsfp_int_loop=zqsfp_int_loop+1 )
      begin : ZQSFP_CTRL_PROC
        asyncsig_edge_detect EDGE_DET ( QSFP_PRSNT_P[zqsfp_int_loop], clock, xcvr_prsnt_rise_sig[zqsfp_int_loop], xcvr_prsnt_fall_sig[zqsfp_int_loop] );
        // INT Proc. for XCVR
        always @( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_qsfp_mod_int_6b[zqsfp_int_loop] <= 1'b1;
          else if (r_qsfp_mod_int_mask_6b[zqsfp_int_loop] == 1'b0 )
          begin
            if ( xcvr_prsnt_rise_sig[zqsfp_int_loop] == 1'b1 || xcvr_prsnt_fall_sig[zqsfp_int_loop] == 1'b1 || QSFP_INT_P[zqsfp_int_loop] == 1'b0 )
              r_qsfp_mod_int_6b[zqsfp_int_loop] <= 1'b0;
            else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_ZQSFP_MOD_INT )    // 0x50 -- IOPLD_REG_ZQSFP_MOD_INT
              r_qsfp_mod_int_6b[zqsfp_int_loop] <= I2C_OFFSET_WDATA[zqsfp_int_loop];
            else if ( I2C_READ_DONE == 1'b1 )
              r_qsfp_mod_int_6b[zqsfp_int_loop] <= 1'b1;
            else
              r_qsfp_mod_int_6b[zqsfp_int_loop] <= 1'b1;
          end
        end
      end
    endgenerate

    generate
      genvar zqsfp_prsnt_loop;
      for ( zqsfp_prsnt_loop=0; zqsfp_prsnt_loop<`ZQSFP_NUM; zqsfp_prsnt_loop=zqsfp_prsnt_loop+1 )
      begin : ZQSFP_PRSNT_PROC
        always@ ( posedge clock or negedge reset_n )
        begin
          if ( reset_n == 1'b0 )
            r_qsfp_prsnt_6b[zqsfp_prsnt_loop] <= 1'b1;
          else
            r_qsfp_prsnt_6b[zqsfp_prsnt_loop] <= QSFP_PRSNT_P[zqsfp_prsnt_loop];
        end
      end
    endgenerate


    //-----------------------------------------------------------------
    // Write Operation Statement by I2C Host
    //-----------------------------------------------------------------
    // 0x02 -- IOPLD_REG_INT_STATUS_MASK
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_cpld2_mod_abs_mask  <= 1'b0;
        r_cpld2_qsfp_int_mask <= 1'b0;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_INT_STATUS_MASK )
      begin
        r_cpld2_mod_abs_mask  <= I2C_OFFSET_WDATA[1];
        r_cpld2_qsfp_int_mask <= I2C_OFFSET_WDATA[0];
      end
    end

    // 0x05 -- IOPLD_REG_XSFP_INT_MASK_4
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_int_mask_24b[7:0] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_INT_MASK_4 )
        r_xsfp_int_mask_24b[7:0] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x07 -- IOPLD_REG_XSFP_INT_MASK_5
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_int_mask_24b[15:8] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_INT_MASK_5 )
        r_xsfp_int_mask_24b[15:8] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x09 -- IOPLD_REG_XSFP_INT_MASK_6
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_int_mask_24b[23:16] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_INT_MASK_6 )
        r_xsfp_int_mask_24b[23:16] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x15 -- IOPLD_REG_XSFP_TX_FAULT_MASK_4
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_fault_mask_24b[7:0] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_TX_FAULT_MASK_4 )
        r_xsfp_tx_fault_mask_24b[7:0] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x17 -- IOPLD_REG_XSFP_TX_FAULT_MASK_5
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_fault_mask_24b[15:8] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_TX_FAULT_MASK_5 )
        r_xsfp_tx_fault_mask_24b[15:8] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x19 -- IOPLD_REG_XSFP_TX_FAULT_MASK_6
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_fault_mask_24b[23:16] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_TX_FAULT_MASK_6 )
        r_xsfp_tx_fault_mask_24b[23:16] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x20 -- IOPLD_REG_XSFP_TX_DIS_4
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_dis_24b[7:0] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_TX_DIS_4 )
        r_xsfp_tx_dis_24b[7:0] <= I2C_OFFSET_WDATA[7:0] & ( ~r_xsfp_tx_dis_mask_24b[7:0] );
    end

    // 0x21 -- IOPLD_REG_XSFP_TX_DIS_MASK_4
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_dis_mask_24b[7:0] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_TX_DIS_MASK_4 )
        r_xsfp_tx_dis_mask_24b[7:0] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x22 -- IOPLD_REG_XSFP_TX_DIS_5
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_dis_24b[15:8] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_TX_DIS_5 )
        r_xsfp_tx_dis_24b[15:8] <= I2C_OFFSET_WDATA[7:0] & ( ~r_xsfp_tx_dis_mask_24b[15:8] );
    end

    // 0x23 -- IOPLD_REG_XSFP_TX_DIS_MASK_5
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_dis_mask_24b[15:8] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_TX_DIS_MASK_5 )
        r_xsfp_tx_dis_mask_24b[15:8] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x24 -- IOPLD_REG_XSFP_TX_DIS_6
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_dis_24b[23:16] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_TX_DIS_6 )
        r_xsfp_tx_dis_24b[23:16] <= I2C_OFFSET_WDATA[7:0] & ( ~r_xsfp_tx_dis_mask_24b[23:16] );
    end

    // 0x25 -- IOPLD_REG_XSFP_TX_DIS_MASK_6
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_tx_dis_mask_24b[23:16] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_TX_DIS_MASK_6 )
        r_xsfp_tx_dis_mask_24b[23:16] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x31 -- IOPLD_REG_XSFP_RX_LOS_MASK_4
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_rx_loss_mask_24b[7:0] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_RX_LOS_MASK_4 )
        r_xsfp_rx_loss_mask_24b[7:0] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x33 -- IOPLD_REG_XSFP_RX_LOS_MASK_5
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_rx_loss_mask_24b[15:8] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_RX_LOS_MASK_5 )
        r_xsfp_rx_loss_mask_24b[15:8] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x35 -- IOPLD_REG_XSFP_RX_LOS_MASK_6
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_xsfp_rx_loss_mask_24b[23:16] <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_XSFP_RX_LOS_MASK_6 )
        r_xsfp_rx_loss_mask_24b[23:16] <= I2C_OFFSET_WDATA[7:0];
    end

    // 0x40 -- IOPLD_REG_1G_10G_LED_7
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_sfp_led_amber_24b[3:0] <= 4'hF;
        r_sfp_led_green_24b[3:0] <= 4'hF;			// 20160504
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_1G_10G_LED_7 )
      begin
        r_sfp_led_amber_24b[3:0] <= { I2C_OFFSET_WDATA[1], I2C_OFFSET_WDATA[3], I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_sfp_led_green_24b[3:0] <= { I2C_OFFSET_WDATA[0], I2C_OFFSET_WDATA[2], I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end

    // 0x41 -- IOPLD_REG_1G_10G_LED_8
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_sfp_led_amber_24b[7:4] <= 4'hF;
        r_sfp_led_green_24b[7:4] <= 4'hF;			// 20160504
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_1G_10G_LED_8 )
      begin
        r_sfp_led_amber_24b[7:4] <= { I2C_OFFSET_WDATA[1], I2C_OFFSET_WDATA[3], I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_sfp_led_green_24b[7:4] <= { I2C_OFFSET_WDATA[0], I2C_OFFSET_WDATA[2], I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end

    // 0x42 -- IOPLD_REG_1G_10G_LED_9
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_sfp_led_amber_24b[11:8] <= 4'hF;
        r_sfp_led_green_24b[11:8] <= 4'hF;			// 20160504
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_1G_10G_LED_9 )
      begin
        r_sfp_led_amber_24b[11:8] <= { I2C_OFFSET_WDATA[1], I2C_OFFSET_WDATA[3], I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_sfp_led_green_24b[11:8] <= { I2C_OFFSET_WDATA[0], I2C_OFFSET_WDATA[2], I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end

    // 0x43 -- IOPLD_REG_1G_10G_LED_10
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_sfp_led_amber_24b[15:12] <= 4'hF;
        r_sfp_led_green_24b[15:12] <= 4'hF;			// 20160504
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_1G_10G_LED_10 )
      begin
        r_sfp_led_amber_24b[15:12] <= { I2C_OFFSET_WDATA[1], I2C_OFFSET_WDATA[3], I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_sfp_led_green_24b[15:12] <= { I2C_OFFSET_WDATA[0], I2C_OFFSET_WDATA[2], I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end

    // 0x44 -- IOPLD_REG_1G_10G_LED_11
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_sfp_led_amber_24b[19:16] <= 4'hF;
        r_sfp_led_green_24b[19:16] <= 4'hF;			// 20160504
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_1G_10G_LED_11 )
      begin
        r_sfp_led_amber_24b[19:16] <= { I2C_OFFSET_WDATA[1], I2C_OFFSET_WDATA[3], I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_sfp_led_green_24b[19:16] <= { I2C_OFFSET_WDATA[0], I2C_OFFSET_WDATA[2], I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end

    // 0x45 -- IOPLD_REG_1G_10G_LED_12
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_sfp_led_amber_24b[23:20] <= 4'hF;
        r_sfp_led_green_24b[23:20] <= 4'hF;			// 20160504
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_1G_10G_LED_12 )
      begin
        r_sfp_led_amber_24b[23:20] <= { I2C_OFFSET_WDATA[1], I2C_OFFSET_WDATA[3], I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_sfp_led_green_24b[23:20] <= { I2C_OFFSET_WDATA[0], I2C_OFFSET_WDATA[2], I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end

    // 0x51 -- IOPLD_REG_ZQSFP_MOD_INT_MASK
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_qsfp_mod_int_mask_6b[5:0] <= 6'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_ZQSFP_MOD_INT_MASK )
        r_qsfp_mod_int_mask_6b[5:0] <= I2C_OFFSET_WDATA[5:0];
    end

    // 0x53 -- IOPLD_REG_ZQSFP_MOD_RST
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_qsfp_mod_rst_6b[5:0] <= 6'h3F;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_ZQSFP_MOD_RST )
        r_qsfp_mod_rst_6b[5:0] <= I2C_OFFSET_WDATA[5:0];
    end

    // 0x54 -- IOPLD_REG_ZQSFP_LPMODE
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_qsfp_lpmode_6b[5:0] <= 6'h3F;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_ZQSFP_LPMODE )
        r_qsfp_lpmode_6b[5:0] <= I2C_OFFSET_WDATA[5:0];
    end

    // 0x60 -- IOPLD_REG_10G_BY4_LED_B
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_10g_by4_led_b_pwm_8b <= 8'h20;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_10G_BY4_LED_B )
        r_10g_by4_led_b_pwm_8b <= I2C_OFFSET_WDATA;
    end

    // 0x61 -- IOPLD_REG_10G_BY4_LED_G
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_10g_by4_led_g_pwm_8b <= 8'hFF;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_10G_BY4_LED_G )
        r_10g_by4_led_g_pwm_8b <= I2C_OFFSET_WDATA;
    end

    // 0x62 -- IOPLD_REG_10G_BY4_LED_R
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_10g_by4_led_r_pwm_8b <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_10G_BY4_LED_R )
        r_10g_by4_led_r_pwm_8b <= I2C_OFFSET_WDATA;
    end

    // 0x63 -- IOPLD_REG_25G_BY4_LED_B
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_25g_by4_led_b_pwm_8b <= 8'h00;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_25G_BY4_LED_B )
        r_25g_by4_led_b_pwm_8b <= I2C_OFFSET_WDATA;
    end

    // 0x64 -- IOPLD_REG_25G_BY4_LED_G
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_25g_by4_led_g_pwm_8b <= 8'h10;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_25G_BY4_LED_G )
        r_25g_by4_led_g_pwm_8b <= I2C_OFFSET_WDATA;
    end

    // 0x65 -- IOPLD_REG_25G_BY4_LED_R
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
        r_25g_by4_led_r_pwm_8b <= 8'hFF;
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_25G_BY4_LED_R )
        r_25g_by4_led_r_pwm_8b <= I2C_OFFSET_WDATA;
    end

    // 0x66 -- IOPLD_REG_40G_100G_LED_1
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_40g_led_blue_6b[3:0]   <= 4'hF;
        r_100g_led_green_6b[3:0] <= 4'hF;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_40G_100G_LED_1 )
      begin
        r_40g_led_blue_6b[3:0]   <= { I2C_OFFSET_WDATA[1], I2C_OFFSET_WDATA[3], I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_100g_led_green_6b[3:0] <= { I2C_OFFSET_WDATA[0], I2C_OFFSET_WDATA[2], I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end

    // 0x67 -- IOPLD_REG_40G_100G_LED_2
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_40g_led_blue_6b[5:4]   <= 2'h3;
        r_100g_led_green_6b[5:4] <= 2'h3;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_40G_100G_LED_2 )
      begin
        r_40g_led_blue_6b[5:4]   <= { I2C_OFFSET_WDATA[5], I2C_OFFSET_WDATA[7] };
        r_100g_led_green_6b[5:4] <= { I2C_OFFSET_WDATA[4], I2C_OFFSET_WDATA[6] };
      end
    end
	 
	 // 0x68
    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_port_config[5:0] <= 6'b000000;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_MISC )
      begin
        r_port_config[5:0] <= I2C_OFFSET_WDATA[7:2];
      end
    end
	 
	 //0x69
	 always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_port_config[11:6] <= 6'b000000;
      end
      else if ( I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `IOPLD_REG_MISC_2 )
      begin
        r_port_config[11:6] <= I2C_OFFSET_WDATA[5:0];
      end
    end


    //----------------------------------------------------------------------
    // Read Operation Statement by I2C Host
    //----------------------------------------------------------------------
    assign I2C_OFFSET_RDATA = r_i2c_readdata;

    always @( posedge clock or negedge reset_n )
    begin
      if ( reset_n == 1'b0 )
      begin
        r_i2c_readdata <= 8'h00;
      end
      else if ( I2C_READ == 1'b1 )
      begin
        case ( I2C_OFFSET_ADDR )
          `IOPLD_REG_CPLD_VERSION                : r_i2c_readdata <= {        `CPLD_VER                                           };  // 0x01 : IOPLD_REG_CPLD_VERSION
          `IOPLD_REG_INT_STATUS_MASK             : r_i2c_readdata <= { 4'h0,  CPLD2_MOD_ABS,                                          // 0x02 : IOPLD_REG_INT_STATUS_MASK
                                                                              CPLD2_QSFP_INT,
                                                                              r_cpld2_mod_abs_mask,
                                                                              r_cpld2_qsfp_int_mask                               };
          `IOPLD_REG_SYSTEM_RESET                : r_i2c_readdata <= { 7'h00, CPLD2_QSFP_RST                                      };  // 0x03 : IOPLD_REG_SYSTEM_RESET
          `IOPLD_REG_XSFP_INT_4                  : r_i2c_readdata <= {        r_xsfp_int_24b[7:0]                                 };  // 0x04 : IOPLD_REG_XSFP_INT_4
          `IOPLD_REG_XSFP_INT_MASK_4             : r_i2c_readdata <= {        r_xsfp_int_mask_24b[7:0]                            };  // 0x05 : IOPLD_REG_XSFP_INT_MASK_4
          `IOPLD_REG_XSFP_INT_5                  : r_i2c_readdata <= {        r_xsfp_int_24b[15:8]                                };  // 0x06 : IOPLD_REG_XSFP_INT_5
          `IOPLD_REG_XSFP_INT_MASK_5             : r_i2c_readdata <= {        r_xsfp_int_mask_24b[15:8]                           };  // 0x07 : IOPLD_REG_XSFP_INT_MASK_5
          `IOPLD_REG_XSFP_INT_6                  : r_i2c_readdata <= {        r_xsfp_int_24b[23:16]                               };  // 0x08 : IOPLD_REG_XSFP_INT_6
          `IOPLD_REG_XSFP_INT_MASK_6             : r_i2c_readdata <= {        r_xsfp_int_mask_24b[23:16]                          };  // 0x09 : IOPLD_REG_XSFP_INT_MASK_6
          `IOPLD_REG_XSFP_PRSNT_4                : r_i2c_readdata <= {        r_xsfp_prsnt_24b[7:0]                               };  // 0x10 : IOPLD_REG_XSFP_PRSNT_4
          `IOPLD_REG_XSFP_PRSNT_5                : r_i2c_readdata <= {        r_xsfp_prsnt_24b[15:8]                              };  // 0x11 : IOPLD_REG_XSFP_PRSNT_5
          `IOPLD_REG_XSFP_PRSNT_6                : r_i2c_readdata <= {        r_xsfp_prsnt_24b[23:16]                             };  // 0x12 : IOPLD_REG_XSFP_PRSNT_6
          `IOPLD_REG_XSFP_TX_FAULT_4             : r_i2c_readdata <= {        r_xsfp_tx_fault_24b[7:0]                            };  // 0x14 : IOPLD_REG_XSFP_TX_FAULT_4
          `IOPLD_REG_XSFP_TX_FAULT_MASK_4        : r_i2c_readdata <= {        r_xsfp_tx_fault_mask_24b[7:0]                       };  // 0x15 : IOPLD_REG_XSFP_TX_FAULT_MASK_4
          `IOPLD_REG_XSFP_TX_FAULT_5             : r_i2c_readdata <= {        r_xsfp_tx_fault_24b[15:8]                           };  // 0x16 : IOPLD_REG_XSFP_TX_FAULT_5
          `IOPLD_REG_XSFP_TX_FAULT_MASK_5        : r_i2c_readdata <= {        r_xsfp_tx_fault_mask_24b[15:8]                      };  // 0x17 : IOPLD_REG_XSFP_TX_FAULT_MASK_5
          `IOPLD_REG_XSFP_TX_FAULT_6             : r_i2c_readdata <= {        r_xsfp_tx_fault_24b[23:16]                          };  // 0x18 : IOPLD_REG_XSFP_TX_FAULT_6
          `IOPLD_REG_XSFP_TX_FAULT_MASK_6        : r_i2c_readdata <= {        r_xsfp_tx_fault_mask_24b[23:16]                     };  // 0x19 : IOPLD_REG_XSFP_TX_FAULT_MASK_6
          `IOPLD_REG_XSFP_TX_DIS_4               : r_i2c_readdata <= {        r_xsfp_tx_dis_24b[7:0]                              };  // 0x20 : IOPLD_REG_XSFP_TX_DIS_4
          `IOPLD_REG_XSFP_TX_DIS_MASK_4          : r_i2c_readdata <= {        r_xsfp_tx_dis_mask_24b[7:0]                         };  // 0x21 : IOPLD_REG_XSFP_TX_DIS_MASK_4
          `IOPLD_REG_XSFP_TX_DIS_5               : r_i2c_readdata <= {        r_xsfp_tx_dis_24b[15:8]                             };  // 0x22 : IOPLD_REG_XSFP_TX_DIS_5
          `IOPLD_REG_XSFP_TX_DIS_MASK_5          : r_i2c_readdata <= {        r_xsfp_tx_dis_mask_24b[15:8]                        };  // 0x23 : IOPLD_REG_XSFP_TX_DIS_MASK_5
          `IOPLD_REG_XSFP_TX_DIS_6               : r_i2c_readdata <= {        r_xsfp_tx_dis_24b[23:16]                            };  // 0x24 : IOPLD_REG_XSFP_TX_DIS_6
          `IOPLD_REG_XSFP_TX_DIS_MASK_6          : r_i2c_readdata <= {        r_xsfp_tx_dis_mask_24b[23:16]                       };  // 0x25 : IOPLD_REG_XSFP_TX_DIS_MASK_6
          `IOPLD_REG_XSFP_RX_LOS_4               : r_i2c_readdata <= {        r_xsfp_rx_loss_24b[7:0]                             };  // 0x30 : IOPLD_REG_XSFP_RX_LOS_4
          `IOPLD_REG_XSFP_RX_LOS_MASK_4          : r_i2c_readdata <= {        r_xsfp_rx_loss_mask_24b[7:0]                        };  // 0x31 : IOPLD_REG_XSFP_RX_LOS_MASK_4
          `IOPLD_REG_XSFP_RX_LOS_5               : r_i2c_readdata <= {        r_xsfp_rx_loss_24b[15:8]                            };  // 0x32 : IOPLD_REG_XSFP_RX_LOS_5
          `IOPLD_REG_XSFP_RX_LOS_MASK_5          : r_i2c_readdata <= {        r_xsfp_rx_loss_mask_24b[15:8]                       };  // 0x33 : IOPLD_REG_XSFP_RX_LOS_MASK_5
          `IOPLD_REG_XSFP_RX_LOS_6               : r_i2c_readdata <= {        r_xsfp_rx_loss_24b[23:16]                           };  // 0x34 : IOPLD_REG_XSFP_RX_LOS_6
          `IOPLD_REG_XSFP_RX_LOS_MASK_6          : r_i2c_readdata <= {        r_xsfp_rx_loss_mask_24b[23:16]                      };  // 0x35 : IOPLD_REG_XSFP_RX_LOS_MASK_6
          `IOPLD_REG_1G_10G_LED_7                : r_i2c_readdata <= {        r_sfp_led_amber_24b[0],    r_sfp_led_green_24b[0],      // 0x40 : IOPLD_REG_1G_10G_LED_7
                                                                              r_sfp_led_amber_24b[1],    r_sfp_led_green_24b[1], 
                                                                              r_sfp_led_amber_24b[2],    r_sfp_led_green_24b[2], 
                                                                              r_sfp_led_amber_24b[3],    r_sfp_led_green_24b[3]   };
          `IOPLD_REG_1G_10G_LED_8                : r_i2c_readdata <= {        r_sfp_led_amber_24b[4],    r_sfp_led_green_24b[4],      // 0x41 : IOPLD_REG_1G_10G_LED_8
                                                                              r_sfp_led_amber_24b[5],    r_sfp_led_green_24b[5], 
                                                                              r_sfp_led_amber_24b[6],    r_sfp_led_green_24b[6], 
                                                                              r_sfp_led_amber_24b[7],    r_sfp_led_green_24b[7]   };
          `IOPLD_REG_1G_10G_LED_9                : r_i2c_readdata <= {        r_sfp_led_amber_24b[8],    r_sfp_led_green_24b[8],      // 0x42 : IOPLD_REG_1G_10G_LED_9
                                                                              r_sfp_led_amber_24b[9],    r_sfp_led_green_24b[9], 
                                                                              r_sfp_led_amber_24b[10],   r_sfp_led_green_24b[10],
                                                                              r_sfp_led_amber_24b[11],   r_sfp_led_green_24b[11]  };
          `IOPLD_REG_1G_10G_LED_10               : r_i2c_readdata <= {        r_sfp_led_amber_24b[12],   r_sfp_led_green_24b[12],     // 0x43 : IOPLD_REG_1G_10G_LED_10
                                                                              r_sfp_led_amber_24b[13],   r_sfp_led_green_24b[13],
                                                                              r_sfp_led_amber_24b[14],   r_sfp_led_green_24b[14],
                                                                              r_sfp_led_amber_24b[15],   r_sfp_led_green_24b[15]  };
          `IOPLD_REG_1G_10G_LED_11               : r_i2c_readdata <= {        r_sfp_led_amber_24b[16],   r_sfp_led_green_24b[16],     // 0x44 : IOPLD_REG_1G_10G_LED_11
                                                                              r_sfp_led_amber_24b[17],   r_sfp_led_green_24b[17],
                                                                              r_sfp_led_amber_24b[18],   r_sfp_led_green_24b[18],
                                                                              r_sfp_led_amber_24b[19],   r_sfp_led_green_24b[19]  };
          `IOPLD_REG_1G_10G_LED_12               : r_i2c_readdata <= {        r_sfp_led_amber_24b[20],   r_sfp_led_green_24b[20],     // 0x45 : IOPLD_REG_1G_10G_LED_12
                                                                              r_sfp_led_amber_24b[21],   r_sfp_led_green_24b[21],
                                                                              r_sfp_led_amber_24b[22],   r_sfp_led_green_24b[22],
                                                                              r_sfp_led_amber_24b[23],   r_sfp_led_green_24b[23]  };
          `IOPLD_REG_ZQSFP_MOD_INT               : r_i2c_readdata <= { 2'h0,  r_qsfp_mod_int_6b[5:0]                              };  // 0x50 : IOPLD_REG_ZQSFP_MOD_INT
          `IOPLD_REG_ZQSFP_MOD_INT_MASK          : r_i2c_readdata <= { 2'h0,  r_qsfp_mod_int_mask_6b[5:0]                         };  // 0x51 :  IOPLD_REG_ZQSFP_MOD_INT_MASK
          `IOPLD_REG_ZQSFP_PRSNT                 : r_i2c_readdata <= { 2'h0,  r_qsfp_prsnt_6b[5:0]                                };  // 0x52 : IOPLD_REG_ZQSFP_PRSNT
          `IOPLD_REG_ZQSFP_MOD_RST               : r_i2c_readdata <= { 2'h0,  r_qsfp_mod_rst_6b[5:0]                              };  // 0x53 : IOPLD_REG_ZQSFP_MOD_RST
          `IOPLD_REG_ZQSFP_LPMODE                : r_i2c_readdata <= { 2'h0,  r_qsfp_lpmode_6b[5:0]                               };  // 0x54 : IOPLD_REG_ZQSFP_LPMODE
          `IOPLD_REG_10G_BY4_LED_B               : r_i2c_readdata <= {        r_10g_by4_led_b_pwm_8b[7:0]                         };  // 0x60 : IOPLD_REG_10G_BY4_LED_B
          `IOPLD_REG_10G_BY4_LED_G               : r_i2c_readdata <= {        r_10g_by4_led_g_pwm_8b[7:0]                         };  // 0x61 :  IOPLD_REG_10G_BY4_LED_G 
          `IOPLD_REG_10G_BY4_LED_R               : r_i2c_readdata <= {        r_10g_by4_led_r_pwm_8b[7:0]                         };  // 0x62 : IOPLD_REG_10G_BY4_LED_R
          `IOPLD_REG_25G_BY4_LED_B               : r_i2c_readdata <= {        r_25g_by4_led_b_pwm_8b[7:0]                         };  // 0x63 : IOPLD_REG_25G_BY4_LED_B
          `IOPLD_REG_25G_BY4_LED_G               : r_i2c_readdata <= {        r_25g_by4_led_g_pwm_8b[7:0]                         };  // 0x64 : IOPLD_REG_25G_BY4_LED_G
          `IOPLD_REG_25G_BY4_LED_R               : r_i2c_readdata <= {        r_25g_by4_led_r_pwm_8b[7:0]                         };  // 0x65 : IOPLD_REG_25G_BY4_LED_R
          `IOPLD_REG_40G_100G_LED_1              : r_i2c_readdata <= {        r_40g_led_blue_6b[0],      r_100g_led_green_6b[0],      // 0x66 : IOPLD_REG_40G_100G_LED_1
                                                                              r_40g_led_blue_6b[1],      r_100g_led_green_6b[1],
                                                                              r_40g_led_blue_6b[2],      r_100g_led_green_6b[2],
                                                                              r_40g_led_blue_6b[3],      r_100g_led_green_6b[3]   };
          `IOPLD_REG_40G_100G_LED_2              : r_i2c_readdata <= {        r_40g_led_blue_6b[4],      r_100g_led_green_6b[4],      // 0x67 : IOPLD_REG_40G_100G_LED_2
                                                                              r_40g_led_blue_6b[5],      r_100g_led_green_6b[5],
                                                                       4'h0                                                       };
          `IOPLD_REG_MISC                        : r_i2c_readdata <= { 	      r_port_config[5:0],
																										CPLD21_SYNC,                                            // 0x68 : IOPLD_REG_MISC
                                                                              CPLD12_SYNC                                         };
			 `IOPLD_REG_MISC_2                      : r_i2c_readdata <= { 2'h0,	r_port_config[11:6]                                 };  // 0x69
			 
			 `SYSPLD_REG_STREAM_LED0_1              : r_i2c_readdata <= {        r_led0_stream[7:0]                                  };  // 0x71 : SYSPLD_REG_STREAM_LED_1
			 `SYSPLD_REG_STREAM_LED0_2              : r_i2c_readdata <= {        r_led0_stream[15:8]                                 };  // 0x72 : SYSPLD_REG_STREAM_LED_2
			 `SYSPLD_REG_STREAM_LED0_3              : r_i2c_readdata <= {        r_led0_stream[23:16]                                };  // 0x73 : SYSPLD_REG_STREAM_LED_3
			 `SYSPLD_REG_STREAM_LED0_4              : r_i2c_readdata <= {        r_led0_stream[31:24]                                };  // 0x74 : SYSPLD_REG_STREAM_LED_4
			 `SYSPLD_REG_STREAM_LED0_5              : r_i2c_readdata <= {        r_led0_stream[39:32]                                };  // 0x75 : SYSPLD_REG_STREAM_LED_5
			 `SYSPLD_REG_STREAM_LED0_6              : r_i2c_readdata <= {        r_led0_stream[47:40]                                };  // 0x76 : SYSPLD_REG_STREAM_LED_6
			 `SYSPLD_REG_STREAM_LED0_7              : r_i2c_readdata <= {        r_led0_stream[55:48]                                };  // 0x77 : SYSPLD_REG_STREAM_LED_7
			 `SYSPLD_REG_STREAM_LED0_8              : r_i2c_readdata <= {        r_led0_stream[63:56]                                };  // 0x78 : SYSPLD_REG_STREAM_LED_8
			 `SYSPLD_REG_STREAM_LED0_9              : r_i2c_readdata <= {        r_led0_stream[71:64]                                };  // 0x79 : SYSPLD_REG_STREAM_LED_9
			 `SYSPLD_REG_STREAM_LED0_10             : r_i2c_readdata <= {        r_led0_stream[79:72]                                };  // 0x80 : SYSPLD_REG_STREAM_LED_10
			 `SYSPLD_REG_STREAM_LED0_11             : r_i2c_readdata <= { 4'h0,  r_led0_stream[83:80]                                };  // 0x81 : SYSPLD_REG_STREAM_LED_11
			 
			 `SYSPLD_REG_STREAM_LED2_1              : r_i2c_readdata <= {        r_led2_stream[7:0]                                  };  // 0x91 : SYSPLD_REG_STREAM_LED_1
			 `SYSPLD_REG_STREAM_LED2_2              : r_i2c_readdata <= {        r_led2_stream[15:8]                                 };  // 0x92 : SYSPLD_REG_STREAM_LED_2
			 `SYSPLD_REG_STREAM_LED2_3              : r_i2c_readdata <= {        r_led2_stream[23:16]                                };  // 0x93 : SYSPLD_REG_STREAM_LED_3
			 `SYSPLD_REG_STREAM_LED2_4              : r_i2c_readdata <= {        r_led2_stream[31:24]                                };  // 0x94 : SYSPLD_REG_STREAM_LED_4
			 `SYSPLD_REG_STREAM_LED2_5              : r_i2c_readdata <= {        r_led2_stream[39:32]                                };  // 0x95 : SYSPLD_REG_STREAM_LED_5
			 `SYSPLD_REG_STREAM_LED2_6              : r_i2c_readdata <= {        r_led2_stream[47:40]                                };  // 0x96 : SYSPLD_REG_STREAM_LED_6
			 `SYSPLD_REG_STREAM_LED2_7              : r_i2c_readdata <= {        r_led2_stream[55:48]                                };  // 0x97 : SYSPLD_REG_STREAM_LED_7
			 `SYSPLD_REG_STREAM_LED2_8              : r_i2c_readdata <= {        r_led2_stream[63:56]                                };  // 0x98 : SYSPLD_REG_STREAM_LED_8
			 `SYSPLD_REG_STREAM_LED2_9              : r_i2c_readdata <= {        r_led2_stream[71:64]                                };  // 0x99 : SYSPLD_REG_STREAM_LED_9
			 `SYSPLD_REG_STREAM_LED2_10             : r_i2c_readdata <= {        r_led2_stream[79:72]                                };  // 0xa0 : SYSPLD_REG_STREAM_LED_10
			 `SYSPLD_REG_STREAM_LED2_11             : r_i2c_readdata <= { 4'h0,  r_led2_stream[83:80]                                };  // 0xa1 : SYSPLD_REG_STREAM_LED_11
          default                                : r_i2c_readdata <= 8'hFF;
        endcase
      end
    end

endmodule
