//---------------------------------------------------------------------
//---------------------------------------------------------------------
// This is a FAN Control logic with PWM and Tach.
// Sam Lin, Galaxy TW
//
// History :
// 08-30-14  Created
//---------------------------------------------------------------------
`include "../top/es7632bt_fanpld_defined.vh"
//---------------------------------------------------------------------


module fan_controller (
    input                               MGMT_CLK,
    input                               MGMT_RST_N,
    input  [`FAN_SENSE_NUM-1 : 0]       FAN_SENSE,
    input  [3 : 0]                      FAN_PWM,
    output                              HALF_SEC_TICK,
    output                              FAN_CONTROL,
    output [(`FAN_SENSE_NUM*9)-1 : 0]   FAN_TACH
) ;
    // Signals


    // Buffers
    wire   clock   = MGMT_CLK;
    wire   reset_n = MGMT_RST_N;
    assign HALF_SEC_TICK = half_tick;

/************** Fan Control Logic ****************/

    /* The idea here is to PWM the fan control
     signals at a frequency of 21-28kHz(typ. 25khz)
     with a 4-bit width value.
    */

    //reg [5:0] master_clk_div; // for 25mhz
    reg [6:0] master_clk_div = 7'd0;
    reg [3:0] pwm_cntr = 4'hF;

    always @(posedge clock) begin
        if (!reset_n) begin
            master_clk_div <= 7'd0;
            pwm_cntr   <= 4'hF;
        end else begin
          //if (master_clk_div == 7'd62) begin
          // if (master_clk_div == 7'd124) begin
          if (master_clk_div == 7'd65) begin // test on 25 mhz
          //if (master_clk_div == 7'd127) begin // test on 50 mhz
                pwm_cntr   <= pwm_cntr + 4'd1;
            /* 16 bit (2^4 - 1) counter that overflows at 26 kHz */
            /* 40us(25khz)/15 == 2.667us, 2.667us/40 ns (25 mhz) == 66.67 */
            /* 25 MHz / 63  / 15 == 26 kHz */
            /* 25 Mhz / 66 / 15 == 25.252 khz*/
            /* 50 Mhz / 126 / 15 == 26 kHz */
            /* 50 Mhz / 128 / 15 == 26.041 khz*/
                master_clk_div <= 7'd0;
          end else begin
                master_clk_div <= master_clk_div + 7'd1;
          end
        end
    end

    fan_pwm FAN_PWM_I (clock, reset_n, pwm_cntr, FAN_PWM, FAN_CONTROL);

/*
    generate
        genvar  pwm_num;
        for ( pwm_num=0; pwm_num<`FAN_PWM_NUM; pwm_num=pwm_num+1) begin : FAN_CTRL_PROC
            fan_pwm FAN_PWM_I (clock, reset_n, pwm_cntr, FAN_PWM, FAN_CONTROL[pwm_num]);
        end
    endgenerate
*/

/*********** Fan Speed Sense Logic *************/

    reg [24:0] half_second_counter = 25'd0;
    reg        half_tick = 1'b0;

    always @(posedge clock) begin
        if (!reset_n) begin
            half_second_counter <= 25'd0;
            half_tick           <= 1'b0;
        end else begin
            if (half_second_counter == `HALT_SEC_TIME) begin
            /* clk = 25MHz, every 12500000 is 0.5s */
            /* clk = 50MHz, every 25000000 is 0.5s */
                half_second_counter <= 25'd0;
                half_tick           <= 1'b1;
            end else begin
                half_second_counter <= half_second_counter + 25'd1;
                half_tick           <= 1'b0;
            end
        end
    end

	// Sampes Time process
	reg 		tick_600ms;
	reg [23:0]	clk_1_67hz_cntr;

	always @(posedge clock) begin
		if (!reset_n) begin
			clk_1_67hz_cntr <= 24'd0;
			tick_600ms	    <= 1'b0;
		end else begin
			if (clk_1_67hz_cntr == 24'd14_999_999) begin
				clk_1_67hz_cntr <= 24'd0;
				tick_600ms	    <= 1'b1;
			end else begin
				clk_1_67hz_cntr <= clk_1_67hz_cntr + 24'd1;
				tick_600ms	    <= 1'b0;
			end
		end
	end

    generate
        genvar  tach_num;
        for ( tach_num=0; tach_num<`FAN_SENSE_NUM; tach_num=tach_num+1) begin : FAN_SPEED_PROC
            fan_sense FAN_TACH_I (clock, reset_n, tick_600ms, FAN_SENSE[tach_num],
                                    FAN_TACH[(tach_num*9)+8 : (tach_num*9)]);
        end
    endgenerate

endmodule

/*
    reg [1:0] prev_fan_sense;

    reg [7:0] fan_speed_tmp_0;
    reg [7:0] fan_speed_tmp_1;

    reg [7:0] debounce_0;
    reg [7:0] debounce_1;

    reg fan_tick_0;
    reg fan_tick_1;

    reg [7:0] half_rev_counter_0;
    reg [7:0] half_rev_counter_1;

    reg [23:0] half_second_counter;

    always @(posedge clock) begin
        fan_tick_0 <= 1'b0;
        fan_tick_1 <= 1'b0;

        prev_fan_sense <= FAN_SENSE;

        if (reset_n) begin
            debounce_0 <= 8'b0;
            debounce_1 <= 8'b0;
        end else begin
            if (debounce_0) begin
                debounce_0 <= debounce_0 - 8'd1;
            end else begin
                if (FAN_SENSE[0] && prev_fan_sense[0] != FAN_SENSE[0]) begin
                  fan_tick_0 <= 1'b1;
                  debounce_0 <= 8'hff;
                end
            end
            if (debounce_1) begin
                debounce_1 <= debounce_1 - 8'd1;
            end else begin
                if (FAN_SENSE[1] && prev_fan_sense[1] != FAN_SENSE[1]) begin
                  fan_tick_1 <= 1'b1;
                  debounce_1 <= 8'hff;
                end
            end
        end
    end


    assign FAN_SPEED_0 = fan_speed_tmp_0;
    assign FAN_SPEED_1 = fan_speed_tmp_1;

endmodule */
