//*********************************************************************
// This is a FAN Speed Sense Logic
//
// sam.lin@gfec.com.tw, GALAXY TW
// History :
// 08-29-14  Created
//
//*********************************************************************
`include "../top/es7632bt_fanpld_defined.vh"

module fan_sense (
    input         MGMT_CLK,  // ~25Mhz
    input         MGMT_RST_N,
    input         HALF_TICK, // a tick of 0.5 sec
    input         FAN_SENSE,
    output [8:0]  FAN_TACH
) ;

    // Signals for this Design
    reg			fan_sense_d;
    reg     	r_fan_sense;
    reg     	prev_fan_sense;
    reg     	fan_tick = 0;

    reg [8:0]  	fan_speed_tmp = 9'h000;
    reg [8:0]  	half_rev_counter = 9'h000;

//    reg [7:0]  fan_speed_tmp;
//    reg [7:0]  debounce;


    // Buffers
    wire   clock    = MGMT_CLK;
    wire   reset_n  = MGMT_RST_N;
    assign FAN_TACH = fan_speed_tmp;

    // Synchronized Filp-Flop
    always @(posedge clock) begin
    	fan_sense_d    <= FAN_SENSE;
        r_fan_sense    <= fan_sense_d;
        prev_fan_sense <= r_fan_sense;
    end


    always @(posedge clock) begin
        if (!reset_n) begin
            fan_tick <= 1'b0;
        end else begin
            if (r_fan_sense && prev_fan_sense != r_fan_sense) begin // signal rising
              fan_tick <= 1'b1;
            end else begin
              fan_tick <= 1'b0;
            end
        end
    end

    always @(posedge clock) begin
        if (!reset_n) begin
            half_rev_counter <= 9'h000;
            fan_speed_tmp    <= 9'h1FF;
        end else begin
            if (fan_tick) begin
                half_rev_counter <= half_rev_counter + 9'd1;
            end
            if (HALF_TICK) begin
                fan_speed_tmp    <= half_rev_counter;
                half_rev_counter <= 9'h00;
            end
        end
    end

endmodule
