/////////////////////////////////////////
// Offset Register for FAN CTRL PLD
/////////////////////////////////////////
`define FANPLD_REG_BOARD_INFO       8'h00
`define FANPLD_REG_CPLD_VER         8'h01
`define FANPLD_REG_CPLD_RST         8'h04
`define FANPLD_REG_INT_STATUS       8'h05
`define FANPLD_REG_INT_MASK         8'h06
`define FANPLD_REG_FAN_PRSNT        8'h0F
`define FANPLD_REG_FAN_DIR          8'h10
`define FANPLD_REG_FAN_PWM          8'h11
`define FANPLD_REG_FRONT_FAN1_TACH  8'h12
`define FANPLD_REG_FRONT_FAN2_TACH  8'h13
`define FANPLD_REG_FRONT_FAN3_TACH  8'h14
`define FANPLD_REG_FRONT_FAN4_TACH  8'h15
`define FANPLD_REG_FRONT_FAN5_TACH  8'h16
`define FANPLD_REG_FRONT_FAN6_TACH  8'h17
`define FANPLD_REG_FAN_LED1_DISPLAY 8'h1C
`define FANPLD_REG_FAN_LED2_DISPLAY 8'h1D
`define FANPLD_REG_FAN_I2C_ENABLE   8'h20
`define FANPLD_REG_REAR_FAN1_TACH   8'h22
`define FANPLD_REG_REAR_FAN2_TACH   8'h23
`define FANPLD_REG_REAR_FAN3_TACH   8'h24
`define FANPLD_REG_REAR_FAN4_TACH   8'h25
`define FANPLD_REG_REAR_FAN5_TACH   8'h26
`define FANPLD_REG_REAR_FAN6_TACH   8'h27
`define FANPLD_REG_FAN_POWER_CTRL   8'h30
`define FANPLD_REG_FAN_WD_TIMER     8'h31
`define FANPLD_REG_FAN_WD_PWM_CTRL  8'h32
`define FANPLD_REG_FAN_WD_DISABLE   8'h33
`define FANPLD_REG_DEBUF_MODE       8'h42

/////////////////////////////////////////
// CPLD Paramter
/////////////////////////////////////////
`define PLD_ID                      8'h08
`define BOARD_ID                    2'b00
`define RES_REG_DEFAULT             8'h00

/////////////////////////////////////////
// FAN Control Paramter
/////////////////////////////////////////
`define FAN_LED_BIT 16*9
`define DEFAULT_PWM 4'h5
`define FAN_PWM_NUM 6
`define FAN_SENSE_NUM 12
/////////////////////////////////////////
// Timer Paramter
/////////////////////////////////////////
`define HALT_SEC_TIME 25'd12_500_000 - 25'd1 // Based on clk 25mhz
//`define HALT_SEC_TIME 25'd25_000_000 - 25'd1 // Based on clk 50mhz
`define ONE_SECOND 25'd24_999_999 // Based on clk 25mhz
`define TWO_SECOND 26'd50_000_000 - 26'd1 // Based on clk 25mhz


/////////////////////////////////////////
// Watchdog Paramter
/////////////////////////////////////////
`define WDT_DEFAULT 8'd06

/////////////////////////////////////////
// I2C Master Core Paramter
/////////////////////////////////////////
`define CNTR_PRESCALE 8'h32 // for 25mhz
//`define CNTR_PRESCALE 8'h64 // for 50mhz

/////////////////////////////////////////
// LM75 Paramter
/////////////////////////////////////////
`define WR_LM75_1 8'h90      // 7'h48
`define WR_LM75_2 8'h92      // 7'h49
`define WR_LM75_3 8'h94      // 7'h4A
`define RD_LM75_1 8'h91      // 7'h48
`define RD_LM75_2 8'h93      // 7'h49
`define RD_LM75_3 8'h95      // 7'h4A
`define TEMPERATURE_30 8'h1E // 11'd240
`define TEMPERATURE_40 8'h28 // 11'd320
`define TEMPERATURE_41 8'h29
`define TEMPERATURE_42 8'h2A
`define TEMPERATURE_43 8'h2B
`define TEMPERATURE_44 8'h2C
`define TEMPERATURE_45 8'h2D
`define TEMPERATURE_46 8'h2E
`define TEMPERATURE_47 8'h2F
`define TEMPERATURE_48 8'h30
`define TEMPERATURE_49 8'h31
`define TEMPERATURE_50 8'h32 // 11'd400
