//---------------------------------------------------------------------
//---------------------------------------------------------------------
// RTL Top File for Model ES7632BT
// Sam Lin, Galaxy TW
// History :
//
// 2014-09-02 Created
//---------------------------------------------------------------------
`include "./es7632bt_fanpld_defined.vh"
`define I2C_S_DEVADDR 7'h66
//---------------------------------------------------------------------
module es7632bt_fanpld_top (
    input           CPLD_CLK,
//    input           CPLD_RESET_N,
	input           I2C_MASTER_ENB,
    input           HOST_I2C_SCL,
    inout           HOST_I2C_SDA,
    inout           DEVICES_I2C_SCL,
    inout           DEVICES_I2C_SDA,
    input  [2:0]    VERSION_ID,
    input  [1:0]    BOARD_ID,
    input  [5:0]    FAN_F_SENSE,
    input  [5:0]    FAN_R_SENSE,
    output [5:0]    FAN_CONTROL,
    output [5:0]    FAN_LED_R,
    output [5:0]    FAN_LED_G,
    input  [5:0]    FAN_DIR,
    input           FAN_IDEL, // Pin A2
    output          FAN_INT,
    output [5:0]    FAN_ENB,
	 output 			  FAN_MOS_CTRL //Fan mos ctrl_20150819
    );

    // signals for top
    wire        host_i2c_scl_in_sig;
    wire        host_i2c_sda_in_sig;
    wire        i2c_s_sda_sig_n;
    wire [5:0]  fan_control_sig;

    // signals for I2C Master Core Instance
    wire        i2c_core_start_sig;
    wire        i2c_core_stop_sig;
    wire        i2c_core_read_sig;
    wire        i2c_core_write_sig;
    wire        i2c_core_send_ack_sig;
    wire        i2c_core_recv_ack_sig;
    wire        i2c_core_cmd_ack_sig;
    wire  [7:0] i2c_core_writedata_sig;
    wire  [7:0] i2c_core_readdata_sig;

    // signals for LM75 Driver Instance
    wire  [3:0] fan_pwm_ctrl_sig;
    wire  [3:0] fan_pwm_ctrl_cpld_sig;
    wire        fan_pwm_updata_sig;
    wire        fan_pwm_sig;
    wire        half_sec_tick_sig;

    // signals for I2C Slave Instance
    wire        i2c_read_sig;
    wire        i2c_read_done_sig;
    wire        i2c_write_sig;
    wire  [7:0] i2c_offset_addr_sig;
    wire  [7:0] i2c_offser_rdata_sig;
    wire  [7:0] i2c_offser_wdata_sig;

    // signals for Regfile Instance
    wire        ireset_n_sig;
    wire  [5:0] fan_disable_sig_6b;
    wire  [8:0] front_fan1_tach_sig_8b;
    wire  [8:0] front_fan2_tach_sig_8b;
    wire  [8:0] front_fan3_tach_sig_8b;
    wire  [8:0] front_fan4_tach_sig_8b;
    wire  [8:0] front_fan5_tach_sig_8b;
    wire  [8:0] front_fan6_tach_sig_8b;
    wire  [8:0] rear_fan1_tach_sig_8b;
    wire  [8:0] rear_fan2_tach_sig_8b;
    wire  [8:0] rear_fan3_tach_sig_8b;
    wire  [8:0] rear_fan4_tach_sig_8b;
    wire  [8:0] rear_fan5_tach_sig_8b;
    wire  [8:0] rear_fan6_tach_sig_8b;

    wire        fan_idel_sig;
    wire        fan_int_sig;
    wire        fan_stop_int_sig;
    wire        fan_i2c_m_mask_sig;

    wire        wd_pwm_ctrl_sig;
    wire  [7:0] reg_wd_timer_8b_sig;
    wire  [3:0] reg_wd_pwm_max_value_4b_sig;
    wire        reg_wd_enable_1b_sig;

    //-----------------------------------------------------------------
    // Buffer
    //-----------------------------------------------------------------
    assign host_i2c_scl_in_sig = HOST_I2C_SCL;

    assign HOST_I2C_SDA = (!i2c_s_sda_sig_n) ? 1'b0 : 1'bZ;
    assign host_i2c_sda_in_sig = HOST_I2C_SDA;

    assign FAN_CONTROL = ~fan_control_sig;

    assign fan_disable_sig_6b = 6'h00;

    assign fan_control_sig[0] = (!fan_disable_sig_6b[0]) ? fan_pwm_sig : 1'b0;
    assign fan_control_sig[1] = (!fan_disable_sig_6b[1]) ? fan_pwm_sig : 1'b0;
    assign fan_control_sig[2] = (!fan_disable_sig_6b[2]) ? fan_pwm_sig : 1'b0;
    assign fan_control_sig[3] = (!fan_disable_sig_6b[3]) ? fan_pwm_sig : 1'b0;
    assign fan_control_sig[4] = (!fan_disable_sig_6b[4]) ? fan_pwm_sig : 1'b0;
    assign fan_control_sig[5] = (!fan_disable_sig_6b[5]) ? fan_pwm_sig : 1'b0;

    assign fan_idel_sig = FAN_IDEL; // 1 : Stop I2C Master operation.

    assign FAN_INT = (~fan_int_sig | fan_stop_int_sig);
	 assign FAN_MOS_CTRL = 1'b0;				//Fan mos ctrl_20150819
	 
    //-----------------------------------------------------------------
    // Call a Instance of I2C Master Core
    //-----------------------------------------------------------------
    i2c_m I2C_MASTER_I (
        // Clock signals
        .clk          ( CPLD_CLK                ),
        .ena          ( I2C_MASTER_ENB          ),
        .nReset       ( ireset_n_sig            ),
        .clk_cnt      ( `CNTR_PRESCALE          ),
        // input signals
        .start        ( i2c_core_start_sig      ),
        .stop         ( i2c_core_stop_sig       ),
        .read         ( i2c_core_read_sig       ),
        .write        ( i2c_core_write_sig      ),
        .ack_in       ( i2c_core_send_ack_sig   ),
        .Din          ( i2c_core_writedata_sig  ),
        // output signals
        .cmd_ack      ( i2c_core_cmd_ack_sig    ),
        .ack_out      ( i2c_core_recv_ack_sig   ),
        .Dout         ( i2c_core_readdata_sig   ),
        // i2c signals
        .SCL          ( DEVICES_I2C_SCL         ),
        .SDA          ( DEVICES_I2C_SDA         )
        );

    //-----------------------------------------------------------------
    // Call a Instance of LM75 Driver with Fan PWM
    //-----------------------------------------------------------------
    temper_ctrl LM75_DRIVER_I (
        .MGMT_CLK      ( CPLD_CLK               ),
        .MGMT_RST_N    ( ireset_n_sig           ),
        .I2C_MASTER_ENB( I2C_MASTER_ENB         ),
        .FAN_IDEL      ( fan_idel_sig           ),
        .MASK_I2C_LM75 ( fan_i2c_m_mask_sig     ),
        .SCL           ( DEVICES_I2C_SCL        ),
        .HALF_SEC_TICK ( half_sec_tick_sig      ),
        // I2C Core
        .I2C_START     ( i2c_core_start_sig     ),
        .I2C_STOP      ( i2c_core_stop_sig      ),
        .I2C_READ      ( i2c_core_read_sig      ),
        .I2C_WRITE     ( i2c_core_write_sig     ),
        .I2C_ACK_SEND  ( i2c_core_send_ack_sig  ),
        .I2C_WDATA     ( i2c_core_writedata_sig ),
        .I2C_CMD_ACK   ( i2c_core_cmd_ack_sig   ),
        .I2C_ACK_RECV  ( i2c_core_recv_ack_sig  ),
        .I2C_RDATA     ( i2c_core_readdata_sig  ),
        // FAN
        .FAN_STOP_INT  ( fan_stop_int_sig       ),
        .FAN_PWM_UPDATA( fan_pwm_updata_sig     ),
        .FAN_PWM_CTRL  ( fan_pwm_ctrl_cpld_sig  )
        );

    //-----------------------------------------------------------------
    // Call a Instance of Fan Control
    //-----------------------------------------------------------------
    fan_controller FAN_CTRL_I (
        .MGMT_CLK      ( CPLD_CLK                                       ),
        .MGMT_RST_N    ( ireset_n_sig                                   ),
        .FAN_SENSE     ({FAN_R_SENSE, FAN_F_SENSE}                      ),
        .FAN_PWM       ( fan_pwm_ctrl_sig                               ),
        .HALF_SEC_TICK ( half_sec_tick_sig                              ),
        .FAN_CONTROL   ( fan_pwm_sig                                    ),
        .FAN_TACH      ({rear_fan6_tach_sig_8b, rear_fan5_tach_sig_8b,
                         rear_fan4_tach_sig_8b, rear_fan3_tach_sig_8b,
                         rear_fan2_tach_sig_8b, rear_fan1_tach_sig_8b,
                         front_fan6_tach_sig_8b, front_fan5_tach_sig_8b,
                         front_fan4_tach_sig_8b, front_fan3_tach_sig_8b,
                         front_fan2_tach_sig_8b, front_fan1_tach_sig_8b})
        );

    //-----------------------------------------------------------------
    // Call a Instance of I2C Slave
    //-----------------------------------------------------------------
    i2c_s #(
        .I2C_DevADDR (`I2C_S_DEVADDR)
        ) I2C_SLAVE_I  (
        .clk           ( CPLD_CLK               ),
        .reset_n       ( 1'b1                   ),
        .soft_reset_n  ( 1'b1                   ),
        .scl_i         ( host_i2c_scl_in_sig    ),
        .sda_i         ( host_i2c_sda_in_sig    ),
        .sda_oe_n      ( i2c_s_sda_sig_n        ),
        .readdata      ( i2c_offser_rdata_sig   ),
        .writedata     ( i2c_offser_wdata_sig   ),
        .address       ( i2c_offset_addr_sig    ),
        .read          ( i2c_read_sig           ),
        .read_done     ( i2c_read_done_sig      ),
        .write         ( i2c_write_sig          )
        );
    //-----------------------------------------------------------------
    // Call a Instance of FAN Watchdog for CPU Crash
    //-----------------------------------------------------------------
    watchdog WATCHDOG_I(
        .MGMT_CLK                ( CPLD_CLK                    ),
        .MGMT_RST_N              ( ireset_n_sig                ),
        .I2C_WRITE               ( i2c_write_sig               ),
        .I2C_OFFSET_ADDR         ( i2c_offset_addr_sig         ),
        .I2C_OFFSER_WDATA        ( i2c_offser_wdata_sig        ),
        .WD_PWM_CTRL             ( wd_pwm_ctrl_sig             ),
        .REG_WD_TIMER_8B         ( reg_wd_timer_8b_sig         ),
        .REG_WD_PWM_MAX_VALUE_4B ( reg_wd_pwm_max_value_4b_sig ),
        .REG_WD_ENABLE_1B        ( reg_wd_enable_1b_sig        )
        );
    //-----------------------------------------------------------------
    // Call a Instance of Register Files
    //-----------------------------------------------------------------
    regfile REGFILES_I (
        .MGMT_CLK                ( CPLD_CLK                    ),
        .MGMT_RST_N              ( ireset_n_sig                ),
        .I2C_READ                ( i2c_read_sig                ),
        .I2C_READ_DONE           ( i2c_read_done_sig           ),
        .I2C_WRITE               ( i2c_write_sig               ),
        .I2C_OFFSET_ADDR         ( i2c_offset_addr_sig         ),
        .I2C_OFFSER_WDATA        ( i2c_offser_wdata_sig        ),
        .I2C_OFFSER_RDATA        ( i2c_offser_rdata_sig        ),
        .FAN_DIR_CTRL            ( FAN_DIR                     ),
        .FAN_PWM_OUT             ( fan_pwm_ctrl_sig            ),
        .FAN_LED_R               ( FAN_LED_R                   ),
        .FAN_LED_G               ( FAN_LED_G                   ),
        .MASK_I2C_LM75           ( fan_i2c_m_mask_sig          ),
        .FAN_INT                 ( fan_int_sig                 ),
        .RESET_OUT               ( ireset_n_sig                ),
        .FAN_ENB                 ( FAN_ENB                     ),
        .HW_BOARD_ID             ( {VERSION_ID, BOARD_ID}      ),
        .FAN_PWM_UPDATA          ( fan_pwm_updata_sig          ),
        .FAN_PWM_IN              ( fan_pwm_ctrl_cpld_sig       ),
        .FRONT_FAN1_TACH         ( front_fan1_tach_sig_8b[8:1] ),
        .FRONT_FAN2_TACH         ( front_fan2_tach_sig_8b[8:1] ),
        .FRONT_FAN3_TACH         ( front_fan3_tach_sig_8b[8:1] ),
        .FRONT_FAN4_TACH         ( front_fan4_tach_sig_8b[8:1] ),
        .FRONT_FAN5_TACH         ( front_fan5_tach_sig_8b[8:1] ),
        .FRONT_FAN6_TACH         ( front_fan6_tach_sig_8b[8:1] ),
        .REAR_FAN1_TACH          ( rear_fan1_tach_sig_8b[8:1]  ),
        .REAR_FAN2_TACH          ( rear_fan2_tach_sig_8b[8:1]  ),
        .REAR_FAN3_TACH          ( rear_fan3_tach_sig_8b[8:1]  ),
        .REAR_FAN4_TACH          ( rear_fan4_tach_sig_8b[8:1]  ),
        .REAR_FAN5_TACH          ( rear_fan5_tach_sig_8b[8:1]  ),
        .REAR_FAN6_TACH          ( rear_fan6_tach_sig_8b[8:1]  ),
        .WD_PWM_CTRL             ( wd_pwm_ctrl_sig             ),
        .REG_WD_TIMER_8B         ( reg_wd_timer_8b_sig         ),
        .REG_WD_PWM_MAX_VALUE_4B ( reg_wd_pwm_max_value_4b_sig ),
        .REG_WD_ENABLE_1B        ( reg_wd_enable_1b_sig        )
        );

endmodule
