//---------------------------------------------------------------------
//---------------------------------------------------------------------
// Offser Register Files
// Sam Lin, Galaxy TW
//
// History :
// 2014-09-02 Created
//---------------------------------------------------------------------
`include "./es7632bt_fanpld_defined.vh"
//---------------------------------------------------------------------
module regfile (
    input           MGMT_CLK,
    input           MGMT_RST_N,
    input           I2C_READ,
    input           I2C_READ_DONE,
    input           I2C_WRITE,
    input  [7:0]    I2C_OFFSET_ADDR,
    input  [7:0]    I2C_OFFSER_WDATA,
    output [7:0]    I2C_OFFSER_RDATA,
    input  [5:0]    FAN_DIR_CTRL,
    output [3:0]    FAN_PWM_OUT,
    output [5:0]    FAN_LED_R,
    output [5:0]    FAN_LED_G,
    output          MASK_I2C_LM75,
    output          FAN_INT,
    output          RESET_OUT,
    output [5:0]    FAN_ENB,
    input  [4:0]    HW_BOARD_ID,
    input           FAN_PWM_UPDATA,
    input  [3:0]    FAN_PWM_IN,
    input  [7:0]    FRONT_FAN1_TACH,
    input  [7:0]    FRONT_FAN2_TACH,
    input  [7:0]    FRONT_FAN3_TACH,
    input  [7:0]    FRONT_FAN4_TACH,
    input  [7:0]    FRONT_FAN5_TACH,
    input  [7:0]    FRONT_FAN6_TACH,
    input  [7:0]    REAR_FAN1_TACH,
    input  [7:0]    REAR_FAN2_TACH,
    input  [7:0]    REAR_FAN3_TACH,
    input  [7:0]    REAR_FAN4_TACH,
    input  [7:0]    REAR_FAN5_TACH,
    input  [7:0]    REAR_FAN6_TACH,
    input           WD_PWM_CTRL,
    input  [7:0]    REG_WD_TIMER_8B,
    input  [3:0]    REG_WD_PWM_MAX_VALUE_4B,
    input           REG_WD_ENABLE_1B
    );

    // Signals
    reg    [7:0]    r_i2c_readdata;

    // Signals for Offset Registers

    reg             r_cpld_reset_1b = 1'b1;
    reg    [3:0]    r_fan_pwm_cpu_4b = 4'h5;
    reg    [7:0]    r_fan_led1_ctrl_8b;
    reg    [3:0]    r_fan_led2_ctrl_4b;
    reg             r_mask_i2c_master;

    // Signals for FAN Control
    reg             r_fan_int_1b_n;
    reg             fan_int_sig;
    reg             fan_int_mask_sig;
    reg             fan_pwm_updata_ff;
    reg             wd_pwm_ctrl_ff;
    reg    [3:0]    wd_pwm_value_4b;
    reg    [3:0]    r_fan_pwm_cpld_4b;
    reg    [7:0]    r_front_fan1_tach_8b;
    reg    [7:0]    r_front_fan2_tach_8b;
    reg    [7:0]    r_front_fan3_tach_8b;
    reg    [7:0]    r_front_fan4_tach_8b;
    reg    [7:0]    r_front_fan5_tach_8b;
    reg    [7:0]    r_front_fan6_tach_8b;
    reg    [7:0]    r_rear_fan1_tach_8b;
    reg    [7:0]    r_rear_fan2_tach_8b;
    reg    [7:0]    r_rear_fan3_tach_8b;
    reg    [7:0]    r_rear_fan4_tach_8b;
    reg    [7:0]    r_rear_fan5_tach_8b;
    reg    [7:0]    r_rear_fan6_tach_8b;
    reg    [5:0]    r_fan_prsnt_6b;
    reg    [5:0]    r_fan_pwr_ctrl_6b = 6'h3F;
    reg             r_led_debug;     //Vincent ada 20150810

    // Clock Signals
    wire clock   = MGMT_CLK;
    wire reset_n = MGMT_RST_N;

    // Buffers
    assign  FAN_PWM_OUT  = r_fan_pwm_cpu_4b;

    assign  FAN_LED_R = {r_fan_led2_ctrl_4b[1], r_fan_led2_ctrl_4b[3],
                         r_fan_led1_ctrl_8b[1], r_fan_led1_ctrl_8b[3],
                         r_fan_led1_ctrl_8b[5], r_fan_led1_ctrl_8b[7]};

    assign  FAN_LED_G = {r_fan_led2_ctrl_4b[0], r_fan_led2_ctrl_4b[2],
                         r_fan_led1_ctrl_8b[0], r_fan_led1_ctrl_8b[2],
                         r_fan_led1_ctrl_8b[4], r_fan_led1_ctrl_8b[6]};

    assign  MASK_I2C_LM75 = r_mask_i2c_master;

    assign  FAN_INT = r_fan_int_1b_n;

    assign  RESET_OUT = r_cpld_reset_1b;

    assign  FAN_ENB = r_fan_pwr_ctrl_6b;

    // Inferred the Filop Flip
    always @(posedge clock) begin
        fan_pwm_updata_ff    <= FAN_PWM_UPDATA;
        wd_pwm_ctrl_ff       <= WD_PWM_CTRL;
        wd_pwm_value_4b      <= REG_WD_PWM_MAX_VALUE_4B;
        r_fan_pwm_cpld_4b    <= FAN_PWM_IN;
        r_front_fan1_tach_8b <= FRONT_FAN1_TACH;
        r_front_fan2_tach_8b <= FRONT_FAN2_TACH;
        r_front_fan3_tach_8b <= FRONT_FAN3_TACH;
        r_front_fan4_tach_8b <= FRONT_FAN4_TACH;
        r_front_fan5_tach_8b <= FRONT_FAN5_TACH;
        r_front_fan6_tach_8b <= FRONT_FAN6_TACH;
        r_rear_fan1_tach_8b  <= REAR_FAN1_TACH;
        r_rear_fan2_tach_8b  <= REAR_FAN2_TACH;
        r_rear_fan3_tach_8b  <= REAR_FAN3_TACH;
        r_rear_fan4_tach_8b  <= REAR_FAN4_TACH;
        r_rear_fan5_tach_8b  <= REAR_FAN5_TACH;
        r_rear_fan6_tach_8b  <= REAR_FAN6_TACH;
    end

    // Fan Interrupt
    always @(posedge clock) begin
        if (r_front_fan1_tach_8b == 8'h00 ||
            r_front_fan2_tach_8b == 8'h00 ||
            r_front_fan3_tach_8b == 8'h00 ||
            r_front_fan4_tach_8b == 8'h00 ||
            r_front_fan5_tach_8b == 8'h00 ||
            r_front_fan6_tach_8b == 8'h00 ||
            r_rear_fan1_tach_8b  == 8'h00 ||
            r_rear_fan2_tach_8b  == 8'h00 ||
            r_rear_fan3_tach_8b  == 8'h00 ||
            r_rear_fan4_tach_8b  == 8'h00 ||
            r_rear_fan5_tach_8b  == 8'h00 ||
            r_rear_fan6_tach_8b  == 8'h00 )

            fan_int_sig = 1'b1;
        else
            fan_int_sig = 1'b0;
    end

    always @(posedge clock) begin
        if (reset_n == 1'b0) begin
            r_fan_int_1b_n <= 1'b1;
        end else begin
            r_fan_int_1b_n <= (fan_int_mask_sig == 1'b0) ? fan_int_sig : 1'b1;
        end
    end

    // FAN Module Present
    always @(posedge clock) begin
        if (reset_n == 1'b0) begin
           r_fan_prsnt_6b[0] <= 1'b1;
        end else begin
            if ( r_front_fan1_tach_8b != 8'h00 || r_rear_fan1_tach_8b != 8'h00) begin
                r_fan_prsnt_6b[0] <= 1'b0;
            end else begin
                r_fan_prsnt_6b[0] <= 1'b1;
            end
        end
    end
    always @(posedge clock) begin
        if (reset_n == 1'b0) begin
           r_fan_prsnt_6b[1] <= 1'b1;
        end else begin
            if ( r_front_fan2_tach_8b != 8'h00 || r_rear_fan2_tach_8b != 8'h00) begin
                r_fan_prsnt_6b[1] <= 1'b0;
            end else begin
                r_fan_prsnt_6b[1] <= 1'b1;
            end
        end
    end
    always @(posedge clock) begin
        if (reset_n == 1'b0) begin
           r_fan_prsnt_6b[2] <= 1'b1;
        end else begin
            if ( r_front_fan3_tach_8b != 8'h00 || r_rear_fan3_tach_8b != 8'h00) begin
                r_fan_prsnt_6b[2] <= 1'b0;
            end else begin
                r_fan_prsnt_6b[2] <= 1'b1;
            end
        end
    end
    always @(posedge clock) begin
        if (reset_n == 1'b0) begin
           r_fan_prsnt_6b[3] <= 1'b1;
        end else begin
            if ( r_front_fan4_tach_8b != 8'h00 || r_rear_fan4_tach_8b != 8'h00) begin
                r_fan_prsnt_6b[3] <= 1'b0;
            end else begin
                r_fan_prsnt_6b[3] <= 1'b1;
            end
        end
    end
    always @(posedge clock) begin
        if (reset_n == 1'b0) begin
           r_fan_prsnt_6b[4] <= 1'b1;
        end else begin
            if ( r_front_fan5_tach_8b != 8'h00 || r_rear_fan5_tach_8b != 8'h00) begin
                r_fan_prsnt_6b[4] <= 1'b0;
            end else begin
                r_fan_prsnt_6b[4] <= 1'b1;
            end
        end
    end
    always @(posedge clock) begin
        if (reset_n == 1'b0) begin
           r_fan_prsnt_6b[5] <= 1'b1;
        end else begin
            if ( r_front_fan6_tach_8b != 8'h00 || r_rear_fan6_tach_8b != 8'h00) begin
                r_fan_prsnt_6b[5] <= 1'b0;
            end else begin
                r_fan_prsnt_6b[5] <= 1'b1;
            end
        end
    end


    //-----------------------------------------------------------------
    // Write Operation Statement by I2C Host
    //-----------------------------------------------------------------
    always @(posedge clock) begin
       if (I2C_WRITE == 1'b1 &&
           I2C_OFFSET_ADDR == `FANPLD_REG_CPLD_RST) begin
            r_cpld_reset_1b <= I2C_OFFSER_WDATA[7];
       end
    end

    always @(posedge clock, negedge reset_n) begin
        if (reset_n == 1'b0 ) begin
            fan_int_mask_sig <= 1'b0;
        end else begin
            if (I2C_WRITE == 1'b1 &&
                I2C_OFFSET_ADDR == `FANPLD_REG_INT_MASK) begin
               fan_int_mask_sig <= I2C_OFFSER_WDATA[7];
            end
        end
    end


    always @(posedge clock) begin
        if (reset_n == 1'b0 ) begin
            r_fan_pwm_cpu_4b <= 4'h4;
        end else begin
            if (wd_pwm_ctrl_ff == 1'b1) begin
               r_fan_pwm_cpu_4b <= wd_pwm_value_4b;
            end else if (fan_pwm_updata_ff == 1'b1) begin
               r_fan_pwm_cpu_4b <= r_fan_pwm_cpld_4b;
            end else if (I2C_WRITE == 1'b1 &&
                         I2C_OFFSET_ADDR == `FANPLD_REG_FAN_PWM) begin
               r_fan_pwm_cpu_4b <= I2C_OFFSER_WDATA[3:0];
            end
        end
    end

    always @(posedge clock)      //Vincent modify 20150810
    begin
       if (reset_n == 1'b0 )
          r_fan_led1_ctrl_8b[7:6] <= 2'h0;
	   else if (r_led_debug)
       begin
         if (I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `FANPLD_REG_FAN_LED1_DISPLAY)
            r_fan_led1_ctrl_8b[7:6] <= I2C_OFFSER_WDATA[7:6];
         else
            r_fan_led1_ctrl_8b[7:6] <= r_fan_led1_ctrl_8b[7:6];
       end
       else
       begin
         if (I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `FANPLD_REG_FAN_LED1_DISPLAY)
            r_fan_led1_ctrl_8b[7:6] <= I2C_OFFSER_WDATA[7:6];
         else if (r_front_fan1_tach_8b != 8'h00 || r_rear_fan1_tach_8b != 8'h00)
         begin
            r_fan_led1_ctrl_8b[7] <= 1'b0; // Lighting Green LED
			r_fan_led1_ctrl_8b[6] <= 1'b1;
         end
         else
            r_fan_led1_ctrl_8b[7:6] <= 2'h2; // Lighting Red LED
       end
    end

    always @(posedge clock)      //Vincent modify 20150810
    begin
       if (reset_n == 1'b0 )
          r_fan_led1_ctrl_8b[5:4] <= 2'h0;
	   else if (r_led_debug)
       begin
         if (I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `FANPLD_REG_FAN_LED1_DISPLAY)
            r_fan_led1_ctrl_8b[5:4] <= I2C_OFFSER_WDATA[5:4];
         else
            r_fan_led1_ctrl_8b[5:4] <= r_fan_led1_ctrl_8b[5:4];
       end
       else
       begin
         if (I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `FANPLD_REG_FAN_LED1_DISPLAY)
            r_fan_led1_ctrl_8b[5:4] <= I2C_OFFSER_WDATA[5:4];
         else if (r_front_fan2_tach_8b != 8'h00 || r_rear_fan2_tach_8b != 8'h00)
         begin
            r_fan_led1_ctrl_8b[5] <= 1'b0; // Lighting Green LED
			r_fan_led1_ctrl_8b[4] <= 1'b1;
         end
         else
            r_fan_led1_ctrl_8b[5:4] <= 2'h2; // Lighting Red LED
       end
    end

    always @(posedge clock)      //Vincent modify 20150810
    begin
       if (reset_n == 1'b0 )
          r_fan_led1_ctrl_8b[3:2] <= 2'h0;
	   else if (r_led_debug)
       begin
         if (I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `FANPLD_REG_FAN_LED1_DISPLAY)
            r_fan_led1_ctrl_8b[3:2] <= I2C_OFFSER_WDATA[3:2];
         else
            r_fan_led1_ctrl_8b[3:2] <= r_fan_led1_ctrl_8b[3:2];
       end
       else
       begin
         if (I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `FANPLD_REG_FAN_LED1_DISPLAY)
            r_fan_led1_ctrl_8b[3:2] <= I2C_OFFSER_WDATA[3:2];
         else if (r_front_fan3_tach_8b != 8'h00 || r_rear_fan3_tach_8b != 8'h00)
         begin
            r_fan_led1_ctrl_8b[3] <= 1'b0; // Lighting Green LED
			r_fan_led1_ctrl_8b[2] <= 1'b1;
         end
         else
            r_fan_led1_ctrl_8b[3:2] <= 2'h2; // Lighting Red LED
       end
    end

    always @(posedge clock)      //Vincent modify 20150810
    begin
       if (reset_n == 1'b0 )
          r_fan_led1_ctrl_8b[1:0] <= 2'h0;
	   else if (r_led_debug)
       begin
         if (I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `FANPLD_REG_FAN_LED1_DISPLAY)
            r_fan_led1_ctrl_8b[1:0] <= I2C_OFFSER_WDATA[1:0];
         else
            r_fan_led1_ctrl_8b[1:0] <= r_fan_led1_ctrl_8b[1:0];
       end
       else
       begin
         if (I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `FANPLD_REG_FAN_LED1_DISPLAY)
            r_fan_led1_ctrl_8b[1:0] <= I2C_OFFSER_WDATA[1:0];
         else if (r_front_fan4_tach_8b != 8'h00 || r_rear_fan4_tach_8b != 8'h00)
         begin
            r_fan_led1_ctrl_8b[1] <= 1'b0; // Lighting Green LED
			r_fan_led1_ctrl_8b[0] <= 1'b1;
         end
         else
            r_fan_led1_ctrl_8b[1:0] <= 2'h2; // Lighting Red LED
       end
    end

    always @(posedge clock)      //Vincent modify 20150810
    begin
       if (reset_n == 1'b0 )
          r_fan_led2_ctrl_4b[3:2] <= 2'h0;
	   else if (r_led_debug)
       begin
         if (I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `FANPLD_REG_FAN_LED2_DISPLAY)
            r_fan_led2_ctrl_4b[3:2] <= I2C_OFFSER_WDATA[3:2];
         else
            r_fan_led2_ctrl_4b[3:2] <= r_fan_led2_ctrl_4b[3:2];
       end
       else
       begin
         if (I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `FANPLD_REG_FAN_LED2_DISPLAY)
            r_fan_led2_ctrl_4b[3:2] <= I2C_OFFSER_WDATA[3:2];
         else if (r_front_fan5_tach_8b != 8'h00 || r_rear_fan5_tach_8b != 8'h00)
         begin
            r_fan_led2_ctrl_4b[3] <= 1'b0; // Lighting Green LED
			r_fan_led2_ctrl_4b[2] <= 1'b1;
         end
         else
            r_fan_led2_ctrl_4b[3:2] <= 2'h2; // Lighting Red LED
       end
    end

    always @(posedge clock)      //Vincent modify 20150810
    begin
       if (reset_n == 1'b0 )
          r_fan_led2_ctrl_4b[1:0] <= 2'h0;
	   else if (r_led_debug)
       begin
         if (I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `FANPLD_REG_FAN_LED2_DISPLAY)
            r_fan_led2_ctrl_4b[1:0] <= I2C_OFFSER_WDATA[1:0];
         else
            r_fan_led2_ctrl_4b[1:0] <= r_fan_led2_ctrl_4b[1:0];
       end
       else
       begin
         if (I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `FANPLD_REG_FAN_LED2_DISPLAY)
            r_fan_led2_ctrl_4b[1:0] <= I2C_OFFSER_WDATA[1:0];
         else if (r_front_fan6_tach_8b != 8'h00 || r_rear_fan6_tach_8b != 8'h00)
         begin
            r_fan_led2_ctrl_4b[1] <= 1'b0; // Lighting Green LED
			r_fan_led2_ctrl_4b[0] <= 1'b1;
         end
         else
            r_fan_led2_ctrl_4b[1:0] <= 2'h2; // Lighting Red LED
       end
    end

    always @(posedge clock) begin
       if (reset_n == 1'b0 ) begin
            r_mask_i2c_master <= 1'b0;
       end else if (I2C_WRITE == 1'b1 &&
                    I2C_OFFSET_ADDR == `FANPLD_REG_FAN_I2C_ENABLE) begin
            r_mask_i2c_master <= I2C_OFFSER_WDATA[0];
       end
    end

    always @(posedge clock) begin
       if (reset_n == 1'b0 ) begin
            r_fan_pwr_ctrl_6b <= 6'h3F;
       end else if (I2C_WRITE == 1'b1 &&
                    I2C_OFFSET_ADDR == `FANPLD_REG_FAN_POWER_CTRL) begin
            r_fan_pwr_ctrl_6b <= I2C_OFFSER_WDATA[5:0];
       end
    end

    always @(posedge clock) begin      //Vincent ada 20150810
       if (reset_n == 1'b0 ) begin
            r_led_debug <= 1'b0;
       end else if (I2C_WRITE == 1'b1 &&
                    I2C_OFFSET_ADDR == `FANPLD_REG_DEBUF_MODE) begin
            r_led_debug <= I2C_OFFSER_WDATA[7];
       end
    end

    //----------------------------------------------------------------------
    // Read Operation Statement by I2C Host
    //----------------------------------------------------------------------
    assign I2C_OFFSER_RDATA = r_i2c_readdata;

    always @(posedge clock) begin
       if (reset_n == 1'b0) begin
          r_i2c_readdata <= 8'h00;
       end else if (I2C_READ == 1'b1)begin
            case (I2C_OFFSET_ADDR)
                `FANPLD_REG_BOARD_INFO           : r_i2c_readdata <= {HW_BOARD_ID[4:2], 3'h0, HW_BOARD_ID[1:0]};
                `FANPLD_REG_CPLD_VER             : r_i2c_readdata <= `PLD_ID;
                `FANPLD_REG_CPLD_RST             : r_i2c_readdata <= {r_cpld_reset_1b, 7'h00};
                `FANPLD_REG_INT_STATUS           : r_i2c_readdata <= {r_fan_int_1b_n, 7'h00};
                `FANPLD_REG_INT_MASK             : r_i2c_readdata <= {fan_int_mask_sig, 7'h00};
                `FANPLD_REG_FAN_PRSNT            : r_i2c_readdata <= {2'h0, r_fan_prsnt_6b};
                `FANPLD_REG_FAN_DIR              : r_i2c_readdata <= {2'h0, FAN_DIR_CTRL};
                `FANPLD_REG_FAN_PWM              : r_i2c_readdata <= {4'h0, r_fan_pwm_cpu_4b};
                `FANPLD_REG_FRONT_FAN1_TACH      : r_i2c_readdata <= r_front_fan1_tach_8b;
                `FANPLD_REG_FRONT_FAN2_TACH      : r_i2c_readdata <= r_front_fan2_tach_8b;
                `FANPLD_REG_FRONT_FAN3_TACH      : r_i2c_readdata <= r_front_fan3_tach_8b;
                `FANPLD_REG_FRONT_FAN4_TACH      : r_i2c_readdata <= r_front_fan4_tach_8b;
                `FANPLD_REG_FRONT_FAN5_TACH      : r_i2c_readdata <= r_front_fan5_tach_8b;
                `FANPLD_REG_FRONT_FAN6_TACH      : r_i2c_readdata <= r_front_fan6_tach_8b;
                `FANPLD_REG_FAN_LED1_DISPLAY     : r_i2c_readdata <= r_fan_led1_ctrl_8b;
                `FANPLD_REG_FAN_LED2_DISPLAY     : r_i2c_readdata <= {4'h0, r_fan_led2_ctrl_4b};
                `FANPLD_REG_FAN_I2C_ENABLE       : r_i2c_readdata <= {7'h00, r_mask_i2c_master};
                `FANPLD_REG_REAR_FAN1_TACH       : r_i2c_readdata <= r_rear_fan1_tach_8b;
                `FANPLD_REG_REAR_FAN2_TACH       : r_i2c_readdata <= r_rear_fan2_tach_8b;
                `FANPLD_REG_REAR_FAN3_TACH       : r_i2c_readdata <= r_rear_fan3_tach_8b;
                `FANPLD_REG_REAR_FAN4_TACH       : r_i2c_readdata <= r_rear_fan4_tach_8b;
                `FANPLD_REG_REAR_FAN5_TACH       : r_i2c_readdata <= r_rear_fan5_tach_8b;
                `FANPLD_REG_REAR_FAN6_TACH       : r_i2c_readdata <= r_rear_fan6_tach_8b;
                `FANPLD_REG_FAN_POWER_CTRL       : r_i2c_readdata <= {2'h0, r_fan_pwr_ctrl_6b};
                `FANPLD_REG_FAN_WD_TIMER         : r_i2c_readdata <= REG_WD_TIMER_8B;
                `FANPLD_REG_FAN_WD_PWM_CTRL      : r_i2c_readdata <= {4'h0, REG_WD_PWM_MAX_VALUE_4B};
                `FANPLD_REG_FAN_WD_DISABLE       : r_i2c_readdata <= {7'h00, REG_WD_ENABLE_1B};
                `FANPLD_REG_DEBUF_MODE           : r_i2c_readdata <= {r_led_debug, 7'h00};      //Vincent ada 20150810
                default                          : r_i2c_readdata <= 8'hFF;
            endcase
       end
    end


endmodule
